/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CyrWrapLib
// @module  Linked Items | This module contains the implementation of the
//          CECOrbLink class.  This class allows the linking of certain OpenSTARep
//          objects.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <CyrWrapLib/CECOrbLink.h>

///////////////////////////////////////////////////////////////////////////////
// @mfunc   The constructor does nothing but pass the paramters on to its base
//          constructor.
// @parm    Pointer to a CORBA helper class.
// @parm    Reference to a Factory.
CECOrbLink::CECOrbLink( CORBAServer* corba, CyrFactory_ptr factory )
        : CECOrbBorrow( corba, factory )
{}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns a copy of the OpenSTARep CORBA Server interface.
//          This will increment the reference count and so will need to be
//          released.
// @rdesc   Reference to the Server object.

CyrStServer_ptr CECOrbLink::GetHiddenServer()
{
    return CyrStServer::_duplicate( m_hidden );
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns a '\|' separated string that describes the item
//          in the derived class.  The first entry is the root folder and also
//          the project name.  The last entry is the name of the item (either
//          a file name, a folder name or a test run name).  All entries in
//          between represent folders.
// @rdesc   The complete link path.
string CECOrbLink::GetLinkPath()
{
    // Get underlying link object
    CyrStLink_var parent = GetLinkServant();
    CORBA::String_var str = parent->GetLinkPath();
    string ret = str.in();
    return ret;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns the link path of the first item linked to this
//          one. c.f. <mf CECOrbLink::GetLinkPath> for details.
// @rdesc   The first linked item's path .
// @xref    <mf CECOrbLink::GetLinkPath>, <mf CECOrbLink::GetNextLinkPath>
string CECOrbLink::GetFirstLinkPath()
{
    // Get underlying link object
    CyrStLink_var parent = GetLinkServant();
    CORBA::String_var str = parent->GetFirstLinkPath();
    string ret = str.in();
    return ret;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns the next linked item's path.
// @rdesc   The complete link path.
// @xref    <mf CECOrbLink::GetLinkPath>, <mf CECOrbLink::GetFirstLinkPath>
string CECOrbLink::GetNextLinkPath()
{
    // Get underlying link object
    CyrStLink_var parent = GetLinkServant();
    CORBA::String_var str = parent->GetNextLinkPath();
    string ret = str.in();
    return ret;
}

