/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef __EXCEPTION__
#define __EXCEPTION__

#ifndef __DEFINITION__
#  include "definition.h"
#endif

#include <fstream>
using std::ostream;
//using namespace std;

#ifndef __MTSAFE__
#  include <mtsafe.hpp>
#endif  // __MTSAFE__

#ifdef  WIN32
#  pragma warning(disable : 4512 4907)
#endif  // WIN32

#include <exception>
#include <string>
using std::string;
using std::exception;

#ifdef  WIN32
#  pragma warning(default : 4512 4907)
#endif  // WIN32



/*+-------------------------------------------------------------------
  | CLASS           : CTException
  +-------------------------------------------------------------------
  | DESCRIPTION     : blase  exception  class
  +-------------------------------------------------------------------
*/
class CTOLogFileList;

class DLLSPEC_CYRTOOLS CTException : virtual public CTMTSafe, public exception
{
public:
    CTException(const string& msg, const string &filename, unsigned int sLineno, long lNumber = -1L);
    CTException(const CTException& exception);
    void operator= (const CTException& exception);
    virtual ~CTException();
    virtual const char* what() const;

    const string getMsg() const
    {
        return ctmsg_;
    }
    const string getFullMsg() const
    {
        return fullMsg_;
    }
    const string getFilename() const
    {
        return filename_;
    }
    unsigned int getLineno() const
    {
        return sLineno_;
    }
    long  getNumber() const
    {
        return lNumber_;
    }

    //---------------------------------------------------------------------------

    static void Throw (const CTException &e) ;

private:
    string ctmsg_;
    string filename_;
    unsigned int sLineno_;
    long lNumber_;
    string fullMsg_ ;
};


DLLSPEC_CYRTOOLS ostream& operator<< (ostream& os, const CTException& exception);
DLLSPEC_CYRTOOLS CTOLogFileList& operator<< (CTOLogFileList& os, const CTException& exception);

#endif // __EXCEPTION__





