/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//==================================================================< Pragmas >
#ifdef  WIN32
#  pragma warning(disable : 4244)
#endif  // WIN32

//#include <iostream.h>
//=================================================================< Includes >
#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifdef UNIX
#include <stdlib.h>
#include <unistd.h>
#endif

#ifdef  WIN32
#  ifndef   _WINDOWS_
#    define WIN32_LEAN_AND_MEAN
#    define NOGDICAPMASKS
#    define NOVIRTUALKEYCODES
#    define NOWINMESSAGES
#    define NOWINSTYLES
#    define NOSYSMETRICS
#    define NOMENUS
#    define NOICONS
#    define NOKEYSTATES
#    define NOSYSCOMMANDS
#    define NORASTEROPS
#    define NOSHOWWINDOW
#    define OEMRESOURCE
#    define NOATOM
#    define NOCLIPBOARD
#    define NOCOLOR
#    define NOCTLMGR
#    define NODRAWTEXT
#    define NOGDI
#    define NOKERNEL
//#    define   NOUSER
#    define NONLS
#    define NOMB
#    define NOMEMMGR
#    define NOMETAFILE
#    define NOMINMAX
#    define NOMSG
#    define NOOPENFILE
#    define NOSCROLL
#    define NOSERVICE
#    define NOSOUND
#    define NOTEXTMETRIC
#    define NOWH
#    define NOWINOFFSETS
#    define NOCOMM
#    define NOKANJI
#    define NOHELP
#    define NOPROFILER
#    define NODEFERWINDOWPOS
#    define NOMCX
#    include <windows.h>
#  endif    // _WINDOWS_
#endif  // WIN32

#ifdef DEBUG
#include <iostream>
#define DEBUG_OUT(x) x
using std::cerr;
using std::cout;
using std::endl;
#else
#define DEBUG_OUT(x) // x
#endif

#ifdef  HPUX
#  include <errno.h>
#  include <dl.h>
#endif  // HPUX

#if defined (DGUX) || defined (AIX) || defined(SUN) || defined(ALPHA) || defined(LINUX)
#ifdef DEBUG
#include <string.h>
#endif
#  include <errno.h>
#  include <dlfcn.h>
#endif  // DGUX || AIX || SUN

#ifdef UNIX
# ifdef AIX
#  define LIBPATH "LIBPATH"
# elif defined(HPUX)
#  define LIBPATH "SHLIB_PATH"
# else
#  define LIBPATH "LD_LIBRARY_PATH"
# endif
#endif

#ifndef __LIBRARY__
#  include "library.hpp"
#endif  // __LIBRARY__

#ifndef __XALLOC__
#  include "xalloc.hpp"
#endif  // __XALLOC__

//======================================================< Definition of class >

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////// Class CTLibraryInfo //
///////////////////////////////////////////////////////////////////////////////
// Class used to load a library.
class CTLibraryInfo : virtual public CTMTSafe
{
    //-----------------------------------------------------------------------------
public:                 // Constructor(s) and destructor.
    CTLibraryInfo (const string& name) ;
    CTLibraryInfo (const CTLibraryInfo& libraryInfo) ;
    ~CTLibraryInfo () ;

public:                 // Members.
#ifdef  WIN32
    HINSTANCE           hLibrary_ ;
#endif  // WIN32
#if defined (DGUX) || defined (AIX) || defined(SUN) || defined(ALPHA) || defined(LINUX)
    void*               hLibrary_ ;
#endif  // DGUX || AIX
#ifdef  HPUX
    shl_t               hLibrary_ ;
#endif  // HPUX


    string          name_ ;

    //-----------------------------------------------------------------------------
}

; // CTLibraryInfo

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////// End of Class CTLibraryInfo //
///////////////////////////////////////////////////////////////////////////////

//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////// Class CTLibraryInfo //
///////////////////////////////////////////////////////////////////////////////
// Class used to load a library.

//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CTLibraryInfo::CTLibraryInfo
//
// Constructor.
//
// Arguments : name : name of the library to load.
// Return    : none.
//-----------------------------------------------------------------------------
CTLibraryInfo::CTLibraryInfo (const string& name)
        : hLibrary_ (NULL), name_ (CTLibrary::getFullPath (CTLibrary::getSystemFileName (name,CTLibrary::LIB_TYPE) ))
{
    DEBUG_OUT(std::cerr<<"trying to load : '" << name_ << "'."<<endl;)
#ifdef  WIN32
    // Add ".DLL" if necessary.
    char DLLext[5];
    string::size_type sz=name_.length();
    for(int i=0;i<=4;i++)
        DLLext[i]=toupper(name_[sz-4+i]);
    DLLext[4]='\0';
    DEBUG_OUT(std::cerr<<"Uppercased : '" << DLLext << "'."<<endl;)
    if (strcmp(".DLL",DLLext))
    {
        name_ += ".DLL" ;
    }
    // Load the library.
    DEBUG_OUT(std::cerr<<"trying to load : '" << name_ << "'."<<endl;)
    hLibrary_ = LoadLibrary ( (LPCTSTR) name_.c_str() ) ;
    if (hLibrary_ == NULL)
    {   // Error from LoadLibrary ().
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        CharToOem (zBuf, zBuf) ;
        throw CTLibraryException (string("CTLibraryInfo: LoadLibrary failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
    }
#endif  // WIN32

#if  defined (DGUX) || defined (SUN) || defined(ALPHA) || defined(LINUX)
    // Add "lib" and ".so" if necessary.
    int iLastSlash = name_.find_last_of('/') ;
    if (name_.substr(iLastSlash+1, 3) != "lib")
    {
	name_.insert(iLastSlash+1, "lib") ;
    }
    if (name_.substr(name_.size() - 3, 3) != ".so" )
    {
        name_ += ".so" ;
    }

#ifdef ALPHA
    hLibrary_ = dlopen (name_.c_str(), RTLD_NOW) ;
#else
    hLibrary_ = dlopen (name_.c_str(), RTLD_NOW | RTLD_GLOBAL) ;
#endif
    // Load the library.
    if (hLibrary_ == NULL)
    {   // Error from dlopen ().
	string err = dlerror();
        DEBUG_OUT(std::cerr<<"### loading error  : '" << err << "'."<<endl;)
        throw CTLibraryException (string ("CTLibraryInfo: dlopen failed : '")+ name_ +string("'. ") + err, __FILE__, __LINE__, errno) ;
    }

#endif  // DGUX || SUN


#if  defined (AIX)
    // Add "lib" and ".a" if necessary.
    if (string (name_(name_.last ('/') + 1, 3) ).compareTo ("lib") )
    {
        name_(name_.last ('/') + 1, 0) = "lib" ;
    }
    if (string (name_(name_.length () - 2, 2) ).compareTo (".a") )
    {
        name_ += ".a" ;
    }
    hLibrary_ = dlopen (name_.c_str(), RTLD_NOW | RTLD_GLOBAL) ;
    // Load the library.
    if (hLibrary_ == NULL)
    {   // Error from dlopen ().
        throw CTLibraryException (string ("CTLibraryInfo: dlopen failed")+ name_ +string("'. "), __FILE__, __LINE__, errno) ;
    }
#endif  // AIX


#ifdef  HPUX
    // Add "lib" and ".sl" if necessary.
    if (string (name_(name_.last ('/') + 1, 3) ).compareTo ("lib") )
    {
        name_(name_.last ('/') + 1, 0) = "lib" ;
    }
    if (string (name_(name_.length () - 3, 3) ).compareTo (".sl") )
    {
        name_ += ".sl" ;
    }
    // Load the library.
    hLibrary_ = shl_load (name_.c_str(), BIND_IMMEDIATE | BIND_VERBOSE, 0L) ;
    if (hLibrary_ == NULL)
    {   // Error from shl_load ().
#ifdef DEBUG
        perror("###### shl_load");
#endif
        throw CTLibraryException (string ("CTLibraryInfo: shl_load failed: ") + name_ ,  __FILE__, __LINE__, errno) ;
    }
#endif  // HPUX


    DEBUG_OUT(cerr<<"Chargement de la library : \'" << name_ << "\' OK." << endl);

} // CTLibraryInfo::CTLibraryInfo

//-----------------------------------------------------------------------------
// CTLibraryInfo::CTLibraryInfo
//
// Copy constructor.
//
// Arguments : libraryInfo : library to copy.
// Return    : none.
//-----------------------------------------------------------------------------
CTLibraryInfo::CTLibraryInfo (const CTLibraryInfo& libraryInfo)
        : hLibrary_ (NULL), name_ (libraryInfo.name_)
{

#ifdef  WIN32
    // Load the library.
    hLibrary_ = LoadLibrary ( (LPCTSTR) name_.c_str() ) ;
    if (hLibrary_ == NULL)
    {   // Error from LoadLibrary ().
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        CharToOem (zBuf, zBuf) ;
        throw (CTLibraryException (string ("CTLibraryInfo: LoadLibrary failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ) ;
    }
#endif  // WIN32
#if defined (DGUX) || defined (AIX) || defined(SUN) || defined(ALPHA) || defined(LINUX)
    // Load the library.
#ifndef ALPHA
    hLibrary_ = dlopen (name_.c_str(), RTLD_NOW | RTLD_GLOBAL) ;
#else
hLibrary_ = dlopen (name_.c_str(), RTLD_NOW ) ;
#endif
    if (hLibrary_ == NULL)
    {   // Error from dlopen ().
        throw (CTLibraryException (string ("CTLibraryInfo: dlopen failed: ") + strerror (errno), __FILE__, __LINE__, errno) ) ;
    }
#endif  // DGUX || AIX || SUN
#ifdef  HPUX
    // Load the library.
    hLibrary_ = shl_load (name_.c_str(), BIND_IMMEDIATE, 0L) ;
    if (hLibrary_ == NULL)
    {   // Error from shl_load ().
        throw (CTLibraryException (string ("CTLibraryInfo: shl_load failed: ") + strerror (errno), __FILE__, __LINE__, errno) ) ;
    }
#endif  // HPUX
} // CTLibraryInfo::CTLibraryInfo

//-----------------------------------------------------------------------------
// CTLibraryInfo::~CTLibraryInfo
//
// Destructor.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------

CTLibraryInfo::~CTLibraryInfo ()
{
    if (hLibrary_)
    {
#ifdef  WIN32
        // Unload the library.
        if (!FreeLibrary (hLibrary_) )
        {   // Error from FreeLibrary ().
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            CTException::Throw (CTLibraryException (string("~CTLibraryInfo: FreeLibrary failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ) ;
        }
#endif  // WIN32
#if defined (DGUX) || defined (AIX) || defined(SUN)  || defined(ALPHA) || defined(LINUX)
        // Unload the library.
        if (dlclose (hLibrary_) != 0)
        {   // Error from dlclose ().
            throw (CTLibraryException (string ("~CTLibraryInfo: dlclose failed: ") + strerror (errno), __FILE__, __LINE__, errno) ) ;
        }
#endif  // DGUX || AIX || SUN
#ifdef  HPUX
        // Unload the library.
        if (shl_unload (hLibrary_) == -1)
        {   // Error from shl_unload ().
            throw (CTLibraryException (string ("~CTLibraryInfo: shl_unload failed: ") + strerror (errno), __FILE__, __LINE__, errno) ) ;
        }
#endif  // HPUX
        hLibrary_ = NULL ;
    }
} // CTLibraryInfo::~CTLibraryInfo

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////// End of Class CTLibraryInfo //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////// Class CTLibrary //
///////////////////////////////////////////////////////////////////////////////
// Class used to load a library.

//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CTLibrary::CTLibrary
//
// Constructor.
//
// Arguments : name : name of the library to load.
// Return    : none.
//-----------------------------------------------------------------------------
CTLibrary::CTLibrary (const string & name)
        : pLibraryInfo_ (NULL)
{
    // Create the library object.
    try
    {
        pLibraryInfo_ = (void*) new CTLibraryInfo (name) ;
    }
    catch (CTxalloc&)
    {
        CTException::Throw (CTLibraryException ("CTLibrary: new CTLibraryInfo (name) failed.", __FILE__, __LINE__) ) ;
    }
} // CTLibrary::CTLibrary

//-----------------------------------------------------------------------------
// CTLibrary::CTLibrary
//
// Copy constructor.
//
// Arguments : library : instance to copy.
// Return    : none.
//-----------------------------------------------------------------------------
CTLibrary::CTLibrary (const CTLibrary& library)
        : pLibraryInfo_ (NULL)
{
    // Verify the library object is correct.
    if (library.pLibraryInfo_ == NULL)
    {   // Error from library.pLibraryInfo_.
        CTException::Throw (CTLibraryException ("CTLibrary: library.pLibraryInfo_ is NULL.", __FILE__, __LINE__) ) ;
    }

    // Get a reference to library object.
    CTLibraryInfo&  libraryInfo = * ( (CTLibraryInfo*) library.pLibraryInfo_) ;

    // Create the library object.
    try
    {
        pLibraryInfo_ = (void*) new CTLibraryInfo (libraryInfo) ;
    }
    catch (CTxalloc&)
    {
        CTException::Throw (CTLibraryException ("CTLibrary: new CTLibraryInfo (libraryInfo) failed.", __FILE__, __LINE__) ) ;
    }
} // CTLibrary::CTLibrary

//-----------------------------------------------------------------------------
// CTLibrary::~CTLibrary
//
// Destructor.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
CTLibrary::~CTLibrary ()
{
    if (pLibraryInfo_)
    {
        // Delete the library object.
        delete (CTLibraryInfo*) pLibraryInfo_ ;
        pLibraryInfo_ = NULL ;
    }
} // CTLibrary::~CTLibrary

//================================================================< Operators >
//-----------------------------------------------------------------------------
// CTLibrary::operator=
//
// Operator=.
//
// Arguments : library : instance to copy.
// Return    : *this.
//-----------------------------------------------------------------------------
CTLibrary& CTLibrary::operator= (const CTLibrary& library)
{
    if (pLibraryInfo_)
    {
        // Delete the library object.
        delete (CTLibraryInfo*) pLibraryInfo_ ;
        pLibraryInfo_ = NULL ;
    }

    // Verify the library object is correct.
    if (library.pLibraryInfo_ == NULL)
    {   // Error from library.pLibraryInfo_.
        CTException::Throw (CTLibraryException ("operator=: library.pLibraryInfo_ is NULL.", __FILE__, __LINE__) ) ;
    }

    // Get a reference to library object.
    CTLibraryInfo&  libraryInfo = * ( (CTLibraryInfo*) library.pLibraryInfo_) ;

    // Create the library object.
    try
    {
        pLibraryInfo_ = (void*) new CTLibraryInfo (libraryInfo) ;
    }
    catch (CTxalloc&)
    {
        CTException::Throw (CTLibraryException ("operator=: new CTLibraryInfo (libraryInfo) failed.", __FILE__, __LINE__) ) ;
    }

    // Return a reference to *this.
    return *this ;
} // CTLibrary::operator=

//=========================================================< Control methodes >
//-----------------------------------------------------------------------------
// CTLibrary::getName
//
// Get the name of the library.
//
// Arguments : none.
// Return    : name : name of the library.
//-----------------------------------------------------------------------------
string CTLibrary::getName (void)
{
    // Verify the library object is correct.
    if (pLibraryInfo_ == NULL)
    {   // Error from pLibraryInfo_.
        CTException::Throw (CTLibraryException ("getName: pLibraryInfo_ is NULL.", __FILE__, __LINE__) ) ;
    }

    // Get a reference to library object.
    CTLibraryInfo&  libraryInfo = * ( (CTLibraryInfo*) pLibraryInfo_) ;

    // Return the name of the library.
    return libraryInfo.name_ ;
} // CTLibrary::getName

//-----------------------------------------------------------------------------
// CTLibrary::getName
//
// Get the name of the library.
//
// Arguments : none.
// Return    : name : name of the library.
//-----------------------------------------------------------------------------
string CTLibrary::getName (void) const
{
    // Verify the library object is correct.
    if (pLibraryInfo_ == NULL)
    {   // Error from pLibraryInfo_.
        CTException::Throw (CTLibraryException ("getName: pLibraryInfo_ is NULL.", __FILE__, __LINE__) ) ;
    }

    // Get a reference to library object.
    CTLibraryInfo&  libraryInfo = * ( (CTLibraryInfo*) pLibraryInfo_) ;

    // Return the name of the library.
    return libraryInfo.name_ ;
} // CTLibrary::getName

//-----------------------------------------------------------------------------
// CTLibrary::getInfo
//
// Get the address of the library information object.
//
// Arguments : none.
// Return    : library info : address of the library information object.
//-----------------------------------------------------------------------------
void* CTLibrary::getInfo (void) const
{
    // Verify the library object is correct.
    if (pLibraryInfo_ == NULL)
    {   // Error from pLibraryInfo_.
        CTException::Throw (CTLibraryException ("getInfo: pLibraryInfo_ is NULL.", __FILE__, __LINE__) ) ;
    }

    // Return the address of the library.
    return pLibraryInfo_ ;
} // CTLibrary::getInfo

//-----------------------------------------------------------------------------
// CTLibrary::getSystemFileName
//
// Get the real file name depending on system and debug/release build.
//
// Arguments : fileName  : partial name of the file.
//             eFileType : specify if the file is a library or an executable.
// Return    : a string object equal to the real name of the file.
//-----------------------------------------------------------------------------
string CTLibrary::getSystemFileName (const string& fileName, enumFileType eFileType /* = UNKNOWN_TYPE */)
{
    string  realFileName = fileName ;

    if (realFileName.find_first_of("\\/") == string::npos)
    {
#if defined(_DEBUG) || defined(DEBUG)
        // Add a 'd' or 'b' at the end of the file name to load the debug version of the file.
        size_t  pointIndex ;
        if ( (pointIndex = realFileName.find_last_of('.') ) != string::npos)
        {
            realFileName.insert(pointIndex,"d") ;
        }
        else
        {
            realFileName += "d" ;
        }
#endif  // _DEBUG

        if (realFileName.find_last_of ('.') == string::npos)
        {   // Add ".exe"/".dll" or "lib"+".so"/".sl".
            if (eFileType == UNKNOWN_TYPE)
            {
#ifdef  WIN32
                if (SearchPath (NULL, realFileName.c_str(), ".dll", 0, NULL, NULL) )
                {
                    eFileType = LIB_TYPE ;
                }
                else if (SearchPath (NULL, realFileName.c_str(), ".exe", 0, NULL, NULL) )
                {
                    eFileType = EXE_TYPE ;
                }
#endif  // WIN32
#ifdef  UNIX
                for (enumFileType eTryFileType = LIB_TYPE ;
                        (eFileType == UNKNOWN_TYPE) && ( (eTryFileType == LIB_TYPE) || (eTryFileType == EXE_TYPE) ) ;
                        eTryFileType = (enumFileType) ((int)eTryFileType + 1))
                {
                    char * stEnv = getenv ( (char*) ( (eTryFileType == LIB_TYPE) ? LIBPATH : "PATH") ) ;


                    if ( stEnv == NULL )
                    {
                        continue;
                    }

                    string       pathToSearch = string (stEnv) ;
		    char * tokenPath = new char[pathToSearch.size() + 1] ;
		    strcpy(tokenPath, pathToSearch.c_str());
		    char * directory = strtok(tokenPath, ":");

                    while (directory != NULL )
                    {
                        string   fullPathName = directory ;
                        fullPathName += "/" ;
                        if (eTryFileType == LIB_TYPE)
                        {
                            // Add "lib" if necessary.
                            if (realFileName.substr (0, 3) != "lib")
                            {
                                fullPathName += "lib" ;
                            }
                            fullPathName += realFileName ;
#  ifdef ALPHA
                            // Add ".so".
                            fullPathName += ".so" ;
#  endif    // ALPHA
#  ifdef SUN
                            // Add ".so".
                            fullPathName += ".so" ;
#  endif    // ALPHA
#  ifdef    AIX
                            // Add ".a".
                            fullPathName += ".a" ;
#  endif    // AIX
#  ifdef    HPUX
                            // Add ".sl".
                            fullPathName += ".sl" ;
#  endif    // HPUX
                        }

                        else
                        {
                            fullPathName += realFileName ;
                        }


                        if (access (fullPathName.c_str(), 0/*F_OK*/) == 0)
                        {
                            eFileType = eTryFileType ;
                            break ;
                        }
			directory = strtok(NULL, ":");
                    }
		    if (tokenPath)
			delete [] tokenPath ;
                }
#endif  // UNIX
            }

            switch (eFileType)
            {
            case LIB_TYPE : // Add ".dll" or "lib"+".so"/".sl".
#ifdef  WIN32
                realFileName += ".dll" ;
#endif  // WIN32
#ifdef  UNIX
                // Add "lib" if necessary.
                if (realFileName.substr (0, 3) != "lib")
                {
                    realFileName.insert(0,"lib") ;
                }
#  ifdef    ALPHA
                // Add ".so".
                realFileName += ".so" ;
#  endif    // ALPHA
#  ifdef    AIX
                // Add ".a".
                realFileName += ".a" ;
#  endif    // AIX
#  ifdef    HPUX
                // Add ".sl".
                realFileName += ".sl" ;
#  endif    // HPUX
#endif  // UNIX
                break ;
            case EXE_TYPE : // Add ".exe" or nothing.
#ifdef  WIN32
                realFileName += ".exe" ;
#endif  // WIN32
                break ;
            }
        }
    }

    return realFileName ;
} // CTLibrary::getSystemFileName

//-----------------------------------------------------------------------------
// CTLibrary::getFullPath
//
// Get the full path of the file.
//
// Arguments : fileName  : real name of the file.
// Return    : a string object equal to the full path of the file.
//-----------------------------------------------------------------------------
string CTLibrary::getFullPath (const string& fileName)
{
    string  fullPath = fileName ;

#ifdef  WIN32
    char    szFullFileName[MAX_PATH] ;
    if (SearchPath (NULL, fullPath.c_str(), NULL, MAX_PATH, szFullFileName, NULL) != 0)
    {
        fullPath = szFullFileName ;
    }
#endif  // WIN32

#ifdef HPUX
    // On HP : libraries must contain their whole path...
    char * stEnv = getenv ( (char*) LIBPATH ) ;

    if ( stEnv == NULL )
    {
        return fullPath;
    }

    string       pathToSearch = string (stEnv) ;
    char * tokenPath = new char[pathToSearch.size() + 1] ;
    strcpy(tokenPath, pathToSearch.c_str());
    char * directory = strtok(tokenPath, ":");

    while (directory != NULL )
    {
	string   fullPathName = directory ;
        fullPathName += "/" + fileName ;

        if (access (fullPathName.c_str(), 0/*F_OK*/) == 0)
        {
            fullPath =fullPathName;
            break ;
        }
	directory = strtok(NULL, ":");
    }
    if (tokenPath)
   	 delete tokenPath ;

#endif

    return fullPath ;
} // CTLibrary::getFullPath

//-----------------------------------------------------------------------------
// CTLibrary::getBaseFileName
//
// Get the base file name depending on system and debug/release build.
//
// Arguments : fileName  : system name or full path name of the file.
// Return    : a string object equal to the base name of the file.
//-----------------------------------------------------------------------------
string CTLibrary::getBaseFileName (const string& fileName)
{
    string  baseFileName = fileName ;
    size_t      index ;

    // Remove the path.
    if ( (index = baseFileName.find_last_of ('\\') ) != string::npos)
    {
        baseFileName.erase(0,index + 1) ;
    }
    if ( (index = baseFileName.find_last_of ('/') ) != string::npos)
    {
        baseFileName.erase(0,index + 1) ;
    }

    // Remove the extension.
    if ( (index = baseFileName.find_last_of('.') ) != string::npos)
    {
        baseFileName.erase(index) ;
    }

#ifdef  UNIX
    // Remove the 'lib' part.
    if (baseFileName.substr (0, 3) == "lib")
    {
        baseFileName.erase (0, 3) ;
    }
#endif  // UNIX

#if defined(_DEBUG) || defined(DEBUG)
    if (baseFileName == "tls7d")
    {
        baseFileName = "tls4d";
    }
    else if (baseFileName == "thr7d")
    {
        baseFileName = "thr4d";
    }
    else if (baseFileName == "net7d")
    {
        baseFileName = "net4d";
    }
    else if ((!baseFileName.empty() ) &&
             (baseFileName [baseFileName.length () - 1] == 'd'))
    {
        baseFileName.erase(baseFileName.length () - 1, 1) ;
    }
#endif  // _DEBUG

    return baseFileName ;
} // CTLibrary::getBaseFileName

//-----------------------------------------------------------------------------
// CTLibrary::getDirectory
//
// Get the directory of the file.
//
// Arguments : fileName  : full path name of the file.
// Return    : a string object equal to the directory of the file.
//-----------------------------------------------------------------------------
string CTLibrary::getDirectory (const string& fileName)
{
    string  directory = fileName ;

    // Get the directory.
    size_t      index1  = directory.find_last_of ('\\') ;
    size_t      index2  = directory.find_last_of ('/') ;
    size_t      index   = (size_t) ( ( (int) index1 > (int) index2) ? index1 : index2) ;
    if (index != string::npos)
    {
        directory.erase(index + 1) ;
    }
    else
    {
        directory = "";
    }
    return directory ;
} // CTLibrary::getDirectory
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////// End of Class CTLibrary //
///////////////////////////////////////////////////////////////////////////////

//======================================================< Definition of class >

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// Class CTProcedureInfo //
///////////////////////////////////////////////////////////////////////////////
class CTProcedureInfo : virtual public CTMTSafe
{
    //-----------------------------------------------------------------------------
public:                 // Constructor(s) and destructor.
    CTProcedureInfo (const CTLibrary& library, const string& name) ;
    CTProcedureInfo (const CTProcedureInfo& procedureInfo) ;
    ~CTProcedureInfo () ;

public:                 // Members.
    CTLibrary&          library_ ;
    void*               pAddress_ ;
    string          name_ ;

    //-----------------------------------------------------------------------------
private:
    CTProcedureInfo&    operator= (const CTProcedureInfo&) ;

    //-----------------------------------------------------------------------------
}

; // CTProcedureInfo

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////// End of Class CTProcedureInfo //
///////////////////////////////////////////////////////////////////////////////

//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// Class CTProcedureInfo //
///////////////////////////////////////////////////////////////////////////////
// Class used to get procedure address.

//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CTProcedureInfo::CTProcedureInfo
//
// Constructor.
//
// Arguments : library : library where to get the procedure.
//             name    : name of the procedure to get.
// Return    : none.
//-----------------------------------------------------------------------------
CTProcedureInfo::CTProcedureInfo (const CTLibrary& library, const string& name)
        : library_ ( (CTLibrary&) library), pAddress_ (NULL), name_ (name)
{
    // Get a reference to library object.
    CTLibraryInfo&  libraryInfo = * ( (CTLibraryInfo*) library_.getInfo () ) ;

#ifdef  WIN32
    // Get the procedure address.
    pAddress_ = (void*) GetProcAddress (libraryInfo.hLibrary_, name_.c_str() ) ;
    if (pAddress_ == NULL)
    {   // Error from GetProcAddress ().
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        CharToOem (zBuf, zBuf) ;
        CTException::Throw (CTLibraryException (string("CTProcedureInfo: GetProcAddress failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ) ;
    }
#endif  // WIN32
#if defined (DGUX) || defined (AIX) || defined(SUN)  || defined(ALPHA) || defined(LINUX)
    // Get the procedure address.
    pAddress_ = dlsym (libraryInfo.hLibrary_, name_.c_str() ) ;

    if (pAddress_ == NULL)
    {   // Error from dlsym ().
        string myString = string("CTProcedureInfo: dlsym failed: ") + string(strerror(errno));
        throw (CTLibraryException (myString, __FILE__, __LINE__, errno) ) ;
    }
#endif  // DGUX || AIX
#ifdef  HPUX
    // Get the procedure address.
    shl_t hLibrary = libraryInfo.hLibrary_ ;
    if (shl_findsym (&hLibrary, name_.c_str(), TYPE_PROCEDURE, &pAddress_) == -1)
    {   // Error from shl_findsym ().
        throw (CTLibraryException (string ("CTProcedureInfo: shl_findsym failed: ") + strerror (errno), __FILE__, __LINE__, errno) ) ;
    }
#endif  // HPUX
} // CTProcedureInfo::CTProcedureInfo

//-----------------------------------------------------------------------------
// CTProcedureInfo::CTProcedureInfo
//
// Copy constructor.
//
// Arguments : procedureInfo : procedure to copy.
// Return    : none.
//-----------------------------------------------------------------------------

CTProcedureInfo::CTProcedureInfo (const CTProcedureInfo& procedureInfo)
        : library_ (procedureInfo.library_), pAddress_ (NULL), name_ (procedureInfo.name_)
{
    // Get a reference to library object.
    CTLibraryInfo&  libraryInfo = * ( (CTLibraryInfo*) library_.getInfo () ) ;

#ifdef  WIN32
    // Get the procedure address.
    pAddress_ = (void*) GetProcAddress (libraryInfo.hLibrary_, name_.c_str() ) ;
    if (pAddress_ == NULL)
    {   // Error from GetProcAddress ().
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        CharToOem (zBuf, zBuf) ;
        CTException::Throw (CTLibraryException (string("CTProcedureInfo: GetProcAddress failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ) ;
    }
#endif  // WIN32
#if defined (DGUX) || defined (AIX) || defined(SUN)  || defined(ALPHAcd  ) || defined(LINUX)
    // Get the procedure address.
    pAddress_ = dlsym (libraryInfo.hLibrary_, name_.c_str() ) ;
    if (pAddress_ == NULL)
    {   // Error from dlsym ().
        throw (CTLibraryException (string ("CTProcedureInfo: dlsym failed: ") + strerror (errno), __FILE__, __LINE__, errno) ) ;
    }
#endif  // DGUX || AIX
#ifdef  HPUX
    // Get the procedure address.
    shl_t hLibrary = libraryInfo.hLibrary_ ;
    if (shl_findsym (&hLibrary, name_.c_str(), TYPE_PROCEDURE, &pAddress_) == -1)
    {   // Error from shl_findsym ().
        throw (CTLibraryException (string ("CTProcedureInfo: shl_findsym failed: ") + strerror (errno), __FILE__, __LINE__, errno) ) ;
    }
#endif  // HPUX
} // CTProcedureInfo::CTProcedureInfo

//-----------------------------------------------------------------------------
// CTProcedureInfo::~CTProcedureInfo
//
// Destructor.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------

CTProcedureInfo::~CTProcedureInfo ()
{
    if (pAddress_)
    {
        pAddress_ = NULL ;
    }
} // CTProcedureInfo::~CTProcedureInfo

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////// End of Class CTProcedureInfo //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////// Class CTProcedure //
///////////////////////////////////////////////////////////////////////////////
// Class used to get procedure address.

//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CTProcedure::CTProcedure
//
// Constructor.
//
// Arguments : library : library where to get the procedure.
//             name    : name of the procedure to get.
// Return    : none.
//-----------------------------------------------------------------------------
CTProcedure::CTProcedure (const CTLibrary& library, const string& name)
        : pProcedureInfo_ (NULL)
{
    // Create the procedure object.
    try
    {
        pProcedureInfo_ = (void*) new CTProcedureInfo (library, name) ;
    }
    catch (CTxalloc&)
    {
        CTException::Throw (CTLibraryException ("CTProcedure: new CTProcedureInfo (library, name) failed.", __FILE__, __LINE__) ) ;
    }
} // CTProcedure::CTProcedure

//-----------------------------------------------------------------------------
// CTProcedure::CTProcedure
//
// Copy constructor.
//
// Arguments : procedure : instance to copy.
// Return    : none.
//-----------------------------------------------------------------------------
CTProcedure::CTProcedure (const CTProcedure& procedure)
        : pProcedureInfo_ (NULL)
{
    // Verify the procedure object is correct.
    if (procedure.pProcedureInfo_ == NULL)
    {   // Error from procedure.pProcedureInfo_.
        CTException::Throw (CTLibraryException ("CTProcedure: procedure.pProcedureInfo_ is NULL.", __FILE__, __LINE__) ) ;
    }

    // Get a reference to procedure object.
    CTProcedureInfo&    procedureInfo = * ( (CTProcedureInfo*) procedure.pProcedureInfo_) ;

    // Create the procedure object.
    try
    {
        pProcedureInfo_ = (void*) new CTProcedureInfo (procedureInfo) ;
    }
    catch (CTxalloc&)
    {
        CTException::Throw (CTLibraryException ("CTProcedure: new CTProcedureInfo (procedureInfo) failed.", __FILE__, __LINE__) ) ;
    }
} // CTProcedure::CTProcedure

//-----------------------------------------------------------------------------
// CTProcedure::~CTProcedure
//
// Destructor.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
CTProcedure::~CTProcedure ()
{
    if (pProcedureInfo_)
    {
        // Delete the procedure object.
        delete (CTProcedureInfo*) pProcedureInfo_ ;
        pProcedureInfo_ = NULL ;
    }
} // CTProcedure::~CTProcedure

//================================================================< Operators >
//-----------------------------------------------------------------------------
// CTProcedure::operator=
//
// Operator=.
//
// Arguments : procedure : instance to copy.
// Return    : *this.
//-----------------------------------------------------------------------------
CTProcedure& CTProcedure::operator= (const CTProcedure& procedure)
{
    if (pProcedureInfo_)
    {
        // Delete the procedure object.
        delete (CTProcedureInfo*) pProcedureInfo_ ;
        pProcedureInfo_ = NULL ;
    }

    // Verify the procedure object is correct.
    if (procedure.pProcedureInfo_ == NULL)
    {   // Error from procedure.pProcedureInfo_.
        CTException::Throw (CTLibraryException ("operator=: procedure.pProcedureInfo_ is NULL.", __FILE__, __LINE__) ) ;
    }

    // Get a reference to procedure object.
    CTProcedureInfo&    procedureInfo = * ( (CTProcedureInfo*) procedure.pProcedureInfo_) ;

    // Create the procedure object.
    try
    {
        pProcedureInfo_ = (void*) new CTProcedureInfo (procedureInfo) ;
    }
    catch (CTxalloc&)
    {
        CTException::Throw (CTLibraryException ("operator=: new CTProcedureInfo (procedureInfo) failed.", __FILE__, __LINE__) ) ;
    }

    // Return a reference to *this.
    return *this ;
} // CTProcedure::operator=

//=========================================================< Control methodes >
//-----------------------------------------------------------------------------
// CTProcedure::getName
//
// Get the name of the procedure.
//
// Arguments : none.
// Return    : name : name of the procedure.
//-----------------------------------------------------------------------------
string CTProcedure::getName (void)
{
    // Verify the procedure object is correct.
    if (pProcedureInfo_ == NULL)
    {   // Error from pProcedureInfo_.
        CTException::Throw (CTLibraryException ("getName: pProcedureInfo_ is NULL.", __FILE__, __LINE__) ) ;
    }

    // Get a reference to procedure object.
    CTProcedureInfo&    procedureInfo = * ( (CTProcedureInfo*) pProcedureInfo_) ;

    // Return the name of the procedure.
    return procedureInfo.name_ ;
} // CTProcedure::getName

//-----------------------------------------------------------------------------
// CTProcedure::getName
//
// Get the name of the procedure.
//
// Arguments : none.
// Return    : name : name of the procedure.
//-----------------------------------------------------------------------------
string CTProcedure::getName (void) const
{
    // Verify the procedure object is correct.
    if (pProcedureInfo_ == NULL)
    {   // Error from pProcedureInfo_.
        CTException::Throw (CTLibraryException ("getName: pProcedureInfo_ is NULL.", __FILE__, __LINE__) ) ;
    }

    // Get a reference to procedure object.
    CTProcedureInfo&    procedureInfo = * ( (CTProcedureInfo*) pProcedureInfo_) ;

    // Return the name of the procedure.
    return procedureInfo.name_ ;
} // CTProcedure::getName

//-----------------------------------------------------------------------------
// CTProcedure::getAddress
//
// Get the address of the procedure.
//
// Arguments : none.
// Return    : address : address of the procedure.
//-----------------------------------------------------------------------------
void* CTProcedure::getAddress (void) const
{
    // Verify the procedure object is correct.
    if (pProcedureInfo_ == NULL)
    {   // Error from pProcedureInfo_.
        CTException::Throw (CTLibraryException ("getAddress: pProcedureInfo_ is NULL.", __FILE__, __LINE__) ) ;
    }

    // Get a reference to procedure object.
    CTProcedureInfo&    procedureInfo = * ( (CTProcedureInfo*) pProcedureInfo_) ;

    // Return the address of the procedure.
    return procedureInfo.pAddress_ ;
} // CTProcedure::getAddress

//-----------------------------------------------------------------------------
// CTProcedure::getLibrary
//
// Get the library of the procedure.
//
// Arguments : none.
// Return    : library : library of the procedure.
//-----------------------------------------------------------------------------
CTLibrary& CTProcedure::getLibrary (void)
{
    // Verify the procedure object is correct.
    if (pProcedureInfo_ == NULL)
    {   // Error from pProcedureInfo_.
        CTException::Throw (CTLibraryException ("getLibrary: pProcedureInfo_ is NULL.", __FILE__, __LINE__) ) ;
    }

    // Get a reference to procedure object.
    CTProcedureInfo&    procedureInfo = * ( (CTProcedureInfo*) pProcedureInfo_) ;

    // Return the library of the procedure.
    return procedureInfo.library_ ;
} // CTProcedure::getLibrary

//-----------------------------------------------------------------------------
// CTProcedure::getInfo
//
// Get the address of the procedure information object.
//
// Arguments : none.
// Return    : procedure info : address of the procedure information object.
//-----------------------------------------------------------------------------
void* CTProcedure::getInfo (void) const
{
    // Verify the procedure object is correct.
    if (pProcedureInfo_ == NULL)
    {   // Error from pProcedureInfo_.
        CTException::Throw (CTLibraryException ("getInfo: pProcedureInfo_ is NULL.", __FILE__, __LINE__) ) ;
    }

    // Return the address of the procedure.
    return pProcedureInfo_ ;
} // CTProcedure::getInfo

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// End of Class CTProcedure //
///////////////////////////////////////////////////////////////////////////////
