/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//==================================================================< Pragmas >
#ifdef  WIN32
#  pragma warning(disable : 4514)
#endif  // WIN32


//=================================================================< Includes >
#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifdef  WIN32
#  ifndef   _WINDOWS_
#    define WIN32_LEAN_AND_MEAN
#    define NOGDICAPMASKS
#    define NOVIRTUALKEYCODES
#    define NOWINMESSAGES
#    define NOWINSTYLES
#    define NOSYSMETRICS
#    define NOMENUS
#    define NOICONS
#    define NOKEYSTATES
#    define NOSYSCOMMANDS
#    define NORASTEROPS
#    define NOSHOWWINDOW
#    define OEMRESOURCE
#    define NOATOM
#    define NOCLIPBOARD
#    define NOCOLOR
#    define NOCTLMGR
#    define NODRAWTEXT
#    define NOGDI
#    define NOKERNEL
//#    define   NOUSER
#    define NONLS
#    define NOMB
#    define NOMEMMGR
#    define NOMETAFILE
#    define NOMINMAX
#    define NOMSG
#    define NOOPENFILE
#    define NOSCROLL
#    define NOSERVICE
#    define NOSOUND
#    define NOTEXTMETRIC
#    define NOWH
#    define NOWINOFFSETS
#    define NOCOMM
#    define NOKANJI
#    define NOHELP
#    define NOPROFILER
#    define NODEFERWINDOWPOS
#    define NOMCX
#    pragma warning(disable : 4201)
#    include <windows.h>
#    pragma warning(default : 4201)
#  endif    // _WINDOWS_
#endif  // WIN32

#ifdef  DGUX
#  ifndef   _POSIX4A_DRAFT6_SOURCE
#    define _POSIX4A_DRAFT6_SOURCE
#  endif    // _POSIX4A_DRAFT6_SOURCE
#  include <pthread.h>
#endif  // DGUX

#ifdef  AIX
#  include <pthread.h>
#endif  // AIX

#ifdef  HPUX
#  include <pthread.h>
#endif  // HPUX

#ifdef  SUN
#  include <pthread.h>
#  include <errno.h>
#endif  // HPUX

#ifndef __MUTEX__
#  include "mutex.hpp"
#endif  // __MUTEX__

#ifndef __XALLOC__
#  include "xalloc.hpp"
#endif  // __XALLOC__


//======================================================< Definition of class >

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////// Class CTSyncObjectInfo //
///////////////////////////////////////////////////////////////////////////////
// Class used to create and control a synchronization object with a counter.
class CTSyncObjectInfo : virtual public CTMTSafe
{
    //-----------------------------------------------------------------------------
public:                         // Control methodes.
    virtual void                lock (void) = 0 ;       // Lock the synchronization object.
    virtual bool                trylock (void) = 0 ;    // Try to lock the synchronization object.
    virtual void                unlock (void) = 0 ;     // Unlock the synchronization object.

    //-----------------------------------------------------------------------------
}

; // CTSyncObjectInfo

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////// End of Class CTSyncObjectInfo //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////// Class CTMutexInfo //
///////////////////////////////////////////////////////////////////////////////
// Class used to create and control a mutex with a counter.
class CTMutexInfo : public CTSyncObjectInfo
{
    //-----------------------------------------------------------------------------
public:                         // Constructor(s) and destructor.
    CTMutexInfo () ;
    ~CTMutexInfo () ;

public:                         // Control methodes.
    void                        lock (void) ;       // Lock the mutex.
    bool                        trylock (void) ;    // Try to lock the mutex.
    void                        unlock (void) ;     // Unlock the mutex.

public:                         // Members.
#ifdef  WIN32
    HANDLE                      hMutex_ ;
#endif  // WIN32
#if defined (DGUX) || defined (AIX) ||defined (HPUX) || defined(SUN)
    pthread_mutex_t*            hMutex_ ;
#  ifdef    DGUX
    pthread_key_t               keyCounter_ ;
#  endif    // DGUX
#endif  // DGUX || AIX || HPUX

    //-----------------------------------------------------------------------------
}

; // CTMutexInfo

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// End of Class CTMutexInfo //
///////////////////////////////////////////////////////////////////////////////


//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////// Class CTMutexInfo //
///////////////////////////////////////////////////////////////////////////////
// Class used to create and control a mutex with a counter.

//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CTMutexInfo::CTMutexInfo
//
// Default constructor.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
CTMutexInfo::CTMutexInfo ()
        : CTSyncObjectInfo (), hMutex_ (NULL)
{
#ifdef  WIN32
    // Create the mutex.
    hMutex_ = CreateMutex (NULL, FALSE, NULL) ;
    if (hMutex_ == NULL)
    {   // Error from CreateMutex ().
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        CharToOem (zBuf, zBuf) ;
        throw CTMutexException (string ("CTMutexInfo: CreateMutex failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
    }
#endif  // WIN32
#ifdef  DGUX
    // Create the counter key.
    if (pthread_key_create (&keyCounter_, NULL) == -1)
    {   // Error from pthread_keycreate ().
        throw CTMutexException (string ("CTMutexInfo: pthread_keycreate failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
    // Initialize the thread specific counter to 0.
    if (pthread_setspecific (keyCounter_, (void*) 0) == -1)
    {   // Error from pthread_setspecific ().
        throw CTMutexException (string ("CTMutexInfo: pthread_setspecific failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
    // Get a mutex info structure.
    try
    {
        hMutex_ = new pthread_mutex_t ;
    }
    catch (CTxalloc&)
    {
        throw CTMutexException ("CTMutexInfo: new pthread_mutex_t failed.", __FILE__, __LINE__) ;
    }
    // Create the mutex.
    if (pthread_mutex_init (hMutex_, NULL) == -1)
    {   // Error from pthread_mutex_init ().
        throw CTMutexException (string ("CTMutexInfo: pthread_mutex_init failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
#endif  // DGUX
#if defined (AIX) ||defined (HPUX) || defined(SUN)
    // Get a mutex info structure.
    try
    {
        hMutex_ = new pthread_mutex_t ;
    }
    catch (CTxalloc&)
    {
        throw CTMutexException ("CTMutexInfo: new pthread_mutex_t failed.", __FILE__, __LINE__) ;
    }
    // Create the mutex attributes.
    pthread_mutexattr_t attributes ;
#  if defined(AIX) || defined(SUN)
    if (pthread_mutexattr_init (&attributes) == -1)
    {   // Error from pthread_mutex_init ().
        throw CTMutexException (string ("CTMutexInfo: pthread_mutex_init failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
#  endif    // AIX
#  ifdef    HPUX
    if (pthread_mutexattr_create (&attributes) == -1)
    {   // Error from pthread_mutexattr_create ().
        throw CTMutexException (string ("CTMutexInfo: pthread_mutexattr_create failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
#  endif    // HPUX
#ifndef SUN
    // Set the mutex attributes to recursive mode.
    if (pthread_mutexattr_setkind_np (&attributes, SYNCHRONIZATION_RECURSIVE_NP) == -1)
    {   // Error from pthread_mutexattr_setkind_np ().
        throw CTMutexException (string ("CTMutexInfo: pthread_mutexattr_setkind_np failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
#endif
    // Create the mutex.
    if (pthread_mutex_init (hMutex_, &attributes) == -1)
    {   // Error from pthread_mutex_init ().
        throw CTMutexException (string ("CTMutexInfo: pthread_mutex_init failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
    // Destroy the attributes.
#  if defined(AIX) || defined(SUN)
    if (pthread_mutexattr_destroy (&attributes) == -1)
    {   // Error from pthread_mutexattr_destroy ().
        throw CTMutexException (string ("CTMutexInfo: pthread_mutexattr_destroy failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
#  endif    // AIX
#  ifdef    HPUX
    if (pthread_mutexattr_delete (&attributes) == -1)
    {   // Error from pthread_mutexattr_delete ().
        throw CTMutexException (string ("CTMutexInfo: pthread_mutexattr_delete failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
#  endif    // HPUX
#endif  // AIX || HPUX
} // CTMutexInfo::CTMutexInfo

//-----------------------------------------------------------------------------
// CTMutexInfo::~CTMutexInfo
//
// Destructor.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------

CTMutexInfo::~CTMutexInfo ()
{
    if (hMutex_)
    {
#ifdef  WIN32
        // Destroy the mutex.
        if (!CloseHandle (hMutex_) )
        {   // Error from CloseHandle ().
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTMutexException (string ("~CTMutexInfo: CloseHandle failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }
#endif  // WIN32
#if defined (DGUX) || defined (AIX) ||defined (HPUX) || defined(SUN)
        // Destroy the mutex.
        if (pthread_mutex_destroy (hMutex_) == -1)
        {   // Error from pthread_mutex_destroy ().
            throw CTMutexException (string ("~CTMutexInfo: pthread_mutex_destroy failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
        }
        // Destroy the mutex info structure.
        delete hMutex_ ;
#endif  // DGUX || AIX || HPUX
        hMutex_ = NULL ;
    }
} // CTMutexInfo::~CTMutexInfo

//=========================================================< Control methodes >
//-----------------------------------------------------------------------------
// CTMutexInfo::lock
//
// Lock the mutex.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CTMutexInfo::lock (void)
{
    // Verify the mutex is correct.
    if (hMutex_ == NULL)
    {   // Error from hMutex_.
        throw CTMutexException ("lock: hMutex_ is NULL.", __FILE__, __LINE__) ;
    }

#ifdef  WIN32
    // Lock the mutex.
    if (WaitForSingleObject (hMutex_, INFINITE) == WAIT_FAILED)
    {   // Error from WaitForSingleObject ().
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        CharToOem (zBuf, zBuf) ;
        throw CTMutexException (string ("lock: WaitForSingleObject failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
    }
#endif  // WIN32
#ifdef  DGUX
    unsigned uCounter ;
    // Get the thread lock counter.
    if (pthread_getspecific (keyCounter_, (void**) &uCounter) == -1)
    {   // Error from pthread_getspecific ().
        throw CTMutexException (string ("lock: pthread_getspecific failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
    if (uCounter == 0)
    {
        // Lock the mutex.
        if (pthread_mutex_lock (hMutex_) == -1)
        {   // Error from pthread_mutex_lock ().
            throw CTMutexException (string ("lock: pthread_mutex_lock failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
        }
    }
    // Increment the lock counter.
    ++ uCounter ;
    // Set the thread lock counter.
    if (pthread_setspecific (keyCounter_, (void*) uCounter) == -1)
    {   // Error from pthread_setspecific ().
        throw CTMutexException (string ("lock: pthread_setspecific failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
#endif  // DGUX
#if defined (AIX) ||defined (HPUX) || defined(SUN)
    // Lock the mutex.
    if (pthread_mutex_lock (hMutex_) == -1)
    {   // Error from pthread_mutex_lock ().
        throw CTMutexException (string ("lock: pthread_mutex_lock failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
#endif  // AIX || HPUX
} // CTMutexInfo::lock

//-----------------------------------------------------------------------------
// CTMutexInfo::trylock
//
// Try to lock the mutex.
//
// Arguments : none.
// Return    : bLocked : mutex locked by calling thread or not.
//-----------------------------------------------------------------------------

bool CTMutexInfo::trylock (void)
{
    bool    bLocked ;

    // Verify the mutex is correct.
    if (hMutex_ == NULL)
    {   // Error from hMutex_.
        throw CTMutexException ("trylock: hMutex_ is NULL.", __FILE__, __LINE__) ;
    }

#ifdef  WIN32
    // Try to lock the mutex.
    DWORD dLocked = WaitForSingleObject (hMutex_, 0) ;
    if (dLocked == WAIT_FAILED)
    {   // Error from WaitForSingleObject ().
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        CharToOem (zBuf, zBuf) ;
        throw CTMutexException (string ("trylock: WaitForSingleObject failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
    }
    bLocked = (dLocked != WAIT_TIMEOUT) ;
#endif  // WIN32
#ifdef  DGUX
    unsigned uCounter ;
    // Get the thread lock counter.
    if (pthread_getspecific (keyCounter_, (void**) &uCounter) == -1)
    {   // Error from pthread_getspecific ().
        throw CTMutexException (string ("trylock: pthread_getspecific failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
    if (uCounter > 0)
    {
        // The mutex is already locked.
        bLocked = TRUE ;
    }
    else
    {
        // Try to lock the mutex.
        int iLocked = pthread_mutex_trylock (hMutex_) ;
        if (iLocked == -1)
        {   // Error from pthread_mutex_trylock ().
            throw CTMutexException (string ("trylock: pthread_mutex_trylock failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
        }
        bLocked = (iLocked == TRUE) ;
    }
    if (bLocked)
    {
        // Increment the lock counter.
        ++ uCounter ;
    }
    // Set the thread lock counter.
    if (pthread_setspecific (keyCounter_, (void*) uCounter) == -1)
    {   // Error from pthread_setspecific ().
        throw CTMutexException (string ("trylock: pthread_setspecific failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
#endif  // DGUX
#if defined (AIX) ||defined (HPUX) || defined(SUN)
    // Try to lock the mutex.
    int iLocked = pthread_mutex_trylock (hMutex_) ;
    if (iLocked == -1)
    {   // Error from pthread_mutex_trylock ().
        throw CTMutexException (string ("trylock: pthread_mutex_trylock failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
    bLocked = (iLocked == true) ;
#endif  // AIX || HPUX

    // Return the locked state.
    return bLocked ;
} // CTMutexInfo::trylock

//-----------------------------------------------------------------------------
// CTMutexInfo::unlock
//
// Unlock the mutex.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CTMutexInfo::unlock (void)
{
    // Verify the mutex is correct.
    if (hMutex_ == NULL)
    {   // Error from hMutex_.
        throw CTMutexException ("unlock: hMutex_ is NULL.", __FILE__, __LINE__) ;
    }

#ifdef  WIN32
    // Unlock the mutex.
    if (!ReleaseMutex (hMutex_) )
    {   // Error from ReleaseMutex ().
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        CharToOem (zBuf, zBuf) ;
        throw CTMutexException (string ("unlock: ReleaseMutex failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
    }
    ::Sleep (0) ;
#endif  // WIN32
#ifdef  DGUX
    unsigned uCounter ;
    // Get the thread lock counter.
    if (pthread_getspecific (keyCounter_, (void**) &uCounter) == -1)
    {   // Error from pthread_getspecific ().
        throw CTMutexException (string ("unlock: pthread_getspecific failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
    if (uCounter == 1)
    {
        if (pthread_mutex_unlock (hMutex_) == -1)
        {   // Error from pthread_mutex_lock ().
            throw CTMutexException (string ("unlock: pthread_mutex_unlock failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
        }
    }
    // Decrement the lock counter.
    -- uCounter ;
    // Set the thread lock counter.
    if (pthread_setspecific (keyCounter_, (void*) uCounter) == -1)
    {   // Error from pthread_setspecific ().
        throw CTMutexException (string ("unlock: pthread_setspecific failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
    ::pthread_yield (NULL) ;
#endif  // DGUX
#if defined (AIX) ||defined (HPUX) 
    // Unlock the mutex.
    if (pthread_mutex_unlock (hMutex_) == -1)
    {   // Error from pthread_mutex_lock ().
        throw CTMutexException (string ("unlock: pthread_mutex_unlock failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
    ::pthread_yield () ;
#endif  // AIX || HPUX
#if defined(SUN)
    // Unlock the mutex.
    if (pthread_mutex_unlock (hMutex_) == -1)
    {   // Error from pthread_mutex_lock ().
        throw CTMutexException (string ("unlock: pthread_mutex_unlock failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
    ::sleep (0) ;
#endif  // AIX || HPUX
} // CTMutexInfo::unlock

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// End of Class CTMutexInfo //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////// Class CTMutex //
///////////////////////////////////////////////////////////////////////////////
// Class used to create and control mutex.

//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CTMutex::CTMutex
//
// Default constructor.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------

CTMutex::CTMutex ()
        : CTSyncObject ()
{
    // Create the mutex object.
    try
    {
        pSyncObjectInfo_ = (void*) new CTMutexInfo ;
    }
    catch (CTxalloc&)
    {
        throw CTMutexException ("CTMutex: new CTMutexInfo failed.", __FILE__, __LINE__) ;
    }
} // CTMutex::CTMutex

//-----------------------------------------------------------------------------
// CTMutex::~CTMutex
//
// Destructor.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
CTMutex::~CTMutex ()
{
    if (pSyncObjectInfo_)
    {
        // Delete the mutex object.
        delete (CTMutexInfo*) pSyncObjectInfo_ ;
        pSyncObjectInfo_ = NULL ;
    }
} // CTMutex::~CTMutex

//=========================================================< Control methodes >
//-----------------------------------------------------------------------------
// CTMutex::lock
//
// Lock the mutex.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CTMutex::lock (void)
{
    // Verify the mutex object is correct.
    if (pSyncObjectInfo_ == NULL)
    {   // Error from pSyncObjectInfo_.
        throw CTMutexException ("lock: pSyncObjectInfo_ is NULL.", __FILE__, __LINE__) ;
    }

    // Get a reference to mutex object.
    CTMutexInfo&    mutexInfo = * ( (CTMutexInfo*) pSyncObjectInfo_) ;

    // Lock the mutex object.
    mutexInfo.lock () ;
} // CTMutex::lock

//-----------------------------------------------------------------------------
// CTMutex::trylock
//
// Try to lock the mutex.
//
// Arguments : none.
// Return    : mutex locked by calling thread or not.
//-----------------------------------------------------------------------------
bool CTMutex::trylock (void)
{
    // Verify the mutex object is correct.
    if (pSyncObjectInfo_ == NULL)
    {   // Error from pSyncObjectInfo_.
        throw CTMutexException ("trylock: pSyncObjectInfo_ is NULL.", __FILE__, __LINE__) ;
    }

    // Get a reference to mutex object.
    CTMutexInfo&    mutexInfo = * ( (CTMutexInfo*) pSyncObjectInfo_) ;

    // Try to lock the mutex object.
    return mutexInfo.trylock () ;
} // CTMutex::trylock

//-----------------------------------------------------------------------------
// CTMutex::unlock
//
// Unlock the mutex.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CTMutex::unlock (void)
{
    // Verify the mutex object is correct.
    if (pSyncObjectInfo_ == NULL)
    {   // Error from pSyncObjectInfo_.
        throw CTMutexException ("unlock: pSyncObjectInfo_ is NULL.", __FILE__, __LINE__) ;
    }

    // Get a reference to mutex object.
    CTMutexInfo&    mutexInfo = * ( (CTMutexInfo*) pSyncObjectInfo_) ;

    // Unlock the mutex object.
    mutexInfo.unlock () ;
} // CTMutex::unlock

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////// End of Class CTMutex //
///////////////////////////////////////////////////////////////////////////////

//======================================================< Definition of class >

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// Class CTSemaphoreInfo //
///////////////////////////////////////////////////////////////////////////////
// Class used to create and control a semaphore.
class CTSemaphoreInfo : public CTSyncObjectInfo
{
    //-----------------------------------------------------------------------------
public:                         // Constructor(s) and destructor.
    CTSemaphoreInfo (long lInitialCount = 0L, long lMaximumCount = 0L) ;
    ~CTSemaphoreInfo () ;

public:                         // Control methodes.
    void                        lock (void) ;       // Decrease the counter.
    bool                        trylock (void) ;    // Try to decrease the counter.
    void                        unlock (void) ;     // Increase the counter.

public:                         // Members.
#ifdef  WIN32
    HANDLE                      hSemaphore_ ;
#endif  // WIN32
#if defined (DGUX) || defined (AIX) ||defined (HPUX) || defined(SUN)
	void * hSemaphore_ ;

    // TODO TODO TODO
#endif  // DGUX || AIX || HPUX

    //-----------------------------------------------------------------------------
}

; // CTSemaphoreInfo

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////// End of Class CTSemaphoreInfo //
///////////////////////////////////////////////////////////////////////////////


//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// Class CTSemaphoreInfo //
///////////////////////////////////////////////////////////////////////////////
// Class used to create and control a semaphore.

//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CTSemaphoreInfo::CTSemaphoreIwnfo
//
// Default constructor.
//
// lInitialCount : initial counter value.
// lMaximumCount : maximum counter value.
// Return        : none.
//-----------------------------------------------------------------------------
CTSemaphoreInfo::CTSemaphoreInfo (long lInitialCount /* = 0L */, long lMaximumCount /* = 0L */)
        : CTSyncObjectInfo (), hSemaphore_ (NULL)
{
#ifdef  WIN32
    // Create the semaphore.
    if (lMaximumCount == 0L)
    {
        lMaximumCount = 255L ;
    }
    hSemaphore_ = CreateSemaphore (NULL, lInitialCount, lMaximumCount, NULL) ;
    if (hSemaphore_ == NULL)
    {   // Error from CreateSemaphore ().
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        CharToOem (zBuf, zBuf) ;
        throw CTSemaphoreException (string ("CTSemaphoreInfo: CreateSemaphore failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
    }
#endif  // WIN32
#if defined (DGUX) || defined (AIX) ||defined (HPUX) || defined(SUN)
    // TODO TODO TODO
#endif  // DGUX || AIX || HPUX
} // CTSemaphoreInfo::CTSemaphoreInfo

//-----------------------------------------------------------------------------
// CTSemaphoreInfo::~CTSemaphoreInfo
//
// Destructor.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------

CTSemaphoreInfo::~CTSemaphoreInfo ()
{
    if (hSemaphore_)
    {
#ifdef  WIN32
        // Destroy the semaphore.
        if (!CloseHandle (hSemaphore_) )
        {   // Error from CloseHandle ().
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTSemaphoreException (string ("~CTSemaphoreInfo: CloseHandle failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }
#endif  // WIN32
#if defined (DGUX) || defined (AIX) ||defined (HPUX) || defined(SUN)
        // TODO TODO TODO
#endif  // DGUX || AIX || HPUX
        hSemaphore_ = NULL ;
    }
} // CTSemaphoreInfo::~CTSemaphoreInfo

//=========================================================< Control methodes >
//-----------------------------------------------------------------------------
// CTSemaphoreInfo::lock
//
// Lock the semaphore.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CTSemaphoreInfo::lock (void)
{
    // Verify the semaphore is correct.
    if (hSemaphore_ == NULL)
    {   // Error from hSemaphore_.
        throw CTSemaphoreException ("lock: hSemaphore_ is NULL.", __FILE__, __LINE__) ;
    }

#ifdef  WIN32
    // Lock the Semaphore.
    if (WaitForSingleObject (hSemaphore_, INFINITE) == WAIT_FAILED)
    {   // Error from WaitForSingleObject ().
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        CharToOem (zBuf, zBuf) ;
        throw CTSemaphoreException (string ("lock: WaitForSingleObject failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
    }
#endif  // WIN32
#if defined (DGUX) || defined (AIX) ||defined (HPUX) || defined(SUN)
    // TODO TODO TODO
#endif  // DGUX || AIX || HPUX
} // CTSemaphoreInfo::lock

//-----------------------------------------------------------------------------
// CTSemaphoreInfo::trylock
//
// Try to lock the semaphore.
//
// Arguments : none.
// Return    : bLocked : semaphore locked by calling thread or not.
//-----------------------------------------------------------------------------

bool CTSemaphoreInfo::trylock (void)
{
    bool    bLocked ;

    // Verify the semaphore is correct.
    if (hSemaphore_ == NULL)
    {   // Error from hSemaphore_.
        throw CTSemaphoreException ("trylock: hSemaphore_ is NULL.", __FILE__, __LINE__) ;
    }

#ifdef  WIN32
    // Try to lock the semaphore.
    DWORD dLocked = WaitForSingleObject (hSemaphore_, 0) ;
    if (dLocked == WAIT_FAILED)
    {   // Error from WaitForSingleObject ().
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        CharToOem (zBuf, zBuf) ;
        throw CTSemaphoreException (string ("trylock: WaitForSingleObject failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
    }
    bLocked = (dLocked != WAIT_TIMEOUT) ;
#endif  // WIN32
#if defined (DGUX) || defined (AIX) ||defined (HPUX) || defined(SUN)
    // TODO TODO TODO
#endif  // DGUX || AIX || HPUX

    // Return the locked state.
    return bLocked ;
} // CTSemaphoreInfo::trylock

//-----------------------------------------------------------------------------
// CTSemaphoreInfo::unlock
//
// Unlock the semaphore.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CTSemaphoreInfo::unlock (void)
{
    // Verify the semaphore is correct.
    if (hSemaphore_ == NULL)
    {   // Error from hSemaphore_.
        throw CTSemaphoreException ("unlock: hSemaphore_ is NULL.", __FILE__, __LINE__) ;
    }

#ifdef  WIN32
    // Unlock the semaphore.
    if (!ReleaseSemaphore (hSemaphore_, 1, NULL) )
    {   // Error from ReleaseSemaphore ().
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        CharToOem (zBuf, zBuf) ;
        throw CTSemaphoreException (string ("unlock: ReleaseSemaphore failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
    }
    ::Sleep (0) ;
#endif  // WIN32
#if defined (DGUX) || defined (AIX) ||defined (HPUX) || defined(SUN)
    // TODO TODO TODO
#endif  // DGUX || AIX || HPUX
} // CTSemaphoreInfo::unlock

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////// End of Class CTSemaphoreInfo //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////// Class CTSemaphore //
///////////////////////////////////////////////////////////////////////////////
// Class used to create and control semaphore.

//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CTSemaphore::CTSemaphore
//
// Default constructor.
//
// lInitialCount : initial counter value.
// lMaximumCount : maximum counter value.
// Return        : none.
//-----------------------------------------------------------------------------

CTSemaphore::CTSemaphore (long lInitialCount /* = 0L */, long lMaximumCount /* = 0L */)
        : CTSyncObject ()
{
    // Create the semaphore object.
    try
    {
        pSyncObjectInfo_ = (void*) new CTSemaphoreInfo (lInitialCount, lMaximumCount) ;
    }
    catch (CTxalloc&)
    {
        throw CTSemaphoreException ("CTSemaphore: new CTSemaphoreInfo failed.", __FILE__, __LINE__) ;
    }
} // CTSemaphore::CTSemaphore

//-----------------------------------------------------------------------------
// CTSemaphore::~CTSemaphore
//
// Destructor.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
CTSemaphore::~CTSemaphore ()
{
    if (pSyncObjectInfo_)
    {
        // Delete the semaphore object.
        delete (CTSemaphoreInfo*) pSyncObjectInfo_ ;
        pSyncObjectInfo_ = NULL ;
    }
} // CTSemaphore::~CTSemaphore

//=========================================================< Control methodes >
//-----------------------------------------------------------------------------
// CTSemaphore::lock
//
// Lock the semaphore.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CTSemaphore::lock (void)
{
    // Verify the semaphore object is correct.
    if (pSyncObjectInfo_ == NULL)
    {   // Error from pSyncObjectInfo_.
        throw CTSemaphoreException ("lock: pSyncObjectInfo_ is NULL.", __FILE__, __LINE__) ;
    }

    // Get a reference to semaphore object.
    CTSemaphoreInfo&    semaphoreInfo = * ( (CTSemaphoreInfo*) pSyncObjectInfo_) ;

    // Lock the semaphore object.
    semaphoreInfo.lock () ;
} // CTSemaphore::lock

//-----------------------------------------------------------------------------
// CTSemaphore::trylock
//
// Try to lock the semaphore.
//
// Arguments : none.
// Return    : semaphore locked by calling thread or not.
//-----------------------------------------------------------------------------
bool CTSemaphore::trylock (void)
{
    // Verify the semaphore object is correct.
    if (pSyncObjectInfo_ == NULL)
    {   // Error from pSyncObjectInfo_.
        throw CTSemaphoreException ("trylock: pSyncObjectInfo_ is NULL.", __FILE__, __LINE__) ;
    }

    // Get a reference to semaphore object.
    CTSemaphoreInfo&    semaphoreInfo = * ( (CTSemaphoreInfo*) pSyncObjectInfo_) ;

    // Try to lock the semaphore object.
    return semaphoreInfo.trylock () ;
} // CTSemaphore::trylock

//-----------------------------------------------------------------------------
// CTSemaphore::unlock
//
// Unlock the semaphore.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CTSemaphore::unlock (void)
{
    // Verify the semaphore object is correct.
    if (pSyncObjectInfo_ == NULL)
    {   // Error from pSyncObjectInfo_.
        throw CTSemaphoreException ("unlock: pSyncObjectInfo_ is NULL.", __FILE__, __LINE__) ;
    }

    // Get a reference to semaphore object.
    CTSemaphoreInfo&    semaphoreInfo = * ( (CTSemaphoreInfo*) pSyncObjectInfo_) ;

    // Unlock the semaphore object.
    semaphoreInfo.unlock () ;
} // CTSemaphore::unlock

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// End of Class CTSemaphore //
///////////////////////////////////////////////////////////////////////////////

//======================================================< Definition of class >

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// Class CTConditionInfo //
///////////////////////////////////////////////////////////////////////////////
// Class used to create and control a condition.
class CTConditionInfo : virtual public CTMTSafe
{
    //-----------------------------------------------------------------------------
public:                         // Constructor(s) and destructor.
    CTConditionInfo (CTMutex& mutex) ;
    ~CTConditionInfo () ;

public:                         // Control methodes.
    void                        wait (void) ;       // Wait for the condition.
    void                        signal (void) ;     // Wake up a waiting thread.
    void                        broadcast (void) ;  // Wake up all waiting threads.

public:                         // Operators.
    CTConditionInfo&            operator= (const CTConditionInfo&) ;

public:                         // Members.
    CTMutex&                    mutex_ ;
#ifdef  WIN32
    PHANDLE                     pCondition_ ;
#endif  // WIN32
#if defined (DGUX) || defined (AIX) ||defined (HPUX) || defined(SUN)
    pthread_cond_t*             pCondition_ ;
#endif  // DGUX || AIX || HPUX

    //-----------------------------------------------------------------------------
}

; // CTConditionInfo

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////// End of Class CTConditionInfo //
///////////////////////////////////////////////////////////////////////////////


//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// Class CTConditionInfo //
///////////////////////////////////////////////////////////////////////////////
// Class used to create and control a condition.

//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CTConditionInfo::CTConditionInfo
//
// Constructor.
//
// mutex  : mutex used to synchronize access to the condition object.
// Return : none.
//-----------------------------------------------------------------------------
CTConditionInfo::CTConditionInfo (CTMutex& mutex)
        : mutex_ (mutex)
{
#ifdef  WIN32
    // Get an event object array.
    try
    {
        pCondition_ = new HANDLE[2] ;
    }
    catch (CTxalloc&)
    {
        throw CTConditionException ("CTConditionInfo: new HANDLE[2] failed.", __FILE__, __LINE__) ;
    }
    // Create an auto-reset event object.
    pCondition_[0] = CreateEvent (NULL, FALSE, FALSE, NULL) ;
    if (pCondition_[0] == NULL)
    {   // Error from CreateEvent ().
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        CharToOem (zBuf, zBuf) ;
        throw CTConditionException (string ("CTConditionInfo: CreateEvent failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
    }
    // Create a manual-reset event object.
    pCondition_[1] = CreateEvent (NULL, TRUE, FALSE, NULL) ;
    if (pCondition_[1] == NULL)
    {   // Error from CreateEvent ().
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        CharToOem (zBuf, zBuf) ;
        throw CTConditionException (string ("CTConditionInfo: CreateEvent failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
    }
#endif  // WIN32
#if defined (DGUX) || defined (AIX) ||defined (HPUX) || defined(SUN)
    // Get a condition info structure.
    try
    {
        pCondition_ = new pthread_cond_t ;
    }
    catch (CTxalloc&)
    {
        throw CTConditionException ("CTConditionInfo: new pthread_cond_t failed.", __FILE__, __LINE__) ;
    }
    // Initialize the condition object.
#  if   defined (DGUX) || defined (AIX) || defined(SUN)
    if (pthread_cond_init (pCondition_, NULL) == -1)
#  endif    // DGUX || AIX
#  ifdef    HPUX
        if (pthread_cond_init (pCondition_, pthread_condattr_default) == -1)
#  endif    // HPUX
        {   // Error from pthread_cond_init ().
            throw CTConditionException (string ("CTConditionInfo: pthread_cond_init failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
        }
#endif  // DGUX || AIX || HPUX
} // CTConditionInfo::CTConditionInfo

//-----------------------------------------------------------------------------
// CTConditionInfo::~CTConditionInfo
//
// Destructor.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------

CTConditionInfo::~CTConditionInfo ()
{
    if (pCondition_)
    {
#ifdef  WIN32
        // Destroy the auto-reset event object.
        if (!CloseHandle (pCondition_[0]) )
        {   // Error from CloseHandle ().
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTConditionException (string ("~CTConditionInfo: CloseHandle failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }
        // Destroy the manual-reset event object.
        if (!CloseHandle (pCondition_[1]) )
        {   // Error from CloseHandle ().
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTConditionException (string ("~CTConditionInfo: CloseHandle failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }
        // Destroy the event object array.
        delete [] pCondition_ ;
#endif  // WIN32
#if defined (DGUX) || defined (AIX) ||defined (HPUX) || defined(SUN)
        // Destroy the condition object.
        if (pthread_cond_destroy (pCondition_) == -1)
        {   // Error from pthread_cond_destroy ().
            throw CTMutexException (string ("~CTConditionInfo: pthread_cond_destroy failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
        }
        // Destroy the condition info structure.
        delete pCondition_ ;
#endif  // DGUX || AIX || HPUX
        pCondition_ = NULL ;
    }
} // CTConditionInfo::~CTConditionInfo

//=========================================================< Control methodes >

//-----------------------------------------------------------------------------
// CTConditionInfo::wait
//
// Wait for the condition object to be signaled.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CTConditionInfo::wait (void)
{
    // Verify the condition object is correct.
    if (pCondition_ == NULL)
    {   // Error from pCondition_.
        throw CTConditionException ("wait: pCondition_ is NULL.", __FILE__, __LINE__) ;
    }

#ifdef  WIN32
    {
        // Release the mutex.
        CTUnlockGuard<CTMutex>  unguard (mutex_) ;
        // Wait for the event object.
        if (WaitForMultipleObjects (2, pCondition_, FALSE, INFINITE) == WAIT_FAILED)
        {   // Error from WaitForMultipleObjects ().
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTConditionException (string ("wait: WaitForMultipleObjects failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }
    }
#endif  // WIN32
#if defined (DGUX) || defined (AIX) ||defined (HPUX) || defined(SUN)
    // Get a reference to mutex object.
    CTMutexInfo&    mutexInfo = * ( (CTMutexInfo*) mutex_.getMutexInfo () ) ;
    // Wait for the condition object.
    if (pthread_cond_wait (pCondition_,  mutexInfo.hMutex_) == -1)
    {   // Error from pthread_cond_wait ().
        throw CTMutexException (string ("wait: pthread_cond_wait failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
#endif  // DGUX || AIX || HPUX
} // CTConditionInfo::wait

//-----------------------------------------------------------------------------
// CTConditionInfo::signal
//
// Signal a condition object, so wake up a waiting thread.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------

void CTConditionInfo::signal (void)
{
    // Verify the condition object is correct.
    if (pCondition_ == NULL)
    {   // Error from pCondition_.
        throw CTConditionException ("signal: pCondition_ is NULL.", __FILE__, __LINE__) ;
    }

#ifdef  WIN32
    {
        // Get the mutex acquisition.
        CTLockGuard<CTMutex>    guard (mutex_) ;
        // Signal the auto-reset event object.
        if (!SetEvent (pCondition_[0]) )
        {   // Error from SetEvent ().
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTConditionException (string ("signal: SetEvent failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }
    }
#endif  // WIN32
#if defined (DGUX) || defined (AIX) ||defined (HPUX) || defined(SUN)
    // Signal the condition object.
    if (pthread_cond_signal (pCondition_) == -1)
    {   // Error from pthread_cond_signal ().
        throw CTMutexException (string ("signal: pthread_cond_signal failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
#endif  // DGUX || AIX || HPUX
} // CTConditionInfo::signal

//-----------------------------------------------------------------------------
// CTConditionInfo::broadcast
//
// Signal a condition object, so wake up all waiting threads.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------

void CTConditionInfo::broadcast (void)
{
    // Verify the condition object is correct.
    if (pCondition_ == NULL)
    {   // Error from pCondition_.
        throw CTConditionException ("broadcast: pCondition_ is NULL.", __FILE__, __LINE__) ;
    }

#ifdef  WIN32
    {
        // Get the mutex acquisition.
        CTLockGuard<CTMutex>    guard (mutex_) ;
        // Signal the manual-reset event object.
        if (!PulseEvent (pCondition_[1]) )
        {   // Error from PulseEvent ().
            char zBuf[255] ;
            FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
            CharToOem (zBuf, zBuf) ;
            throw CTConditionException (string ("broadcast: PulseEvent failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
        }
    }
#endif  // WIN32
#if defined (DGUX) || defined (AIX) ||defined (HPUX) || defined(SUN)
    // Signal the condition object.
    if (pthread_cond_broadcast (pCondition_) == -1)
    {   // Error from pthread_cond_broadcast ().
        throw CTMutexException (string ("broadcast: pthread_cond_broadcast failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
    }
#endif  // DGUX || AIX || HPUX
} // CTConditionInfo::broadcast

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////// End of Class CTConditionInfo //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////// Class CTCondition //
///////////////////////////////////////////////////////////////////////////////
// Class used to synchronize execution on a condition.

//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CTCondition::CTCondition
//
// Constructor.
//
// mutex  : mutex used to synchronize access to the condition object.
// Return : none.
//-----------------------------------------------------------------------------

CTCondition::CTCondition (CTMutex& mutex)
        : pConditionInfo_ (NULL)
{
    // Create the condition object.
    try
    {
        pConditionInfo_ = (void*) new CTConditionInfo (mutex) ;
    }
    catch (CTxalloc&)
    {
        throw CTConditionException ("CTCondition: new CTConditionInfo (mutex) failed.", __FILE__, __LINE__) ;
    }
} // CTCondition::CTCondition

//-----------------------------------------------------------------------------
// CTCondition::~CTCondition
//
// Destructor.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
CTCondition::~CTCondition ()
{
    if (pConditionInfo_)
    {
        // Delete the condition object.
        delete (CTConditionInfo*) pConditionInfo_ ;
        pConditionInfo_ = NULL ;
    }
} // CTCondition::~CTCondition

//=========================================================< Control methodes >

//-----------------------------------------------------------------------------
// CTCondition::wait
//
// Wait for the condition object to be signaled.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CTCondition::wait (void)
{
    // Verify the condition object is correct.
    if (pConditionInfo_ == NULL)
    {   // Error from pConditionInfo_.
        throw CTMutexException ("wait: pConditionInfo_ is NULL.", __FILE__, __LINE__) ;
    }

    // Get a reference to condition object.
    CTConditionInfo&    conditionInfo = * ( (CTConditionInfo*) pConditionInfo_) ;

    // Wait for the condition object to be signaled.
    conditionInfo.wait () ;
} // CTCondition::wait

//-----------------------------------------------------------------------------
// CTCondition::signal
//
// Signal the condition object to wake up a waiting thread.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CTCondition::signal (void)
{
    // Verify the condition object is correct.
    if (pConditionInfo_ == NULL)
    {   // Error from pConditionInfo_.
        throw CTMutexException ("signal: pConditionInfo_ is NULL.", __FILE__, __LINE__) ;
    }

    // Get a reference to condition object.
    CTConditionInfo&    conditionInfo = * ( (CTConditionInfo*) pConditionInfo_) ;

    // Signal the condition object to wake up a waiting thread.
    conditionInfo.signal () ;
} // CTCondition::signal

//-----------------------------------------------------------------------------
// CTCondition::broadcast
//
// Signal the condition object to wake up all waiting threads.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CTCondition::broadcast (void)
{
    // Verify the condition object is correct.
    if (pConditionInfo_ == NULL)
    {   // Error from pConditionInfo_.
        throw CTMutexException ("broadcast: pConditionInfo_ is NULL.", __FILE__, __LINE__) ;
    }

    // Get a reference to condition object.
    CTConditionInfo&    conditionInfo = * ( (CTConditionInfo*) pConditionInfo_) ;

    // Signal the condition object to wake up all waiting threads
    conditionInfo.broadcast () ;
} // CTCondition::broadcast

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// End of Class CTCondition //
///////////////////////////////////////////////////////////////////////////////
