/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// This class implements a shared pointer, that is, a pointer wrapper
// able to share a pointer among several owners. The lifetime of
// the actual pointer is managed by a reference count.

#ifndef _SHARED_POINTER_H
#define _SHARED_POINTER_H

template <class T>
class SharedPtr
{
    T*    ptr;      // Actual pointer.
    long* pCount;   // Reference count.

public:
    // Constructor.
    explicit SharedPtr(T* p = NULL) : ptr(p), pCount(new long(1)) {}

    // Copy constructor.
    SharedPtr(const SharedPtr<T>& p) : ptr(p.ptr), pCount(p.pCount) {++*pCount;}

    // Destructor.
    ~SharedPtr() {release();}

    // Assignment.
    SharedPtr<T>& operator=(const SharedPtr<T>& p)
    {
        if(this != &p)
        {
            release();
            ptr = p.ptr;
            pCount = p.pCount;
            ++*pCount;
        }
        return *this;
    }

    // Dereference the pointer.
    T& operator*() const {return *ptr;}

    // -> operator.
    T* operator->() const {return ptr;}

    // NULL test.
    bool IsNull() const {return ptr == NULL;}

private:
    void release()
    {
        if(--*pCount == 0)
        {
            delete pCount;
            delete ptr;
        }
    }
};

#endif  // _SHARED_POINTER_H
