/* Copyright (C) 2000-2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECOperand.cpp

#include <string>

#include "CECOperand.h"
#include "cxxutils.h"
#include "TPLog.h"


///////////////////////////////////////////////////////////////////////////////
// @mfunc   Construct and return a string listing the operand contents. This
//			string can be used log diagnostic information.
//
// @rdesc	String listing of the operand contents.
//
string CECOperand::GetContent()
{
    string val_addrm;
    string val_dtyp;
    string val_chrval = "None";
    string val_opr;
    string val_intval;
    string val_varnam = "None";
    string val_vardims;
    string val_fnc;

#ifndef CYR_NO_STD_NAMESPACE
    // Set addressing mode
    switch (_addrM)
    {
    case addr_none:
        val_addrm = "None";
        break;
    case addr_immediate:
        val_addrm = "Immediate";
        break;
    case addr_operator:
        val_addrm = "Operator";
        break;
    case addr_variable:
        val_addrm = "Variable";
        break;
    default:
        val_addrm = "Unknown (" + int_to_string(_addrM) + ')';
    }

    // Set data type
    switch (_dTyp)
    {
    case dtyp_none:
        val_dtyp = "None";
        break;
    case dtyp_integer:
        val_dtyp = "Integer";
        break;
    case dtyp_character:
        val_dtyp = "Character";
        break;
    default:
        val_dtyp = "Unknown (" + int_to_string(_dTyp) + ')';
    }

    // Set integer value, character value, variable name and variable dimensions.
    val_intval = int_to_string (_intVal);
    if (_chrValP.empty() == false) val_chrval = _chrValP;   // Otherwise, value is "None".
    if (_varNamP.empty() == false) val_varnam = _varNamP;   // Otherwise, value is "None".

    val_vardims = int_to_string(_varDims);

    // Set operator.
    switch (_opr)
    {
    case opr_none:
        val_opr = "None";
        break;
    case opr_not:
        val_opr = "Not";
        break;
    case opr_and:
        val_opr = "And";
        break;
    case opr_or:
        val_opr = "Or";
        break;
    case opr_eq:
        val_opr = "Eq";
        break;
    case opr_ne:
        val_opr = "Ne";
        break;
    case opr_lt:
        val_opr = "Lt";
        break;
    case opr_le:
        val_opr = "Le";
        break;
    case opr_gt:
        val_opr = "Gt";
        break;
    case opr_ge:
        val_opr = "Ge";
        break;
    case opr_contains:
        val_opr = "Contains";
        break;
    case opr_not_contains:
        val_opr = "Not_contains";
        break;
    case opr_add:
        val_opr = "Add";
        break;
    case opr_sub:
        val_opr = "Sub";
        break;
    case opr_mul:
        val_opr = "Mul";
        break;
    case opr_div:
        val_opr = "Div";
        break;
    case opr_mod:
        val_opr = "Mod";
        break;
    case opr_bitior:
        val_opr = "BitIOR";
        break;
    case opr_bitxor:
        val_opr = "BitXOR";
        break;
    case opr_bitand:
        val_opr = "BitAND";
        break;
    default:
        val_opr = "Unknown (" + int_to_string(_opr) + ')';
    }

    // Set function code.
    switch (_fncCod)
    {
    case fnc_none:
        val_fnc = "None";
        break;
    case fnc_extract:
        val_fnc = "~Extract";
        break;
    case fnc_locate:
        val_fnc = "~Locate";
        break;
    case fnc_neg_exp_dist:
        val_fnc = "~Neg_Exp_Dist";
        break;
    default:
        val_fnc = "Unknown (" + int_to_string(_fncCod) + ')';
    }

#endif
    return ("OPERAND: Addrm: " + val_addrm + "; Dtyp: " + val_dtyp +
            "; ChrVal: " + val_chrval + "; IntVal: " + val_intval +
            "; Opr: " + val_opr + "; VarNam: " + val_varnam +
            "; VarDims: " + val_vardims + "; FncCod: " + val_fnc + '.');
} // End GetContent()



