/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
///////////////////////////////////////////////////////////////////////////////
// @doc     CECVar
// @module  CECVar | Variable base class
///////////////////////////////////////////////////////////////////////////////
#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif
#ifndef _CECVAR_H_
#define _CECVAR_H_
#include <string>
#include "mdbexcept.h"
#include "mdbmtx.h"
#include "CECVarOptions.h"
#ifndef CYR_NO_STD_NAMESPACE
using std::string;
#endif

class CECVar
{
public:
    CECVar(const string& name):_name(name)
    {}
    virtual ~CECVar()
    {}
    virtual string  get_value(const string& expression)=0;
    virtual int	    set_value(const string& newval,const string& expression)=0;
    virtual void    next_value()        //generate next value
    {
        return;
    }
    virtual void    generate_value()    //generate random value
    {
        return;
    }
    virtual void    reset_seed(unsigned long=0) //reset random seed
    {
        return;
    }

    //things required by Cvarlist
    // All the methods that use _options can probably be made non-pure virtuals
    // and be used by all classes
    virtual int	    get_options()
    {
        return 0;
    }
    virtual string  get_name()
    {
        return "";
    }
    virtual bool    is_volatile()=0;
    virtual bool    is_temp(void)=0;
    virtual bool    is_private(void)=0;
    virtual void    clear_values(void)=0;
    virtual int	    get_size(void)=0;
protected:
    unsigned long ParseIndecies(const string& indecies,vector<int> *min_index,vector<int> *max_index,vector<int> *size_index);
    unsigned long GetIndexValue(const string& index,const vector<int> &min_index,const vector<int> &max_index,const vector<int> &size_index);
    static int GetRandomSeed();
    string _name;
};

#endif
