/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef _TECORBAREPOSITORY_H_
#define _TECORBAREPOSITORY_H_
///////////////////////////////////////////////////////////////////////////////
// @doc     TECORBARepository
// @module  TECORBARepository |
//
///////////////////////////////////////////////////////////////////////////////
#include "CORBARepository.h"
#include "mdbexcept.h"
#ifdef WIN32
#pragma warning(disable:4786)
#endif
extern CORBAServer*    pCServer;
//@class CORBARepository
class TECORBARepository:public CORBARepository
{
public:
    TECORBARepository(CORBAServer* corba,const string& node,const string& user,const string& pass,const string& NodeName,int pid,const string& exename)
        :CORBARepository(corba,node,user,pass,pid,exename)
    {}

    virtual ~TECORBARepository  ()
    {}

#ifndef CYRANO_TP_STANDALONE

    virtual int MakeActualTestRun(const string& TestRunString,const string& TestName)
    {
        m_TestRun=GetParentTestRunFromString(TestRunString);
        return 0;
    }

    virtual CECOrbTestRun*    CreateParentTestRuns(const TestRunNameList_t& TestRunNameList,const string& TestRunString)
    {
        //for each Parent EXCEPT the last one create a std read-only test run
        //for the last one attach to the Registered CORBA object and create Parent
        //from that and return
        if(TestRunNameList.size()<1)
            throw mdbParseException("Tyring to create Parent TestRuns with no parents");

        CECOrbFolder* Parent=m_TestStore;//first Parent is the TestStore
        TestRunNameList_t::const_iterator i;
        TestRunNameList_t::const_iterator next_i;
        i=TestRunNameList.begin();
        next_i=i;

        while(true)
        {
            if(++next_i==TestRunNameList.end())break;   //If the next one is last finish now

            Parent=new CECOrbTestRun(Parent,i->c_str());//assign new parent
            m_ParentTestRunList.push_back(Parent);      //store for later deletion
            i++;
        }

        // Retrieve CORBA Object
        CORBA::Object_ptr obj;
        obj = pCServer->FindObjectWithName(CyrStTestRun::CLASS_NAME,TestRunString.c_str());
        if( CORBA::is_nil(obj) )
        {
            throw mdbParseException("Parent CORBA TestRun object does not exist");
        }

        CyrStTestRun_var l_TestRun = CyrStTestRun::_narrow(obj);
        CECOrbTestRun* RealParent=new CECOrbTestRun(Parent,l_TestRun);//Make immeadiate parent out of CORBA Object

        return RealParent;
    }
#endif//#ifndef CYRANO_TP_STANDALONE
}
;
#endif
