/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
///////////////////////////////////////////////////////////////////////////////
// @doc     TestInit
// @module  TestInit| This command line utility may be used to start, monitor,
//          stop and kill tests.
//
///////////////////////////////////////////////////////////////////////////////
#include <vector>
#include <exception>
#include <iostream>
#include "TestInitiator.h"
#include <TestExecuter.h>
#include "Repository\CyrStRepository.h"
#include <CyrHelper.inl>
using namespace std;
void usage();
void usage()
{
    std::cout<<"usage:  TestInit -start -n node -P project_name -T test_name"<<std::endl<<
               "        TestInit -monitor"<<std::endl<<
               "        TestInit -stop"<<std::endl<<
               "        TestInit -kill"<<std::endl;
}

int main(int argc, char * argv[])
{
    std::string node="";
    std::string Project="";
    std::string Test="";
    std::string Rnode="";
    std::string Ruser="";
    std::string Rpass="";
    bool start=false;
    bool stop=false;
    bool kill=false;
    bool killing=false;
    bool monitor=false;

    for(int i=1;i<argc;i++)
    {
        if(strcmp(argv[i],"-n")==0)             // Node to run on
        {
            if(i+1<argc)node=argv[++i];
        }
        else if(strcmp(argv[i],"-P")==0)        // Project name
        {
            if(i+1<argc)Project=argv[++i];
        }
        else if(strcmp(argv[i],"-T")==0)        // Test name
        {
            if(i+1<argc)Test=argv[++i];
        }
        else if(strcmp(argv[i],"-Rn")==0)       // Repository node
        {
            if(i+1<argc)Rnode=argv[++i];
        }
        else if(strcmp(argv[i],"-Ru")==0)       // Repository User
        {
            if(i+1<argc)Ruser=argv[++i];
        }
        else if(strcmp(argv[i],"-Rp")==0)       // Repository Password
        {
            if(i+1<argc)Rpass=argv[++i];
        }
        else if(strcmp(argv[i],"-start")==0)    // Want to start a test
        {
            start=true;
        }
        else if(strcmp(argv[i],"-stop")==0)     // Want to stop a test
        {
            stop=true;
        }
        else if(strcmp(argv[i],"-kill")==0)     // Want to kill a test
        {
            kill=true;
        }
        else if(strcmp(argv[i],"-monitor")==0)  // Want to monitor a test
        {
            monitor=true;
        }
        else
        {
            usage();
            return 1;
        }
    }
    try
    {
        // If 'Stop Test' requested - prompt for Test Manager to stop and stop it.
        if (stop)
        {
            CORBA::Object_var obj;
            TestManager_var TM;
            CORBAServer* corba=new CORBAServer ( argc, argv );
            CORBAServer::NameList_t TMList=corba->GetListOfNames(TestManager::CLASS_NAME);
            std::cout<<"There are "<<TMList.size()<<" TestManagers running"<<std::endl; 
            for (std::vector<std::string>::size_type iter=0;iter<TMList.size();iter++)  
            {                                                                           
		        std::cout<<iter<<". "<<TMList[iter].Id<<std::endl;                      
            }
            std::cout<<"Choose TestManager to Stop"<<std::endl;
            int number;
            char num=getc(stdin);
            number=atoi(&num);
            if(number>TMList.size())
            {
                std::cout<<num<<" is invalid"<<std::endl;
            }
            else
            {
                obj=corba->FindObjectWithName(TestManager::CLASS_NAME, TMList[number].Id.c_str()); 
                TM=TestManager::_narrow(obj.inout());   
                CORBA::String_var err;
                std::cout<<"Stopping "<< TMList[number].Id<<std::endl<<std::endl;   
                TM->Stop();
            }
            delete corba;
        }

        // If 'Kill Test' requested - prompt for Test Manager to kill and kill it.
        else if (kill)
        {
            CORBA::Object_var obj;
            TestManager_var TM;
            CORBAServer* corba=new CORBAServer ( argc, argv );
            CORBAServer::NameList_t TMList=corba->GetListOfNames(TestManager::CLASS_NAME);
            std::cout<<"There are "<<TMList.size()<<" TestManagers running"<<std::endl; 
            for (std::vector<std::string>::size_type iter=0;iter<TMList.size();iter++)  
            {                                                                           
		        std::cout<<iter<<". "<<TMList[iter].Id<<std::endl;                      
            }
            std::cout<<"Choose TestManager to Kill"<<std::endl;
            int number;
            char num=getc(stdin);
            number=atoi(&num);
            if(number>TMList.size())
            {
                std::cout<<num<<" is invalid"<<std::endl;
            }
            else
            {
                obj=corba->FindObjectWithName(TestManager::CLASS_NAME, TMList[number].Id.c_str()); 
                TM=TestManager::_narrow(obj.inout());   
                CORBA::String_var err;
                std::cout<<"Killing "<< TMList[number].Id<<std::endl;
                killing=true;
                TM->Kill();
            }
            delete corba;
        }

        // Else if 'Start Test' requested, start a test using the parameters provided
        // on the command line.
        else if(start)
        {
            if(node=="")
            {
                usage();
                return 1;
            }
            if(Project=="")
            {
                usage();
                return 1;
            }
            if(Test=="")
            {
                usage();
                return 1;
            }
            if(Rnode=="")
            {
                CECInetInfo info;
                Rnode=info.GetHostName();
            }
            if(Ruser=="")
            {
                Ruser=CyrGetCurrentUser();
            }
            //if(Rpass==""){usage();return 1;}
            CECNodeDefinition nodedef(node);
            TestInitiator* tinit;

            // Creating new Test Initiator
            std::cout<<"Initializing Test System..."<<std::endl;
            tinit=new TestInitiator(Project.c_str(),Test.c_str(),nodedef);

            // And proceed with initialization if successful
            if (tinit != NULL)
            {
                // Preparing repository info
                std::cout<<"Configuring Test System..."<<std::endl;
                tinit->SetRepositoryInformation(Rnode.c_str(),Ruser.c_str(),Rpass.c_str());

                // Create new Test Manager.
                std::cout<<"Initializing Test Management..."<<std::endl;
                tinit->CreateTestManager();

                // Start test.
                std::cout<<"Starting Test..."<<std::endl;
                tinit->ExecuteTest();
                std::cout<<"Test started"<<std::endl;
                ::Sleep(500);   // Delay to allow above messages to be read 

				TestManager_var TM = tinit->GetTestManager();
				CORBA::Object_var  obj=TM->GetTestRunAsObject();
				CyrStTestRun_var m_TestRun=CyrStTestRun::_narrow(obj);
				std::cout<<m_TestRun->GetName()<<std::endl;


                delete tinit;
            }
            else
            {
                // Report an error
                std::cout<<"Failed to Start Test."<<std::endl;
                ::Sleep(500);   // Delay to allow above messages to be read 
            }
        }

        // Else if 'Monitor Test' requested, output test data to standard output.
        else if (monitor)
        {
            TestManager_var TM;
            TestLevel_var TL;
            CORBAServer* corba=new CORBAServer ( argc, argv );
            CORBA::Object_var obj;
            std::cout<<std::endl<<"GetList of TestManagers"<<std::endl;
            CORBAServer::NameList_t TMList=corba->GetListOfNames(TestManager::CLASS_NAME);  
            for(int i=0;i<TMList.size();i++)    
            {
                string TMname = TMList[i].Id;   
                std::cout<<TMname<<std::endl<<std::endl;
                obj=corba->FindObjectWithName(TestManager::CLASS_NAME, TMname.c_str()); 
                TM=TestManager::_narrow(obj.inout());   
                std::cout<<"TestManager Name:" << TM->Name() <<std::endl;
                std::cout<<"TestManager PID:" << TM->GetPID() <<std::endl;
                std::cout<<"NodeName:" << TM->GetNodeName() <<std::endl;
                std::cout<<"Test Cases Passed:" << TM->GetTotalTestCasePasses() <<std::endl;
                std::cout<<"Test Cases Failed:" << TM->GetTotalTestCaseFails() <<std::endl<<std::endl;
                TestLevelList TestList;
                try
                {
                    TL=TM->GetTestLevel();
                    std::cout<<"TestName:" << TL->TestName() <<std::endl;
                    std::cout<<"StartTime:" << TL->StartTime() <<std::endl;
                    std::cout<<"NodeName:" << TL->GetNode() <<std::endl;
                    std::cout<<"Contains "<<TL->NoChildTestLevels(false)<<" TestLevels"<<std::endl;
                    std::cout<<"Contains "<<TL->NoChildThreads(false)<<" TestThreads"<<std::endl<<std::endl;
                    TestExecuterList TEList;
                    CORBA::ULong no_executers=TL->GetTestExecuterList(TEList);
                    std::cout<<no_executers<<" TestExecuters Counted"<<std::endl;
                    {
                        for(long TEiter=0;TEiter<TEList.length();TEiter++)
                        {
                            std::cout<<"TestExecuter:"<<TEList[TEiter]->Name()<<std::endl;
                            ThreadInfoList TIList;
                            CORBA::ULong no_threads=TEList[TEiter]->GetThreadInformation(TIList);
                            for(long TIiter=0;TIiter<TIList.length();TIiter++)
                            {
                                std::cout<<"Thread:"<<(const char*)TIList[TIiter].UIdent<<
                                ","<<TIList[TIiter].id<<","<<(const char*)TIList[TIiter].ScriptName<<
                                ","<<TIList[TIiter].StartTime<<
                                ","<<(const char*)TIList[TIiter].NoteText<<","<<TIList[TIiter].Status<<std::endl;

                            }
                        }
                    }
                    unsigned long no_tests=TL->GetChildTestLevelList(TestList);
                }
                catch (const CyrClosingDownException& )
                {
                    std::cout<<"Test Level Closing Down"<<std::endl;
                    break;
                }
                catch(...)
                {
                    break;
                }

                for(long TLiter=0;TLiter<TestList.length();TLiter++)
                {
                    TestLevel_var  STL=TestList[TLiter];
                    std::cout<<"SubTestName:" << STL->TestName() <<std::endl;
                    std::cout<<"StartTime:" << STL->StartTime() <<std::endl;
                    std::cout<<"Node:" << STL->GetNode() <<std::endl;
                    std::cout<<"Contains "<<STL->NoChildTestLevels(false)<<" TestLevels"<<std::endl;
                    std::cout<<"Contains "<<STL->NoChildThreads(false)<<" TestThreads"<<std::endl<<std::endl;
                    TestExecuterList TEList;
                    CORBA::ULong no_executers=STL->GetTestExecuterList(TEList);
                    std::cout<<no_executers<<" TestExecuters Counted"<<std::endl;
                    for(long TEiter=0;TEiter<TEList.length();TEiter++)
                    {
                        std::cout<<"TestExecuter:"<<TEList[TEiter]->Name()<<std::endl;
                        ThreadInfoList TIList;
                        CORBA::ULong no_threads=TEList[TEiter]->GetThreadInformation(TIList);
                        for(long TIiter=0;TIiter<TIList.length();TIiter++)
                        {
                            std::cout<<"Thread:"<<(const char*)TIList[TIiter].UIdent<<
                            ","<<TIList[TIiter].id<<","<<(const char*)TIList[TIiter].ScriptName<<
                            ","<<TIList[TIiter].StartTime<<std::endl;

                        }
                    }

                }

            }
        }
        else
        {
            usage();
            return 1;
        }
    }
    catch (const CyrClosingDownException& )
    {
        std::cout<<"Test Closing Down Monitoring to Stop"<<std::endl;
    }
    catch (const CyrProcessException& e)
    {
        std::cout<<"A CyrProcessException "<<string(e.what)<<", "<<string(e.where)<<" status " <<e.status<<std::endl;
    }
    catch (CyrRepositoryException e)
    {
        std::cout<<"A CyrRepositoryException "<<e.what()<<", "<<e.where()<<" status "<<e.status()<<std::endl;
    }
    catch (CORBA::Exception &e)
    {
        // If we were in the process of killing a test, don't report CORBA exceptions.
        if (!killing) std::cout<<"A CORBA::"<<e<<std::endl;
    }
    catch (CyrException & e)
    {
        std::cout<<"A CyrException "<<e.what()<<"," <<e.where()<<e.status()<<std::endl;
    }
    catch (exception& e)
    {
        std::cout<<"An exception "<<e.what()<<std::endl;
    }
    catch(...)
    {
        std::cout<<"Unknown"<<std::endl;
    }

    // If we issued a 'kill', report that the kill is complete.
    if (killing) std::cout<<"Kill complete"<<std::endl;

    return 0;
}
