#ifndef PSL__LIST_CTXDEF_LOADED
#define PSL__LIST_CTXDEF_LOADED 1
/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*============================================================================*/
/*                                        */
/*               **** C O M M O N ****                    */
/*                                        */
/*============================================================================*/
/*                                        */
/*          Module Name: LIST_CTXDEF                  */
/*          ============                          */
/*                                        */
/*  Description:                                  */
/*  ============                                  */
/*                                        */
/*  This modules defines the format of the standard listing routines      */
/*  context block which is used by the listing routines to allow          */
/*  a utility to have multiple listing files.                 */
/*                                        */
/*============================================================================*/

/*** MODULE LIST_CTXDEF ***/


/*    The LIST_CTX structure contains the general "Context block Data" */
/*    associated with the Listing routines, such as page information: */
/*    the page number, the current lines on this page, the maximum */
/*    page size; and listing file specifics, such as header section */
/*    information, listing file name and spooling information, and */
/*    the like. */

struct list_ctx
{
    char                    ftypc4[4];    /* File Type */
    PSL_LONGWORD_T          fvrnl;    /* Listing routine Version */
#define            LIST_FVRN 0X23   /* Listing version */

    /*    Flag to show listing initialized */

    Boolean                 list_initn;   /* Listing initialized */

    /*    Print file expanded file specification. */

    char                    prtfilc256[256];  /* Print file filename */

    /*    Print file specification as specified on /LIST. */

    char                    lstvalc256[256];  /* Value on /LIST */

    /*    Pointer to the file structure for this listing file */

    FILE                    *filepf;  /* Pointer to file structure */

    /*    Utility name through which the listing has been generated, as */
    /*    output in each listing page header. */

    char                    utlnamc8[8];  /* Utility name */

    /*    Length of the Utility name. */

    PSL_LONGWORD_T          utlnam_lenl;  /* Length of UTLNAM */

    /*    Length of the Utility version number ("Vn.m"). */

    PSL_LONGWORD_T          lstver_lenl;  /* Length of LSTVER */

    /*    Utility version number ("Vn.m"), as output in each listing page header. */

    char                    lstverc8[8];  /* Version in listing */

    /*    Length of the Date/Time field. */

    PSL_LONGWORD_T          lsttim_lenl;  /* Length of LSTTIM */

    /*    Sub-heading lines (6 in all), as output in each listing page header. */
    /*    Sub-header lines 3 through 6 are 'Floating', and may be set as and */
    /*    when required. */

    char                    subhdgc256_6[6][256]; /* Subheading Text */

    /*    Reference number of current line. This is only applicable when */
    /*    listing "numbered" lines. */

    PSL_LONGWORD_T          ln_refnol;    /* Reference no of line */

    /*    Current page number. */

    PSL_LONGWORD_T          page_nol; /* Current page number */

    /*    Number of lines output to the current page. */

    PSL_LONGWORD_T          ln_on_pagl;   /* Current lines on page */

    /*    Maxumum permissible lines per page. */

    PSL_LONGWORD_T          max_pag_sizl; /* Max lines per page */

    /*    Main heading line as output at the top of each listing line. */
    /*    There are 80 characters available for this heading but we round the size */
    /*    up to 84 to allow for a null terminator and alignment */

    char                    headingc84[84];

    /*    Logical Unit Number through which the Listing file is accessed. */

    PSL_LONGWORD_T          unit_nol; /* Print file unit no   */

    /*    Spool flag to identify whether the listing file should, or should not */
    /*    be spooled directly to the print queue. */

    Boolean                 spool_qualn;  /* Setting of /SPOOL    */

    /*     Recursive Error trapping flag for LIST_OUTPUT_ERROR. This is used */
    /*     to prevent errors from being intercepted and listed whilst in the */
    /*     middle of listing and error! */

    PSL_LONGWORD_T          err_trappingl;    /* Error recursion Flag */
#define            ERT_ENABLED 1    /* Trapping enabled  */
#define            ERT_DISABLED 2   /* Trapping disabled */

    /*    Flag to indicate if last output was an error message, and hence, */
    /*    if a "spacer" blank line should be listed. */

    PSL_LONGWORD_T          last_errl;    /* Previous Line status */
#define            ERR_YES 1    /* Last o/p was error */
#define            ERR_NO 2 /* Last o/p not error */

    /*    Diagnostic stack pointer */

    struct diagstk          *diag_ptrps;
    /* Diagnostic Stack Pointer */

    /*    A "Filler" to pad out to the next page boundary, and to allow for */
    /*    further expansion. */

    char                    fill1b348[348];   /* Pad to next page */

} ;


/*  End of C Include File */

#endif  /* PSL__LIST_CTXDEF_LOADED */
