/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECServerConfigDlg.cpp : implementation file
//

#include "stdafx.h"
#include "daemoncfg.h"
#include "mainfrm.h"
#include "CECServerConfigDlg.h"
#include "CECRegistryTools.h"
#include "..\include\Solutions.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

///////////////////////////////////////////////////////////////////////////////
int CALLBACK BrowseCallback(HWND hwnd,UINT uMsg,LPARAM lp, LPARAM /*pData*/)
{
    TCHAR szDir[MAX_PATH];

    switch(uMsg)
    {
    case BFFM_INITIALIZED:
        {
            if (GetCurrentDirectory(sizeof(szDir)/sizeof(TCHAR), szDir))
            {
                // WParam is TRUE since you are passing a path.
                // It would be FALSE if you were passing a pidl.
                SendMessage(hwnd,BFFM_SETSELECTION,TRUE,(LPARAM)szDir);
            }
            break;

        }
    case BFFM_SELCHANGED:
        {
            // Set the status window to the currently selected path.
            if (SHGetPathFromIDList((LPITEMIDLIST) lp ,szDir))
            {
                SendMessage(hwnd,BFFM_SETSTATUSTEXT,0,(LPARAM)szDir);
            }
            break;
        }
    default:
        {
            break;
        }
    }
    return 0;
}
/////////////////////////////////////////////////////////////////////////////
// CECServerConfigDlg dialog


CECServerConfigDlg::CECServerConfigDlg(BOOL RuntimeOnly ,CWnd* pParent /*=NULL*/)
        : m_bRunNamesService (RuntimeOnly) ,CDialog(CECServerConfigDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECServerConfigDlg)
    m_WorkDir = _T("");
    m_Host = _T("");
    m_NameService = _T("49201");
    m_Port = _T("1250");
    m_bTrace = FALSE;
    m_bNotify = TRUE;
    //}}AFX_DATA_INIT

    m_bModified = FALSE;
}


void CECServerConfigDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECServerConfigDlg)
    DDX_Text(pDX, IDC_FOLDER_NAME, m_WorkDir);
    DDX_Text(pDX, IDC_HOST, m_Host);
    DDX_Check(pDX, IDC_TRACE, m_bTrace);
    DDX_Check(pDX, IDC_NOTIFY, m_bNotify);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECServerConfigDlg, CDialog)
//{{AFX_MSG_MAP(CECServerConfigDlg)
ON_BN_CLICKED(IDC_TRACE, OnModify)
ON_EN_UPDATE(IDC_HOST, OnModify)
ON_EN_UPDATE(IDC_FOLDER_NAME, OnModify)
ON_BN_CLICKED(IDC_NOTIFY, OnModify)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECServerConfigDlg message handlers


BOOL CECServerConfigDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    // Reset the modified flag
    m_bModified = FALSE;

    //
    //    Set up the initial values from the registry
    //
    CECRegistryTools reg;
    CString KeyName ,ValueName ,KeyValue;

    KeyName = SOLUTIONS_OMNIORB_KEY;
    ValueName = "ORBInitialHost";
    m_Host = reg.GetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName);

    KeyName = CURRENT_PROJECT_KEY ;
    ValueName = "Path";
    m_WorkDir = reg.GetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName);

    KeyName = CURRENT_PROJECT_KEY ;
    ValueName = "Tracing";
    CString szTrace = reg.GetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName);
    m_bTrace = (szTrace.CompareNoCase("1") == 0) ? 1 : 0;

    KeyName = CURRENT_PROJECT_KEY ;
    ValueName = "Notify";
    CString szNotify = reg.GetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName);
    m_bNotify = (szNotify.CompareNoCase("1") == 0) ? 1 : 0;


    //
    // ...then ServerAddress
    ValueName = "ServerAddress";
    m_Host = reg.GetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName);

    //
    // ...then ServerPort
    ValueName = "ServerPort";
    m_Port = reg.GetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName);

    KeyName = SOLUTIONS_OMNIORB_KEY;
    ValueName = "ORBInitialPort";
    m_NameService = reg.GetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName);

    //
    // And set up the trace option in the dialog
    CButton* pTrace = (CButton*) GetDlgItem(IDC_TRACE);
    ASSERT (pTrace != NULL);

    pTrace->SetCheck(m_bTrace);

    //
    // And set up the notify option in the dialog
    CButton* pNotify = (CButton*) GetDlgItem(IDC_NOTIFY);
    ASSERT (pNotify != NULL);

    pNotify->SetCheck(m_bNotify);

    UpdateData (FALSE);

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}




void CECServerConfigDlg::OnOK()
{
    UpdateData (TRUE);

    CEdit* pWorkingFolder = (CEdit*) GetDlgItem (IDC_FOLDER_NAME);
    ASSERT (pWorkingFolder != NULL);

    CString InitialFolder;
    pWorkingFolder->GetWindowText(InitialFolder);

    //remove any dodgy characters
    if(InitialFolder.GetAt(InitialFolder.GetLength()-1) != '\\')
        InitialFolder += "\\";

    //Check if the folder exists...
    CFileStatus fs;
    if(!CFile::GetStatus(InitialFolder+".", fs))
    {
        //if it doesn't ask whether to create it or not
        CString strMessage;
        AfxFormatString1(strMessage,IDS_CREATE_NEW_FOLDER,InitialFolder);
        if(AfxMessageBox(strMessage,MB_YESNO)==IDNO)
            return;

		char szCurDir[MAX_PATH];
		GetCurrentDirectory(MAX_PATH,szCurDir);

		char *pszToken= strtok(InitialFolder.GetBuffer(0),"\\");
		while(pszToken)
		{
			
			CreateDirectory(pszToken,NULL);
			SetCurrentDirectory(pszToken);

			pszToken= strtok(NULL,"\\");
		}
		SetCurrentDirectory(szCurDir);
    }

    //
    //    Ensure that we have values for the other fields.
    //
    if (m_Host.IsEmpty())
    {
        CString strError;
        VERIFY(strError.LoadString(IDS_ERR_SERVERADDRESS));
        AfxMessageBox(strError);
        return;
    }

    //
    //    And that it is not LocalHost
    //
    if (m_Host.CompareNoCase("LOCALHOST") == 0)
    {
        //
        // If so - then lets use the current Computer Name
        CECRegistryTools reg;
        CString szKeyName = "SYSTEM\\CurrentControlSet\\Control\\ComputerName\\ActiveComputerName";

        m_Host = reg.GetStringValue(HKEY_LOCAL_MACHINE ,szKeyName ,"ComputerName");

        //
        // And winge if it's unobtainable for some strange reason
        if (m_Host.IsEmpty())
        {
            CString strError;
            VERIFY(strError.LoadString(IDS_ERR_LOCALHOST));
            AfxMessageBox(strError);

            UpdateData(FALSE);
            return;
        }
    }


    //
    // If anything has changed then we should inform inform the user to restart
    // the daemon to pick up this new configuration
    if (m_bModified)
    {
        AfxMessageBox(IDS_RESTART_DAEMON);
    }

    //
    // Get the state of the Trace Check Box
    CButton* pTrace = (CButton*) GetDlgItem(IDC_TRACE);
    ASSERT (pTrace != NULL);

    m_bTrace = pTrace->GetCheck();

    //
    // Get the state of the Notify Check Box
    CButton* pNotify = (CButton*) GetDlgItem(IDC_NOTIFY);
    ASSERT (pNotify != NULL);

    m_bNotify = pNotify->GetCheck();

    //
    // ...then save the entries back into the registry
    //
    CECRegistryTools reg;
    CString KeyName ,ValueName ,KeyValue;

    KeyName = CURRENT_PROJECT_KEY;
    ValueName = "Path";
    reg.SetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName ,m_WorkDir.GetBuffer(0));

    KeyName = CURRENT_PROJECT_KEY;
    ValueName = "Tracing";
    CString szTrace = (m_bTrace) ? "1" : "0";
    reg.SetStringValue(HKEY_CURRENT_USER ,KeyName ,ValueName, szTrace.GetBuffer(0));

    KeyName = CURRENT_PROJECT_KEY;
    ValueName = "Notify";
    CString szNotify = (m_bNotify) ? "1" : "0";
    reg.SetStringValue(HKEY_CURRENT_USER ,KeyName ,ValueName, szNotify.GetBuffer(0));

    //
    //    ...then ServerAddress
    //
    ValueName = "ServerAddress";
    reg.SetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName ,m_Host.GetBuffer(0),HKEY_LOCAL_MACHINE);

    KeyName = SOLUTIONS_OMNIORB_KEY;
    ValueName = "ORBInitialHost";
    reg.SetStringValue(HKEY_LOCAL_MACHINE ,KeyName ,ValueName ,m_Host.GetBuffer(0), HKEY_LOCAL_MACHINE);

    CDialog::OnOK();
}

//
// OnModify
// ========
//
// Called when the user turns the tracing on/off, or starts to edit the edit fields
void CECServerConfigDlg::OnModify()
{
    // Simply Flag that something in this dialog has changed
    m_bModified = TRUE;

}

