/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CORBAInfo.cpp: implementation of the CCORBAInfo class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "testplugin.h"
#include "CORBAInfo.h"
#include <string>
#include <vector>
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CCORBAInfo::CCORBAInfo()
{
	m_pCorba = new CORBAServer ( 0, NULL );
}

CCORBAInfo::~CCORBAInfo()
{
	try
	{
		delete m_pCorba;
	}
	catch(...)
	{
	}
}

BOOL CCORBAInfo::IsTestActive(CString TestName, CString& strReg)
{
	try
	{
		CORBAServer::NameList_t TMList=m_pCorba->GetListOfNames(TestManager::CLASS_NAME);
		for (std::vector<std::string>::size_type iter=0;iter<TMList.size();iter++)  
		{                                                                           
			CORBA::Object_var obj;
			obj=m_pCorba->FindObjectWithName(TestManager::CLASS_NAME, TMList[iter].Id.c_str()); 
		    TestManager_var TM=TestManager::_narrow(obj.inout());
			TestLevel_var TL = TM->GetTestLevel();

			string strTest = TL->TestName();
			if(TestName.CompareNoCase(strTest.c_str()) == 0)
			{
				strReg = TMList[iter].Id.c_str();
				return true;
			}

		}
	}
	catch(...)
	{
	}
	return false;
}

void CCORBAInfo::StopTest(CString strTest)
{
	try
	{
		CORBAServer::NameList_t TMList=m_pCorba->GetListOfNames(TestManager::CLASS_NAME);
		for (std::vector<std::string>::size_type iter=0;iter<TMList.size();iter++)  
		{                                                                           
			CORBA::Object_var obj;
			obj=m_pCorba->FindObjectWithName(TestManager::CLASS_NAME, TMList[iter].Id.c_str()); 
		    TestManager_var TM=TestManager::_narrow(obj.inout());
			TestLevel_var TL = TM->GetTestLevel();
			string TestName = TL->TestName();

			if(strTest.CompareNoCase(TestName.c_str()) == 0)
			{
				TM->Stop();
				return;
			}

		}
	}
	catch(...)
	{
	}
}

void CCORBAInfo::KillTest(CString strTest)
{
	try
	{
		CORBAServer::NameList_t TMList=m_pCorba->GetListOfNames(TestManager::CLASS_NAME);
		for (std::vector<std::string>::size_type iter=0;iter<TMList.size();iter++)  
		{                                                                           
			CORBA::Object_var obj;
			obj=m_pCorba->FindObjectWithName(TestManager::CLASS_NAME, TMList[iter].Id.c_str()); 
		    TestManager_var TM=TestManager::_narrow(obj.inout());
			TestLevel_var TL = TM->GetTestLevel();
			string TestName = TL->TestName();

			if(strTest.CompareNoCase(TestName.c_str()) == 0)
			{
				TM->Kill();
				return;
			}

		}
	}
	catch(...)
	{
	}
}

void CCORBAInfo::GetActiveTests(CStringArray arrTests)
{
	try
	{
		CORBAServer::NameList_t TMList=m_pCorba->GetListOfNames(TestManager::CLASS_NAME);
		for (std::vector<std::string>::size_type iter=0;iter<TMList.size();iter++)  
		{                                                                           
			CORBA::Object_var obj;
			obj=m_pCorba->FindObjectWithName(TestManager::CLASS_NAME, TMList[iter].Id.c_str()); 
		    TestManager_var TM=TestManager::_narrow(obj.inout());
			TestLevel_var TL = TM->GetTestLevel();

			string strTest = TL->TestName();
			arrTests.Add(strTest.c_str());
		}
	}
	catch(...)
	{
	}
}

void CCORBAInfo::GetSummaryInfo(CString strManagerReg, SUMMARY_INFO& arrSummary)
{
	try
	{
		CORBA::Object_var obj;
        obj=m_pCorba->FindObjectWithName(TestManager::CLASS_NAME, strManagerReg); 
        TestManager_var TM=TestManager::_narrow(obj.inout());
		TestLevel_var TL = TM->GetTestLevel();

		// update information about child levels
		TestLevelList TLList;
		long nTLS = TL->GetChildTestLevelList(TLList);
		long TLiter;
		TestLevel_var CL;
	    for(TLiter=0;TLiter<TLList.length();TLiter++)
	    {
			CL = TLList[TLiter];
			SUMMARY_MAP sMap;
			sMap.strTaskGroup = CL->TestName();
			sMap.nUsers = CL->NoChildThreads(true);
			arrSummary.Add(sMap);
		}
	}
	catch(...)
	{
	}
}

