/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ConfigurationDlg.cpp : implementation file
//

#include "stdafx.h"
#include "TestPlugin.h"
#include "ConfigurationDlg.h"
#include "OstaDefs.h"
#include "TestFile.h"
#include "RegFunctions.h"
#include "TestPluginDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CConfigurationDlg dialog
CConfigurationDlg::CConfigurationDlg(CWnd* pParent /*=NULL*/)
	: CDragDropDialog(CConfigurationDlg::IDD, pParent)
{
	CECGridCtrl::GridCtrlInit(AfxGetInstanceHandle());
	//{{AFX_DATA_INIT(CConfigurationDlg)
	m_strDescription = _T("");
	//}}AFX_DATA_INIT
	m_pTestFile = NULL;
	m_pDoc = NULL;
}

void CConfigurationDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CConfigurationDlg)
	DDX_Control(pDX, IDC_DESCRIPTION, m_ctlTestDescription);
	DDX_Control(pDX, IDC_TESTS_LIST, m_ctlTestList);
	DDX_Text(pDX, IDC_DESCRIPTION, m_strDescription);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CConfigurationDlg, CDialog)
	//{{AFX_MSG_MAP(CConfigurationDlg)
	ON_WM_SIZE()
	ON_COMMAND(ID_DELETESELECTION, OnDeleteselection)
	ON_UPDATE_COMMAND_UI(ID_DELETESELECTION, OnUpdateDeleteselection)
	ON_EN_CHANGE(IDC_DESCRIPTION, OnChangeDescription)
	//}}AFX_MSG_MAP
	ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CConfigurationDlg message handlers

void CConfigurationDlg::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);
	
	if(m_ctlTestDescription.GetSafeHwnd() && m_ctlTestList.GetSafeHwnd())
	{
		CRect rcDesc;
		m_ctlTestDescription.GetWindowRect(&rcDesc);
		ScreenToClient(&rcDesc);

		m_ctlTestDescription.SetWindowPos(NULL,0,0,cx-rcDesc.left-5,rcDesc.Height(),SWP_NOZORDER|SWP_NOMOVE);

		m_ctlTestList.MoveWindow(0,rcDesc.bottom + 5 ,cx,cy-rcDesc.bottom-5);
	}
}

BOOL CConfigurationDlg::OnEraseBkgnd(CDC *pDC)
{
	return CDialog::OnEraseBkgnd(pDC);
}

BOOL CConfigurationDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	m_strDescription = m_pTestFile->GetDescription();	

	UpdateData(FALSE);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

DROPEFFECT CConfigurationDlg::OnDragEnter(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point)
{
	return DROPEFFECT_COPY;
}

void CConfigurationDlg::OnDragLeave()
{
}

DROPEFFECT CConfigurationDlg::OnDragOver(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point)
{
	CRect rcTestList;
	m_ctlTestList.GetWindowRect(&rcTestList);
	ScreenToClient(&rcTestList);

	if(rcTestList.PtInRect(point))
	{
		CPoint pt(point);
		pt.y -= rcTestList.top;
		
		return m_ctlTestList.OnDragOver(pDataObject,dwKeyState,pt);
	}

	return DROPEFFECT_NONE;
}

BOOL CConfigurationDlg::OnDrop(COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint point)
{
	CRect rcTestList;
	m_ctlTestList.GetWindowRect(&rcTestList);
	ScreenToClient(&rcTestList);

	if(rcTestList.PtInRect(point))
	{
		CPoint pt(point);
		pt.y -= rcTestList.top;
		
		return m_ctlTestList.OnDrop(pDataObject,dropEffect,pt);
	}

	return FALSE;
}

void CConfigurationDlg::GenerateScripts()
{
	m_ctlTestList.GenerateScripts();
}

void CConfigurationDlg::SetDocument(CTestPluginDoc *pDoc)
{
	m_pDoc = pDoc;
	m_pTestFile = pDoc->GetTestFile();
	m_ctlTestList.SetDocument(pDoc);
}

BOOL CConfigurationDlg::PreTranslateMessage(MSG* pMsg) 
{
	switch(pMsg->message)
	{
	case WM_KEYDOWN:
		switch(pMsg->wParam)
		{
		case VK_DELETE:
			m_ctlTestList.DeleteSelection();
		break;
		}
	break;
	}
	
	return CDialog::PreTranslateMessage(pMsg);
}

void CConfigurationDlg::OnDeleteselection() 
{
	m_ctlTestList.DeleteSelection();	
}

void CConfigurationDlg::OnUpdateDeleteselection(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_ctlTestList.GetSelectedCount()?TRUE:FALSE);	
}

void CConfigurationDlg::OnChangeDescription() 
{
	UpdateData(TRUE);
	m_pTestFile->SetDescription(m_strDescription);	
	m_pDoc->SetModifiedFlag();
}

void CConfigurationDlg::OnOK() 
{
}

void CConfigurationDlg::OnCancel() 
{
}
