/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TestPluginView.cpp : implementation of the CTestPluginView class
//

#include "stdafx.h"
#include "TestPlugin.h"
#include "TestExecuteDlg.h"
#include "SCLCompiler.h"
#include "CompileDlg.h"
#include "RegFunctions.h"

#include "TestPluginDoc.h"
#include "TestPluginView.h"
#include "CORBAInfo.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define IDC_TABS 0x1000
static CString LOGFILE = "Active.log";
#define IDT_FIND_TEST 1
/////////////////////////////////////////////////////////////////////////////
// CTestPluginView
IMPLEMENT_DYNCREATE(CTestPluginView, CView)

BEGIN_MESSAGE_MAP(CTestPluginView, CView)
	//{{AFX_MSG_MAP(CTestPluginView)
	ON_COMMAND(ID_CANCEL_EDIT_SRVR, OnCancelEditSrvr)
	ON_WM_ERASEBKGND()
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_COMMAND(ID_EXECUTE_TEST, OnExecuteTest)
	ON_COMMAND(ID_COMPILE, OnCompile)
	ON_COMMAND(ID_STOP_TEST, OnStopTest)
	ON_UPDATE_COMMAND_UI(ID_EXECUTE_TEST, OnUpdateExecuteTest)
	ON_UPDATE_COMMAND_UI(ID_STOP_TEST, OnUpdateStopTest)
	ON_WM_TIMER()
	ON_COMMAND(ID_KILL_TEST, OnKillTest)
	ON_UPDATE_COMMAND_UI(ID_KILL_TEST, OnUpdateKillTest)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
	ON_NOTIFY(TCN_SELCHANGING,IDC_TABS,OnTabsSelChanging)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTestPluginView construction/destruction

CTestPluginView::CTestPluginView()
{
	m_bTestActive = false;
}

CTestPluginView::~CTestPluginView()
{
}

BOOL CTestPluginView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CTestPluginView drawing

void CTestPluginView::OnDraw(CDC* pDC)
{
	CTestPluginDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CTestPluginView printing

BOOL CTestPluginView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CTestPluginView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CTestPluginView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// OLE Server support

// The following command handler provides the standard keyboard
//  user interface to cancel an in-place editing session.  Here,
//  the server (not the container) causes the deactivation.
void CTestPluginView::OnCancelEditSrvr()
{
	GetDocument()->OnDeactivateUI(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// CTestPluginView diagnostics

#ifdef _DEBUG
void CTestPluginView::AssertValid() const
{
	CView::AssertValid();
}

void CTestPluginView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CTestPluginDoc* CTestPluginView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CTestPluginDoc)));
	return (CTestPluginDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTestPluginView message handlers

BOOL CTestPluginView::OnEraseBkgnd(CDC* pDC) 
{
	return TRUE;	
//	return CView::OnEraseBkgnd(pDC);
}

int CTestPluginView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if(!m_ctlTabs.Create(WS_CHILD|WS_VISIBLE,CRect(0,0,0,0),this,IDC_TABS))
		return -1;

	m_TabImages.Create(IDB_TAB_IMAGES,16,0,RGB(0,128,128));
	m_ctlTabs.SetImageList(&m_TabImages);

	m_DropTarget.Register(this);

	CFont *pFont=CFont::FromHandle((HFONT)GetStockObject(ANSI_VAR_FONT));
	m_ctlTabs.SetFont(pFont);

	return 0;
}

void CTestPluginView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);

	if(m_ctlTabs.GetSafeHwnd())
		m_ctlTabs.MoveWindow(0,0,cx,cy);
}

DROPEFFECT CTestPluginView::OnDragEnter(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point) 
{
	CRect rcClient;
	GetClientRect(&rcClient);
	m_ctlTabs.AdjustRect(false,rcClient);
	
	point.x -= rcClient.left;
	point.y -= rcClient.top;
	CDragDropDialog *pDlg = (CDragDropDialog *)m_ctlTabs.GetCurrentTab();
	if(pDlg)
		return pDlg->OnDragEnter(pDataObject,dwKeyState,point);
	else
		return DROPEFFECT_NONE;
}

void CTestPluginView::OnDragLeave() 
{
	CDragDropDialog *pDlg = (CDragDropDialog *)m_ctlTabs.GetCurrentTab();
	if(pDlg)
		pDlg->OnDragLeave();
	
//	CView::OnDragLeave();
}

DROPEFFECT CTestPluginView::OnDragOver(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point) 
{
	CRect rcClient;
	GetClientRect(&rcClient);
	m_ctlTabs.AdjustRect(false,rcClient);
	
	point.x -= rcClient.left;
	point.y -= rcClient.top;

	CDragDropDialog *pDlg = (CDragDropDialog *)m_ctlTabs.GetCurrentTab();
	if(pDlg)
		return pDlg->OnDragOver(pDataObject,dwKeyState,point);
	else
		return DROPEFFECT_NONE;
}

BOOL CTestPluginView::OnDrop(COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint point) 
{
	CRect rcClient;
	GetClientRect(&rcClient);
	m_ctlTabs.AdjustRect(false,rcClient);
	
	point.x -= rcClient.left;
	point.y -= rcClient.top;

	CDragDropDialog *pDlg = (CDragDropDialog *)m_ctlTabs.GetCurrentTab();
	if(pDlg)
		return pDlg->OnDrop(pDataObject,dropEffect,point);
	else
		return false;
}

void CTestPluginView::OnInitialUpdate() 
{
	CView::OnInitialUpdate();
	
	m_ConfigurationDlg.SetDocument(GetDocument());
	m_ConfigurationDlg.Create(CConfigurationDlg::IDD,&m_ctlTabs);

	m_ResultsDlg.SetTestFile(GetDocument()->GetTestFile());
	m_ResultsDlg.Create(CResultsDlg::IDD,&m_ctlTabs);
	
	m_MonitoringDlg.SetTestFile(GetDocument()->GetTestFile());
	m_MonitoringDlg.Create(CMonitoringDlg::IDD, &m_ctlTabs);

	m_ctlTabs.InsertItem(enConfiguration,"Configuration",enConfiguration,&m_ConfigurationDlg);
	m_ctlTabs.InsertItem(enMonitoring,"Monitoring",enMonitoring,&m_MonitoringDlg);
	m_ctlTabs.InsertItem(enResults,"Results",enResults,&m_ResultsDlg);
	
	m_ctlTabs.SetCurSel(enConfiguration);

	//Get Test Name
	CString strPath(GetDocument()->GetTestFile()->GetTestFilePath());
	int nSlashPos = strPath.ReverseFind('\\');
	strPath.Delete(0,nSlashPos+1);

	int nPos = strPath.ReverseFind('.');
	m_strTest = strPath.Left(nPos);

	OnTimer(IDT_FIND_TEST);
	if (m_bTestActive)
		SetTimer(IDT_FIND_TEST, 1000, NULL);
}

void CTestPluginView::OnExecuteTest() 
{
	//First of all generate the scripts
	m_ConfigurationDlg.GenerateScripts();

	if(!CompileScripts())
		return;

	CTestExecuteDlg init(m_strTest);
	init.DoModal();//!=IDOK)
//		return;

	//Create a Test summary file in the test run folder which 
	//has just been created...
	//...Look for the newest run in this test folder
	CString strPath(GetDocument()->GetTestFile()->GetTestFilePath());
	int nSlashPos = strPath.ReverseFind('\\');
	strPath = strPath.Left(nSlashPos+1);

	CFileFind FF;
	if(!FF.FindFile(strPath + "*.00?"))
		return;

	CTime tNewest(0);
	CString strFolder;
	BOOL bContinue = TRUE;
	while(bContinue)
	{
		bContinue = FF.FindNextFile();
		if(!FF.IsDirectory() || FF.IsDots() || FF.IsHidden())
			continue;
		
		CTime tRun;
		FF.GetCreationTime(tRun);
		if(tNewest < tRun)
		{
			tNewest = tRun;
			strFolder = FF.GetFilePath();
		}
	}	

	CStdioFile f;
	if(f.Open(strFolder + "\\" + strSummaryFile,CFile::modeCreate|CFile::modeWrite))
	{
		CString strSummary;
		GetDocument()->GetTestFile()->GetSummary(strSummary);
		f.WriteString(strSummary);
		f.Close();
	}

	//Copy configuration file to results folder
	CopyFile(GetDocument()->GetTestFile()->GetTestFilePath(),strFolder+"\\"+strResultsTestFile,TRUE);

	SetTimer(IDT_FIND_TEST, 1000, NULL);
}

BOOL CTestPluginView::CompileScripts()
{
	//Save test before compiling
	GetDocument()->SaveEmbedding();

	//Get the output directory
	CString strOutputDir = GetCompilerOutputDir();

	CTestFile *pTestFile = GetDocument()->GetTestFile();

	//Get the name of this test
	CString strTestPath(pTestFile->GetTestFilePath());

	//Get the test name without its extension
	CString strTestNoExt;
	int nDotPos = strTestPath.ReverseFind('.');
	if(nDotPos != -1)
		strTestNoExt = strTestPath.Left(nDotPos);
	
	//Get the name of the main test TSS file
	CString strTSSFile(strTestNoExt + ".TSS");
	if(!Compile(strTSSFile))
		return false;

	int nNumGroups = pTestFile->GetNumTaskGroups();
	for(int nGroup=0;nGroup<nNumGroups;nGroup++)
	{
		int nNumTasks = pTestFile->GetNumTasks(nGroup);	
		for(int nTask=0;nTask<nNumTasks;nTask++)
		{
			CTestFile::TaskType nType = pTestFile->GetTaskType(nGroup,nTask);
			CString strPath = pTestFile->GetTaskPath(nGroup,nTask);

			switch(nType)
			{
			case CTestFile::enProfile:
			{
				CString strName;
				int nSlashPos = strPath.ReverseFind('\\');
				strName = strPath.Mid(nSlashPos+1);

				//Just copy profile to objectcode directory
				if(!CopyFile(strPath,strOutputDir + strName,FALSE))
				{
					AfxMessageBox("Test execution failed.\nCould not copy profile \"" + strPath + "\" to \""+strOutputDir+"\"",MB_OK);
					return false;
				}	
			}
			break;
			case CTestFile::enScript:
			{	
				//Compile the script
				if(!Compile(strPath))
					return false;

				//There is one TSS and one SCL file for each group 
				//if this is a test so compile these as well
				CString strTSS;
				strTSS.Format(strTestNoExt + "_%d.TSS",nGroup+1);
				if(!Compile(strTSS))
					return false;

				CString strSCL;
				strSCL.Format(strTestNoExt + "_%d.SCL",nGroup+1);
				if(!Compile(strSCL))
					return false;
			}
			break;
			default:
				ASSERT(FALSE);
			break;
			}
		}
	}
	return true;
}

BOOL CTestPluginView::Compile(const CString& strPath)
{
	CSCLCompiler compiler;
	if(!compiler.CompileScript(strPath))
	{
		CCompileDlg dlg(strPath,compiler.GetErrorMessage());
		dlg.DoModal();
		return false;
	}
	return true;
}

void CTestPluginView::OnCompile() 
{
	//First of all generate the scripts
	m_ConfigurationDlg.GenerateScripts();

	if(CompileScripts())
		AfxMessageBox("Compiled successfully");
}

BOOL CTestPluginView::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo) 
{
	if(m_ctlTabs.GetCurrentTab()->OnCmdMsg(nID,nCode,pExtra,pHandlerInfo))
		return true;

	return CView::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

void CTestPluginView::OnStopTest() 
{
	//Get Test Name
	CString strPath(GetDocument()->GetTestFile()->GetTestFilePath());
	int nSlashPos = strPath.ReverseFind('\\');
	strPath.Delete(0,nSlashPos+1);

	int nPos = strPath.ReverseFind('.');
	if(nPos==-1)
	{
		return;
	}
		
	try
	{
		m_strTest = strPath.Left(nPos);
		CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
		CCORBAInfo* pInfo = pApp->GetInfo();
		pInfo->StopTest(m_strTest);
	}
	catch(...)
	{
	}
}

BOOL CTestPluginView::PreTranslateMessage(MSG* pMsg) 
{
	if(m_ctlTabs.GetCurrentTab()->PreTranslateMessage(pMsg))
		return true;
	
	return CView::PreTranslateMessage(pMsg);
}

void CTestPluginView::OnTabsSelChanging(NMHDR* pNMHDR,LRESULT* pResult)
{
	//Save file when change tabs
	GetDocument()->SaveEmbedding();
	*pResult =0;
}


void CTestPluginView::OnUpdateExecuteTest(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_bTestActive);
}

void CTestPluginView::OnUpdateStopTest(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_bTestActive);	
}

void CTestPluginView::OnTimer(UINT nIDEvent) 
{
	static int nRetries = 3;
	try
	{
		CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
		CCORBAInfo* pInfo = pApp->GetInfo();
		if (pInfo == NULL)
		{
			nRetries++;
		}
		else
		{
			CString strReg;
			BOOL bActive = pInfo->IsTestActive(m_strTest, strReg);
			if(bActive && m_bTestActive == false)
			{
				m_bTestActive = true;
				m_MonitoringDlg.SetTestActive(strReg);
				nRetries = 0;
			}
			else if (bActive && nRetries > 0)
			{
				nRetries = 0;
			}
			else if (!bActive && nRetries == 3)
			{
				KillTimer(IDT_FIND_TEST);
				m_MonitoringDlg.StopActiveTest();
				m_bTestActive = false;
			}
			else if (!bActive && nRetries < 3)
			{
				nRetries++;
			}
		}
	}
	catch(...)
	{
	}	
	CView::OnTimer(nIDEvent);
}

void CTestPluginView::OnKillTest() 
{
	//Get Test Name
	CString strPath(GetDocument()->GetTestFile()->GetTestFilePath());
	int nSlashPos = strPath.ReverseFind('\\');
	strPath.Delete(0,nSlashPos+1);

	int nPos = strPath.ReverseFind('.');
	if(nPos==-1)
	{
		return;
	}
		
	try
	{
		m_strTest = strPath.Left(nPos);
		CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
		CCORBAInfo* pInfo = pApp->GetInfo();
		pInfo->KillTest(m_strTest);
	}
	catch(...)
	{
	}
}

void CTestPluginView::OnUpdateKillTest(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_bTestActive);	
}
