/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// OpenSTABHO.cpp : Implementation of COpenSTABHO
#include "stdafx.h"
#include "CaptureBHO.h"
#include "OpenSTABHO.h"
#include "Hooker.h"
#include <wininet.h>


#ifdef _DEBUG
#   include<fstream>
    using std::endl;
    extern std::ofstream fout;
#   pragma comment(lib, "wininet.lib")
#endif

/////////////////////////////////////////////////////////////////////////////
// COpenSTABHO


// Type definitions.
typedef BOOL (WINAPI *PFNINTERNETSETCOOKIEW)(LPCWSTR,LPCWSTR,LPCWSTR);
typedef BOOL (WINAPI *PFNINTERNETGETCOOKIEW)(LPCWSTR,LPCWSTR,LPWSTR,LPDWORD);

// Forward declarations.
BOOL WINAPI Hook_InternetSetCookieW(LPCWSTR,LPCWSTR,LPCWSTR);
BOOL WINAPI Hook_InternetGetCookieW(LPCWSTR,LPCWSTR,LPWSTR,LPDWORD);
// END Forward declarations.

// Custom hooks
CHooker* g_InternetSetCookieW = NULL;
CHooker* g_InternetGetCookieW = NULL;
// END Custom hooks

void COpenSTABHO::Init()
{
#ifdef _DEBUG
    fout << "~~Creating the custom hooks..." << endl;
#endif

    g_InternetSetCookieW = new CHooker("InternetSetCookieW", "Wininet.dll", (PROC)Hook_InternetSetCookieW);
    g_InternetGetCookieW = new CHooker("InternetGetCookieW", "Wininet.dll", (PROC)Hook_InternetGetCookieW);

#ifdef _DEBUG
    fout << "~~Custom hooks created." << endl;
#endif
}

void COpenSTABHO::Uninit()
{
#ifdef _DEBUG
    fout << "~~Deleting the custom hooks..." << endl;
#endif

    if(g_InternetSetCookieW)
    {
        delete g_InternetSetCookieW;
        g_InternetSetCookieW = NULL;
    }
    if(g_InternetGetCookieW)
    {
        delete g_InternetGetCookieW;
        g_InternetGetCookieW = NULL;
    }

#ifdef _DEBUG
    fout << "~~Custom hooks deleted." << endl;
#endif
}

BOOL WINAPI Hook_InternetSetCookieW(LPCWSTR lpszUrl,
    LPCWSTR lpszCookieName, LPCWSTR lpszCookieData)
{
#ifdef _DEBUG
    DWORD dwThread = GetCurrentThreadId();
    char zVarA[8192];
    WideCharToMultiByte(CP_ACP, 0, lpszUrl, -1, zVarA, sizeof(zVarA), NULL, NULL);
    fout << dwThread << ": "
         << "InternetSetCookieW( "
         << zVarA
         << ", ";
    zVarA[0] = 0;
    WideCharToMultiByte(CP_ACP, 0, lpszCookieName, -1, zVarA, sizeof(zVarA), NULL, NULL);
    fout << zVarA
         << ", ";
    if(lpszCookieData)
    {
        zVarA[0] = 0;
        WideCharToMultiByte(CP_ACP, 0, lpszCookieData, -1, zVarA, sizeof(zVarA), NULL, NULL);
        fout << zVarA;
    }
    else
        fout << "NULL";
    fout << " )"
         << endl;
#endif

    WCHAR zOtherURL[1024] = L"http://";

    if(
        lpszUrl[0] == L'h'
     && lpszUrl[1] == L't'
     && lpszUrl[2] == L't'
     && lpszUrl[3] == L'p'
     && lpszUrl[4] == L':'
     && lpszUrl[5] == L'/'
     && lpszUrl[6] == L'/')
    {
        int iOffset = (lpszUrl[7] == L'{' ? 8 : 7);
        lstrcatW(zOtherURL, lpszUrl + iOffset);
    }
    else
        lstrcpyW(zOtherURL, lpszUrl);

    // Call the original InternetSetCookieA function.
    BOOL bRet = ((PFNINTERNETSETCOOKIEW)g_InternetSetCookieW->GetOriginalFunction())
        (zOtherURL, lpszCookieName, lpszCookieData);

    return bRet;
}

BOOL WINAPI Hook_InternetGetCookieW(LPCWSTR lpszUrl, LPCWSTR lpszCookieName,
    LPWSTR lpszCookieData, LPDWORD lpdwSize)
{
    WCHAR zOtherURL[1024] = L"http://";

    if(
        lpszUrl[0] == L'h'
     && lpszUrl[1] == L't'
     && lpszUrl[2] == L't'
     && lpszUrl[3] == L'p'
     && lpszUrl[4] == L':'
     && lpszUrl[5] == L'/'
     && lpszUrl[6] == L'/')
    {
        int iOffset = (lpszUrl[7] == L'{' ? 8 : 7);
        lstrcatW(zOtherURL, lpszUrl + iOffset);
    }
    else
        lstrcpyW(zOtherURL, lpszUrl);

    // Call the original InternetGetCookieW function for the associated URL.
    BOOL bRet = ((PFNINTERNETGETCOOKIEW)g_InternetGetCookieW->GetOriginalFunction())
        (zOtherURL, lpszCookieName, lpszCookieData, lpdwSize);

#ifdef _DEBUG
    DWORD dwThread = GetCurrentThreadId();
    char zVarA[8192];
    WideCharToMultiByte(CP_ACP, 0, lpszUrl, -1, zVarA, sizeof(zVarA), NULL, NULL);
    fout << dwThread << ": "
         << "InternetGetCookieW( "
         << zVarA
         << ", ";
    zVarA[0] = 0;
    WideCharToMultiByte(CP_ACP, 0, lpszCookieName, -1, zVarA, sizeof(zVarA), NULL, NULL);
    fout << zVarA
         << ", ";
    if(lpszCookieData)
    {
        zVarA[0] = 0;
        WideCharToMultiByte(CP_ACP, 0, lpszCookieData, -1, zVarA, sizeof(zVarA), NULL, NULL);
        fout << zVarA;
    }
    else
        fout << "NULL";
    fout << " )"
         << endl;
#endif

    return bRet;
}
