/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef __MEM_MAPPED_FILE_HPP__
#define __MEM_MAPPED_FILE_HPP__

#include <windows.h>
#include <string>

class CMemMappedFile
{
public:
    CMemMappedFile();
    ~CMemMappedFile();

    BOOL OpenFile   (const std::string &zFileName, DWORD dwViewSize, DWORD dwAccessMode/*GENERIC_READ, GENERIC_WRITE*/, DWORD dwShareMode/*FILE_SHARE_READ*/);
    void CloseFile  ();
    BOOL WriteData  (LPCSTR zBuff,  DWORD dwBuffLen);                       // Output data into the memory mapped file.
    BOOL ReadData   (LPSTR zBuff,   DWORD dwBuffLen, DWORD &dwReadBytes);   // Input data from the memory mapped file.

    void Rewind     ();
    BOOL Seek       (DWORD dwPosInFile);

protected:
    //// Helpers.
    BOOL AllocMemMappedFile(const std::string &zFileName, DWORD dwAccessMode, DWORD dwShareMode);
    void ReleaseMemMappedFile();
    BOOL MoveView(DWORD dwStartPos);

    //// Data.
    HANDLE  m_hFile;            // File handle.
    LPSTR   m_pView;            // Current view handle.
    DWORD   m_dwCurrViewOffset; // Current offset in the m_pView view.
    DWORD   m_dwFileSize;       // Current file size (while writting).
    DWORD   m_dwViewSize;       // View size.

    DWORD   m_dwAccessMode;     // Store the file access mode to restrict access mode.

private:
    // Forbidden operators.
    const CMemMappedFile& operator=(const CMemMappedFile &);
    CMemMappedFile(const CMemMappedFile &);
};

#endif
