/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////////////
//
// Specialisations of PhLexer for CDataBuffListAccessor and CDataBuffListChunkAccessor.
//
///////////////////////////////////////////////////////////////////////////////////////

template <>
inline BOOL PhLexer<CDataBuffListAccessor>::InScript() const
{
    return pcFirstChar_.MatchFirstWordToLower("<script", sizeof "<script" -1);
}

template <>
inline BOOL PhLexer<CDataBuffListChunkAccessor>::InScript() const
{
    return pcFirstChar_.MatchFirstWordToLower("<script", sizeof "<script" -1);
}

template <>
inline 
void PhLexer<CDataBuffListAccessor>::CollectScript(CSymbolAttributes<CDataBuffListAccessor>& sa)
{
    // Skip characters until </script (not included).
    CDataBuffListAccessor pcBegin = pcCurChar_;
//    CDataBuffListAccessor pcEnd   = pcBegin;

    // Move pcEnd to the first char in the "</script" sequence.
    if(pcCurChar_.MoveToStr("</script", sizeof "</script" -1, FALSE) == TRUE)
    {
        if(pcBegin != pcCurChar_)
        {
            sa.label_.pcBegin_ = pcBegin;
            sa.label_.ulSize_  = pcCurChar_ - pcBegin;   // TODO: MoveToStr pourrait renvoyer soit la distance parcourrue, soit ULONG_MAX.

            // Update the line and column.
            CDataBuffListAccessor pc = pcBegin;
            for(DWORD dw = 0; dw < sa.label_.ulSize_; ++dw, ++pc)
                UpdateLineCol(*pc);
        }
//        pcCurChar_ = pcEnd;
    }
}

template <>
inline 
void PhLexer<CDataBuffListChunkAccessor>::CollectScript(CSymbolAttributes<CDataBuffListChunkAccessor>& sa)
{
    // Skip characters until </script (not included).
    CDataBuffListChunkAccessor pcBegin = pcCurChar_;
//    CDataBuffListChunkAccessor pcEnd   = pcBegin;

    if(pcCurChar_.MoveToStr("</script", sizeof "</script" -1, FALSE) == TRUE)
    {
        if(pcBegin != pcCurChar_)
        {
            sa.label_.pcBegin_ = pcBegin;
            sa.label_.ulSize_  = pcCurChar_ - pcBegin;

            // Update the line and column.
            CDataBuffListChunkAccessor pc = pcBegin;
            for(DWORD dw = 0; dw < sa.label_.ulSize_; ++dw, ++pc)
                UpdateLineCol(*pc);
        }
//        pcCurChar_ = pcEnd;
    }
}

template <>
inline 
void PhLexer<CDataBuffListAccessor>::CollectText(CSymbolAttributes<CDataBuffListAccessor>& sa)
{
    // Skip to the end of the text.
    CDataBuffListAccessor pcBegin = pcCurChar_;
    CDataBuffListAccessor pcEnd   = pcBegin;
    int iState = 0;     // 0 =
                        // 1 = <
                        // 2 = </
                        // 3 = </a or <a  or  <! or end of input (where a is an alphabetic character).
    while(*pcCurChar_ && iState != 3)
    {
        switch(*pcCurChar_)
        {
            case '<':
                iState = 1;
                pcEnd = pcCurChar_;
                break;

            case '/':
                if(iState == 1)
                    iState = 2;
                else
                    iState = 0;
                break;

            case '!':
                if(iState == 1)
                    iState = 3;
                else
                    iState = 0;
                break;

            default:
                if((iState == 1 || iState == 2) && isalpha(*pcCurChar_))
                    iState = 3;
                else
                    iState = 0;
                break;
        }

        // Skip without updating line and column.
        if(iState != 3)
            ++pcCurChar_;
    }

    if(*pcCurChar_ == 0)
        pcEnd = pcCurChar_;
    else
        pcCurChar_ = pcEnd;

    sa.label_.pcBegin_ = pcFirstChar_;
    sa.label_.ulSize_  = pcEnd - pcFirstChar_;

    // Update the line and column.
    CDataBuffListAccessor pc = pcBegin;
    for(DWORD dw = 0 ; dw < sa.label_.ulSize_ ; ++dw, ++pc)
        UpdateLineCol(*pc);
}

template <>
inline 
void PhLexer<CDataBuffListChunkAccessor>::CollectText(CSymbolAttributes<CDataBuffListChunkAccessor>& sa)
{
    // Skip to the end of the text.
    CDataBuffListChunkAccessor pcBegin = pcCurChar_;
    CDataBuffListChunkAccessor pcEnd   = pcBegin;
    int iState = 0;     // 0 =
                        // 1 = <
                        // 2 = </
                        // 3 = </a or <a  or  <! or end of input (where a is an alphabetic character).
    while(*pcCurChar_ && iState != 3)
    {
        switch(*pcCurChar_)
        {
            case '<':
                iState = 1;
                pcEnd = pcCurChar_;
                break;

            case '/':
                if(iState == 1)
                    iState = 2;
                else
                    iState = 0;
                break;

            case '!':
                if(iState == 1)
                    iState = 3;
                else
                    iState = 0;
                break;

            default:
                if((iState == 1 || iState == 2) && isalpha(*pcCurChar_))
                    iState = 3;
                else
                    iState = 0;
                break;
        }

        // Skip without updating line and column.
        if(iState != 3)
            ++pcCurChar_;
    }

    if(*pcCurChar_ == 0)
        pcEnd = pcCurChar_;
    else
        pcCurChar_ = pcEnd;

    sa.label_.pcBegin_ = pcFirstChar_;
    sa.label_.ulSize_  = pcEnd - pcFirstChar_;

    // Update the line and column.
    CDataBuffListChunkAccessor pc = pcBegin;
    for(DWORD dw = 0 ; dw < sa.label_.ulSize_ ; ++dw, ++pc)
        UpdateLineCol(*pc);
}


#ifdef PHTRACE

template <>
inline void PhLexer<CDataBuffListAccessor>::DumpLexeme(std::ostream& os)
{
    for(CDataBuffListAccessor puc = pcFirstChar_; puc != pcCurChar_; ++puc)
        os << *puc;
}

template <>
inline void PhLexer<CDataBuffListChunkAccessor>::DumpLexeme(std::ostream& os)
{
    for(CDataBuffListChunkAccessor puc = pcFirstChar_; puc != pcCurChar_; ++puc)
        os << *puc;
}

#endif

template <>
inline std::string& MakeString(CDataBuffListAccessor strIn, unsigned long ulLen, std::string& strOut)
{
    BOOL bRes = strIn.BuildString(strOut, ulLen);
    assert(bRes);

    return strOut;
}

template <>
inline std::string& MakeString(CDataBuffListChunkAccessor strIn, unsigned long ulLen, std::string& strOut)
{
    BOOL bRes = strIn.BuildString(strOut, ulLen);
    assert(bRes);

    return strOut;
}
