/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
#ifndef __HTMLTREE__
#define __HTMLTREE__

// -----------------------------------------------------------------------------
// HTML Tree classes.
// These classes are used to build a tree representation of an HTML page.
// They are not scalable.
// -----------------------------------------------------------------------------


#include "TagParsing.hpp"
#include <iostream>
#include <string>
#include <list>


struct CTag
{
    // Types.
    struct CNode
    {
        std::string strAddress_;
        std::string strText_;
    };
    struct CAttribute : public CNode
    {
        std::string strLabel_;
    };
    typedef std::list<CNode>      NodeList;
    typedef std::list<CAttribute> AttributeList;
    typedef std::list<CTag*>      TagList;

    // Construction / Destruction.
    CTag() : eTagType_(TAG_NONE), pParent_(NULL) {}
    ~CTag();    // Delete the sons.

    // Output to a stream.
    void Dump(std::ostream& os = std::cout) const;

    // Data.
    Tags            eTagType_;
    std::string     strLabel_;
    AttributeList   attributes_;
    NodeList        texts_;
    NodeList        scripts_;
    NodeList        comments_;
    TagList         sons_;
    CTag*           pParent_;

private:
    // Helpers.
    void DeleteTag();
    void DumpTag(std::ostream& os, int iLevel) const;
    void DumpNode(std::ostream& os, const CNode& node) const;
    void DumpAttribute(std::ostream& os, const CAttribute& att) const;
    void Margin(std::ostream& os, int iLevel) const;
};

#endif  // __HTMLTREE__
