/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include <definition.h>
#include <socket.hpp>
#include "httpCaptureCmd.hpp"
#include "gatewayInstance.hpp"

#define DLLSPEC_GWCONSCMD
#include "gwconscmd.hpp"

#include <exception.hpp>

#pragma warning(disable: 4514)

#define HTTP_DECODING_DLL   "gwhttp"
#define HTTP_COMMAND_DLL    "gwhttp"

#define MAX_CONNECT_TRY     5       // admin connect try number
#define CONNECT_SLEEP       2000    // time between each connect try

CGatewayInstanceList    gGatewayInstanceList ;

//---------------------------------------------------------------------------
// HttpCaptureCmd::HttpCaptureCmd
//---------------------------------------------------------------------------
// Constructor
//---------------------------------------------------------------------------
// string commandDllName : gateway command Dll
// port_t adminPort : the port number to be connected
//---------------------------------------------------------------------------
HttpCaptureCmd::HttpCaptureCmd(port_t adminPort)
        : adminPort_(adminPort), adminGateSocket_(NULL)
{
    WORD wVersionRequested;
    WSADATA wsaData;
    int err;

    wVersionRequested = MAKEWORD( 2, 2 );

    err = WSAStartup( wVersionRequested, &wsaData );
}

//---------------------------------------------------------------------------
// HttpCaptureCmd::~HttpCaptureCmd
//---------------------------------------------------------------------------
// destructor
//---------------------------------------------------------------------------
// Return    :  None.
//---------------------------------------------------------------------------
HttpCaptureCmd::~HttpCaptureCmd()
{
    try
    {
        if (adminGateSocket_ != NULL)
        {
            adminGateSocket_->close();
        }
    }
    catch (CTException & )
    {
        return;
    }
}

//---------------------------------------------------------------------------
// HttpCaptureCmd::startGateway
//---------------------------------------------------------------------------
// Start the gateway
//---------------------------------------------------------------------------
// Return    :  true or false.
//---------------------------------------------------------------------------
bool HttpCaptureCmd::startGateway(string _strBufDirectory, bool bConsole, bool bTrace)
{
    //
    // Try to start the gateway process
    //
    bool            bRet = true;
    ;
    STARTUPINFO         si;
    PROCESS_INFORMATION pi;

    // Add an instance of the gateway in the CGatewayInstance list
    // ot inrease the instanceNumber if the gateway already exists
    CGatewayInstance * pgInstance = new CGatewayInstance(adminPort_);
    CGatewayInstance * pgi = NULL ;
    if ((pgi = gGatewayInstanceList.find(pgInstance)) != NULL)
    {
        pgi->increase() ;
        delete pgInstance ;
    }
    else
    {
        pgInstance->increase() ;
        gGatewayInstanceList.add(pgInstance) ;

        memset(&si, 0, sizeof(si));
        si.cb = sizeof(si);

        char * applicationName = new char[512 + _strBufDirectory.length()] ;
        char bufExecCommand[512] ;
        char strTrace[25] ;
        strTrace[0] ='\0';

        if ( bTrace )
            strcpy(strTrace,"-trace");
#ifdef _DEBUG
        sprintf(bufExecCommand, "gatewayd.exe  %s -adm_port %d -log \"%s\\gateway.log\"", strTrace, adminPort_,_strBufDirectory.c_str());
        sprintf(applicationName, "%s\\gatewayd.exe", _strBufDirectory.c_str()) ;
#else
sprintf(bufExecCommand, "gateway.exe %s -adm_port %d -log \"%s\\gateway.log\"", strTrace, adminPort_,_strBufDirectory.c_str());
        sprintf(applicationName, "%s\\gateway.exe", _strBufDirectory.c_str()) ;
#endif

        DWORD creationFlag  = CREATE_DEFAULT_ERROR_MODE ;
        if (bConsole)
            creationFlag |= CREATE_NEW_CONSOLE ;
        else
            creationFlag |= DETACHED_PROCESS;

        if (CreateProcess(LPCTSTR(applicationName), LPTSTR(LPCTSTR(bufExecCommand)), NULL, NULL, false,
                          creationFlag,
                          NULL, NULL, &si, &pi) == false)
        {
            LPVOID lpMsgBuf;
            FormatMessage(
                FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
                NULL,
                GetLastError(),
                MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                (LPTSTR) &lpMsgBuf,
                0,
                NULL
            );
            string strErrMsg = (char *)lpMsgBuf ;
            LocalFree( lpMsgBuf );
            if (applicationName)
                delete [] applicationName ;
            throw CTException(strErrMsg, __FILE__, __LINE__);
        }
        else
        {
            if (applicationName)
                delete [] applicationName ;
            CloseHandle(pi.hThread);
            CloseHandle(pi.hProcess);
        }
    }

    //
    // Specify the commandDll name to the gateway
    //
    char name[512] ;
    gethostname(name, sizeof(name)) ;

    bool bConnected = false;
    int iNbTry = MAX_CONNECT_TRY ;
    for (int i=0; i<iNbTry; i++)
    {
        try
        {
            string  result ;
            if (adminGateSocket_ != NULL)
            {
                delete adminGateSocket_ ;
            }
            adminGateSocket_ = new CTSocket();
            adminGateSocket_->connect(name,adminPort_);

            string useCommand = string ("CONSOLE:USE ") + string(HTTP_COMMAND_DLL) ;
            *adminGateSocket_ << useCommand ;
            *adminGateSocket_ >> result ;
        }

        catch (CTException & )
        {
            bConnected = false;
            if (adminGateSocket_ != NULL)
            {
                delete adminGateSocket_;
                adminGateSocket_ = NULL;
            }
            if (i == iNbTry - 1) break;
            Sleep(CONNECT_SLEEP);
            continue;
        }
        bConnected = true;
        break;
    }
    if (!bConnected)
    {
        throw CTException(string("startGateway:Cannot initialize the gateway"), __FILE__, __LINE__);
    }
    return true ;
}

//---------------------------------------------------------------------------
// HttpCaptureCmd::listen
//---------------------------------------------------------------------------
// create a gateway
//---------------------------------------------------------------------------
// Arguments :  port_t listenPort : gateway port number
//              string & decodingDllName : decoding dll to load
//
// Return    :  bool : true or false
//---------------------------------------------------------------------------
bool HttpCaptureCmd::listen(port_t listenPort)
{
    try
    {
        string  listenPortTmp;
        char        buffer[10];

        sprintf(buffer, "%d", listenPort);
        listenPortTmp = buffer;

        CGListen    listen (false, "WEB", listenPortTmp, string(HTTP_DECODING_DLL), false) ;
        string  result ;

        *adminGateSocket_ << string ("CONSOLE:LISTEN") ;
        *adminGateSocket_ << listen ;
        *adminGateSocket_ >> result ;
        return (result == "OK")?true:false;
    }
    catch (CTException & e)
    {
        string strError = "listen:";
        strError += e.what();
        throw CTException(strError, string(__FILE__), __LINE__);
    }

    return false;
}


//---------------------------------------------------------------------------
// HttpCaptureCmd::browserParameters
//---------------------------------------------------------------------------
// send the browser parameters to the gateway
//---------------------------------------------------------------------------
// string httpProxy  : proxy for HTTP (in format machine:port)
// string httpsProxy : proxy for HTTPS
// string ftpProxy   : proxy for FTP
// string gopherProxy : proxy for GOPHER
// string socksProxy : proxy for SOCKS
// string exceptions : exceptions machines (not through proxy)
// bool bNoProxyForLocal : true = no proxy for local address
//---------------------------------------------------------------------------
// Return    :  bool : true or false
//---------------------------------------------------------------------------
bool HttpCaptureCmd::browserParameters(string httpProxy,
                                       string httpsProxy,
                                       string ftpProxy,
                                       string gopherProxy,
                                       string socksProxy,
                                       string exceptions,
                                       bool bNoProxyForLocal)
{
    try
    {
        string  result ;

        *adminGateSocket_ << string ("SELECT:WEBCONNECTION") ;

        string proxyParams = "";
        if (httpProxy == "" || httpsProxy == "")
        {
            throw CTException(string("Must supply HTTP and HTTPS proxy definition"), __FILE__, __LINE__) ;
        }

        proxyParams += "HTTP:" ;
        proxyParams += httpProxy ;
        proxyParams += ";SSL:" ;
        proxyParams += httpsProxy ;
        if (ftpProxy != "")
        {
            proxyParams += ";FTP:" ;
            proxyParams += ftpProxy ;
        }
        if (gopherProxy != "")
        {
            proxyParams += ";GOPHER:" ;
            proxyParams += gopherProxy ;
        }
        if (socksProxy != "")
        {
            proxyParams += ";SOCKS:" ;
            proxyParams += socksProxy ;
        }

        *adminGateSocket_ << proxyParams ; // proxy parameters
        *adminGateSocket_ << exceptions ; // exceptions
        if (bNoProxyForLocal)
        {
            *adminGateSocket_ << string("no_proxy_for_local") ;
        }
        else
        {
            *adminGateSocket_ << string("proxy_for_local") ;
        }
        *adminGateSocket_ >> result ;
        return (result == "OK")?true:false;
    }
    catch (CTException & e)
    {
        string strError = "browserParameters:";
        strError += e.what();
        throw CTException(strError, string(__FILE__), __LINE__);
    }

    return false;
}

//---------------------------------------------------------------------------
// HttpCaptureCmd::browserParameters
//---------------------------------------------------------------------------
// send the browser parameters to the gateway
// FTP, GOPHER and SOCKS are put to HTTP value
//---------------------------------------------------------------------------
// string httpProxy  : proxy for HTTP (in format machine:port)
// string httpsProxy : proxy for HTTPS
// string exceptions : exceptions machines (not through proxy)
// bool bNoProxyForLocal : true = no proxy for local address
//---------------------------------------------------------------------------
// Return    :  bool : true or false
//---------------------------------------------------------------------------
bool HttpCaptureCmd::browserParameters(string httpProxy,
                                       string httpsProxy,
                                       string exceptions,
                                       bool bNoProxyForLocal)
{
    return browserParameters(httpProxy, httpsProxy, httpProxy, httpProxy, httpProxy, exceptions, bNoProxyForLocal) ;
}

//---------------------------------------------------------------------------
// HttpCaptureCmd::browserParameters
//---------------------------------------------------------------------------
// send the browser parameters to the gateway
//---------------------------------------------------------------------------
// string proxy  : proxy for every protocol (in format machine:port)
// string exceptions : exceptions machines (not through proxy)
// bool bNoProxyForLocal : true = no proxy for local address
//---------------------------------------------------------------------------
// Return    :  bool : true or false
//---------------------------------------------------------------------------
bool HttpCaptureCmd::browserParameters(string proxy,
                                       string exceptions,
                                       bool bNoProxyForLocal)
{
    return browserParameters(proxy, proxy, proxy, proxy, proxy, exceptions, bNoProxyForLocal) ;
}


//---------------------------------------------------------------------------
// HttpCaptureCmd::setDocumentsPath
//---------------------------------------------------------------------------
// send the path where the gateway must save all the documents
//---------------------------------------------------------------------------
// string path : the path where the gateway must save all the documents
//---------------------------------------------------------------------------
// Return    :  bool : true or false
//---------------------------------------------------------------------------
bool HttpCaptureCmd::setDocumentsPath(string path)
{
    try
    {
        string message;

        *adminGateSocket_ << string("SELECT:SETDOCUMENTSPATH");
        *adminGateSocket_ << path ;

        string  result ;
        *adminGateSocket_ >> result ;

        return (result == "OK")?true:false;
    }
    catch (CTException & e)
    {
        string strError = "setDocumentsPath:";
        strError += e.what();
        throw CTException(strError, string(__FILE__), __LINE__);
    }

    return false;
}

//---------------------------------------------------------------------------
// HttpCaptureCmd::record
//---------------------------------------------------------------------------
// send the record message to the adminGateway
//---------------------------------------------------------------------------
// string tofFileName : Tof file Name
// int iConid : first conid that the gaeway must use
// string tofTitle : title that will be writtent at the top of
//                      the tof file
//---------------------------------------------------------------------------
// Return    :  bool : true or false
//---------------------------------------------------------------------------
bool HttpCaptureCmd::record(string tofFileName, int iConid, string tofTitle)
{
    try
    {
        string message;
        char zBuf[64] ;
        sprintf(zBuf, "%d", iConid);

        *adminGateSocket_ << string("SELECT:CAPTURE");
        *adminGateSocket_ << string(zBuf);
        *adminGateSocket_ << string("record");

        *adminGateSocket_ << tofFileName;

        *adminGateSocket_ << string("only");

        if (tofTitle != "")
            *adminGateSocket_ << tofTitle;
        else
            *adminGateSocket_ << string (" ");

        string  result ;
        *adminGateSocket_ >> result ;

        return (result == "OK")?true:false;
    }
    catch (CTException & e)
    {
        string strError = "record:";
        strError += e.what();
        throw CTException(strError, string(__FILE__), __LINE__);
    }

    return false;
}


//---------------------------------------------------------------------------
// HttpCaptureCmd::pause
//---------------------------------------------------------------------------
// send the pause message to the adminGateway
//---------------------------------------------------------------------------
// Return    :  bool : true or false
//---------------------------------------------------------------------------
bool HttpCaptureCmd::pause()
{
    try
    {
        *adminGateSocket_ << string("SELECT:CAPTURE");
        *adminGateSocket_ << string("WEB");
        *adminGateSocket_ << string("pause");

        string  result ;
        *adminGateSocket_ >> result ;
        return (result == "OK")?true:false;
    }
    catch (CTException & e)
    {
        string strError = "pause:";
        strError += e.what();
        throw CTException(strError, string(__FILE__), __LINE__);
    }

    return false;
}

//---------------------------------------------------------------------------
// HttpCaptureCmd::resume
//---------------------------------------------------------------------------
// send the resume message to the adminGateway
//---------------------------------------------------------------------------
// Return    :  bool : true or false
//---------------------------------------------------------------------------
bool HttpCaptureCmd::resume()
{
    try
    {
        *adminGateSocket_ << string("SELECT:CAPTURE");
        *adminGateSocket_ << string("WEB");
        *adminGateSocket_ << string("resume");

        string  result ;
        *adminGateSocket_ >> result ;
        return (result == "OK")?true:false;
    }
    catch (CTException & e)
    {
        string strError = "resume:";
        strError += e.what();
        throw CTException(strError, string(__FILE__), __LINE__);
    }

    return false;
}

//---------------------------------------------------------------------------
// HttpCaptureCmd::stop
//---------------------------------------------------------------------------
// send the stop message to the adminGateway
//---------------------------------------------------------------------------
// Return    :  bool : true or false
//---------------------------------------------------------------------------
bool HttpCaptureCmd::stop()
{
    try
    {
        *adminGateSocket_ << string("SELECT:CAPTURE");
        *adminGateSocket_ << string("WEB");
        *adminGateSocket_ << string("stop");

        string  result ;
        *adminGateSocket_ >> result ;
        return (result == "OK")?true:false;
    }
    catch (CTException & e)
    {
        string strError = "stop:";
        strError += e.what();
        throw CTException(strError, string(__FILE__), __LINE__);
    }

    return false;
}


//---------------------------------------------------------------------------
// HttpCaptureCmd::comment
//---------------------------------------------------------------------------
// send the comment message to the adminGateway
//---------------------------------------------------------------------------
// string comment : comment to add in tof file
//---------------------------------------------------------------------------
// Return    :  bool : true or false
//---------------------------------------------------------------------------
bool HttpCaptureCmd::comment(string tofComment)
{
    try
    {
        *adminGateSocket_ << string("SELECT:CAPTURE");
        *adminGateSocket_ << string("WEB");
        *adminGateSocket_ << string("comment");
        *adminGateSocket_ << tofComment;

        string  result ;
        *adminGateSocket_ >> result ;
        return (result == "OK")?true:false;
    }
    catch (CTException & e)
    {
        string strError = "comment:";
        strError += e.what();
        throw CTException(strError, string(__FILE__), __LINE__);
    }

    return false;
}

//---------------------------------------------------------------------------
// HttpCaptureCmd::shutdown
//---------------------------------------------------------------------------
// send the shutdown message to the adminGateway
//---------------------------------------------------------------------------
// Return    :  bool : true or false
//---------------------------------------------------------------------------
bool HttpCaptureCmd::shutdown()
{
    if (adminGateSocket_ == NULL)
        return true ;

    CGatewayInstance * pgInstance = new CGatewayInstance(adminPort_);
    CGatewayInstance * pi = NULL ;
    if ((pi = gGatewayInstanceList.find(pgInstance)) == NULL)
        return true ;

    gGatewayInstanceList.remove(pi) ;

    try
    {
        CGShutdown  shutdown (true) ;
        *adminGateSocket_ << string ("CONSOLE:SHUTDOWN") ;
        *adminGateSocket_ << shutdown ;

        string  result ;
        *adminGateSocket_ >> result ;
    }
    catch (CTException & )
    {
        return true;
    }

    return true;
}

//---------------------------------------------------------------------------
// HttpCaptureCmd::automaticCookiesModelling
//---------------------------------------------------------------------------
// send the automaticModel message to the adminGateway
// true  : the gateway model the cookies
// false : the gateway don't model the cookies
//---------------------------------------------------------------------------
// Return    :  bool : true or false
//---------------------------------------------------------------------------
bool HttpCaptureCmd::automaticCookiesModelling(bool bModel)
{
    try
    {
        *adminGateSocket_ << string("SELECT:AUTOMATICMODEL");
        *adminGateSocket_ << bModel ;

        string  result ;
        *adminGateSocket_ >> result ;
        return (result == "OK")?true:false;
    }
    catch (CTException & e)
    {
        string strError = "stop:";
        strError += e.what();
        throw CTException(strError, string(__FILE__), __LINE__);
    }

    return false;
}

//---------------------------------------------------------------------------
// CGatewayInstanceList
//---------------------------------------------------------------------------
void CGatewayInstanceList::add(CGatewayInstance * pGatewayInstance)
{
    LockGuard lock(monitor()) ;
    bool bFound = false ;
    for (CGatewayInstanceStlList::iterator itr = list_.begin(); itr != list_.end(); itr++)
    {
        if (*(*itr) == *pGatewayInstance)
        {
            bFound = true ;
            break ;
        }
    }
    if (!bFound)
    {
        list_.push_back(pGatewayInstance);
    }
}

void CGatewayInstanceList::remove(CGatewayInstance * pGatewayInstance)
{
    LockGuard lock(monitor()) ;
    list_.remove(pGatewayInstance);
}

CGatewayInstance * CGatewayInstanceList::find(CGatewayInstance * pGatewayInstance)
{
    LockGuard lock(monitor()) ;
    bool bFound = false ;
    for (CGatewayInstanceStlList::iterator itr = list_.begin(); itr != list_.end(); itr++)
    {
        if (*(*itr) == *pGatewayInstance)
        {
            return (*itr) ;
        }
    }
    return NULL;
}

void CGatewayInstanceList::reset(void)
{
    LockGuard lock(monitor()) ;
    for (CGatewayInstanceStlList::iterator itr = list_.begin(); itr != list_.end(); itr++)
    {
        delete *itr ;
    }
    list_.clear();
}

