/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TabWnd.cpp: implementation of the CTabWnd class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TabWnd.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
// CTabWnd

CTabWnd::CTabWnd()
{
	m_pCurrentWnd = NULL;
}

CTabWnd::~CTabWnd()
{
}


BEGIN_MESSAGE_MAP(CTabWnd, CTabCtrl)
	//{{AFX_MSG_MAP(CTabWnd)
	ON_NOTIFY_REFLECT(TCN_SELCHANGE, OnSelchange)
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CTabWnd::Create (CWnd* pParentWnd, UINT nID)
{
	return CTabCtrl::Create(TCS_BOTTOM|WS_CHILD|WS_VISIBLE,CRect(0,0,0,0),pParentWnd,nID);
}

/////////////////////////////////////////////////////////////////////////////
// CTabWnd message handlers
//**********************************************************************************************
CWnd* CTabWnd::CreateView(CRuntimeClass* pViewClass,
                               CCreateContext* pContext, UINT nID)
{
    ASSERT_VALID(this);
    ASSERT(pViewClass != NULL);
#ifdef WIN32
    ASSERT(pViewClass->IsDerivedFrom(RUNTIME_CLASS(CWnd)));
#endif

#if _MFC_VER < 0x042
    ASSERT(AfxIsValidAddress(pViewClass, sizeof(CRuntimeClass)));
#endif
    CWnd* pWnd = NULL;
    TRY
    {
        pWnd = (CWnd*)pViewClass->CreateObject();
        if (pWnd == NULL)
        AfxThrowMemoryException();
    }
    CATCH_ALL(e)
    {
        TRACE0("Out of memory creating a tab window view\n");
        return pWnd;
    }
    END_CATCH_ALL

    ASSERT(pWnd->IsKindOf(RUNTIME_CLASS(CWnd)));
    ASSERT(pWnd->m_hWnd == NULL);       // not yet created

    if (!pWnd->Create(NULL, NULL, WS_CHILD|WS_BORDER ,
                      CRect(0,0,0,0), this, nID, pContext))
    {
        TRACE0("Warning: couldn't create the tab window view\n");
        return NULL;
    }


    return pWnd;
}

//**********************************************************************************************
void CTabWnd::AddTab(CRuntimeClass* pViewClass,
                          LPCTSTR lpszLabel,
                          CCreateContext* pContext,
                          UINT nID, UINT uiImageId)
{
    ASSERT_VALID(this);

	int nNextTab = GetItemCount();
    if (nID == -1)
        nID = nNextTab;

    CWnd* pWnd = CreateView(pViewClass, pContext, nID);

	m_mapTabToWnd.SetAt(lpszLabel,pWnd);
	InsertItem(nNextTab,lpszLabel, uiImageId);
}

void CTabWnd::OnSelchange(NMHDR* pNMHDR, LRESULT* pResult) 
{
	int nCurSel = GetCurSel();

	SetActiveTab(nCurSel);
	
	*pResult = 0;
}

void CTabWnd::SetImageList(UINT uiIdBmp,int nSize,COLORREF clrMask)
{
	m_ImageList.DeleteImageList();
	m_ImageList.Create(uiIdBmp,nSize,1,clrMask);
	CTabCtrl::SetImageList(&m_ImageList);
}

void CTabWnd::SetActiveTab(int nNdx)
{
	CWnd *pWnd = GetTabWnd(nNdx);
	if(pWnd)
	{
		if(m_pCurrentWnd)
			m_pCurrentWnd->ShowWindow(SW_HIDE);
		m_pCurrentWnd = pWnd;

		CRect rcClient;
		GetClientRect(&rcClient);
		AdjustRect(FALSE,&rcClient);
		m_pCurrentWnd->MoveWindow(rcClient);
		m_pCurrentWnd->ShowWindow(SW_SHOW);
	}
}

CWnd *CTabWnd::GetTabWnd(int nNdx)
{
	CString strName;
	TCITEM item;
	item.mask = TCIF_TEXT;
	item.pszText = strName.GetBuffer(100);
	item.cchTextMax = 100;
	GetItem(nNdx,&item);
	strName.ReleaseBuffer();

	CWnd *pWnd;
	if(m_mapTabToWnd.Lookup(strName,pWnd))
		return pWnd;

	return NULL;
}

void CTabWnd::OnSize(UINT nType, int cx, int cy) 
{
	CTabCtrl::OnSize(nType, cx, cy);
	
	CRect rcClient(0,0,cx,cy);
	AdjustRect(FALSE,&rcClient);
	if(m_pCurrentWnd)
		m_pCurrentWnd->MoveWindow(rcClient);
}

