// GraphTypes.h: interface for the CTimerGraph class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_GRAPHTYPES_H__24021AB3_F437_11D3_BD83_00C04F89E392__INCLUDED_)
#define AFX_GRAPHTYPES_H__24021AB3_F437_11D3_BD83_00C04F89E392__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "HttpGraphs.h"
#define WMU_DATA_CHANGED (WM_APP+1)

#define GRAPH_MENU_ITEM_ID_START 0x8000
#define GRAPH_MENU_ITEM_ID_END 0x8005

class CXChartCtrl;
//CGraphTypeBase is the base class for all the graph types and
//controls the basic style of the graph,eg background , title
class CGraphTypeBase
{
protected:
    CGraphTypeBase(CXChartCtrl *pChart);

public:
    virtual ~CGraphTypeBase();
    virtual void SetGraphType(int nType);
    virtual int GetGraphType();

    // @cmember Called to add a legend to the graph display
    virtual void DisplayLegend();

    virtual BOOL RemoveTestRun(int nRun);

    BOOL ExportToExcel();

    struct MenuItems
    {
        CString strItem;
        UINT uiCommandID;
    };
    typedef CArray<MenuItems,MenuItems&> MenuItemArray;

    virtual void GetMenuItems(MenuItemArray& arMenuItems);
    virtual void OnCommand(UINT uiCommandID,CWnd *pParent);

	virtual BOOL DoesGraphHaveFilter(){return m_bHasFilter;}
	virtual void ShowFilter(){}

    //Add all the data from another graph to this one
//    void AddDataFromGraph(CGraphTypeBase *m_pView);

//	CWnd* GetView(){return m_pView;}
    // @access Private Members
protected:
    int m_nNextGroup;

    void SetValue(int nIndex,int nGroup,double dValue,const CString& strAnnotation = "");

	CXChartCtrl* m_pChart;
    CString m_strTitle;

	BOOL m_bHasFilter;
};

//CTimerGraph is the base class of all the graphs which use timer data
class CTimerGraph : public CGraphTypeBase
{
protected:
    CTimerGraph(CXChartCtrl *pChart);
    virtual ~CTimerGraph();
public:
    virtual BOOL AddTestRun(TimerList& listTimers,DWORD dwTestStartTime);
};

class CRTATGraph : public CTimerGraph
{
public:
    CRTATGraph(CXChartCtrl *pChart);
    virtual ~CRTATGraph();

    BOOL AddTestRun(TimerList& listTimers,DWORD dwTestStartTime);
};

class CRTTimeGraph : public CTimerGraph
{
public:
    CRTTimeGraph(CXChartCtrl *pChart);
    virtual ~CRTTimeGraph();

	BOOL AddTestRun(TimerList& listTimers,DWORD dwTestStartTime);

};

//CHttpGraph is the base class of all the graphs which use HTTP data
class CHttpGraph : public CGraphTypeBase
{
    typedef enum{enMenuItemFilter=GRAPH_MENU_ITEM_ID_START}MenuItemIDs;

public:
    virtual BOOL AddTestRun(HttpArray&,URLMap* m_mapUrls,DWORD dwTestStartTime);
    virtual ~CHttpGraph();

protected:
    CHttpGraph(CXChartCtrl *pChart);

    virtual void GetMenuItems(MenuItemArray& arMenuItems);
    virtual void OnCommand(UINT uiCommandID,CWnd *pParent);

    CDWordArray m_ardwURLFilters[20];   //This means that analyzer will Explode if
    //someone adds more than 20 http graphs
    BOOL IsURLInFilter(int nURL,int nGroup);
	void ShowFilter();
	
	URLMap* m_pmapUrls;
	HttpArray* m_parHttpData;
	DWORD m_dwTestStartTime;
};

class CThroughputGraph : public CHttpGraph
{
public:
    CThroughputGraph(CXChartCtrl *pChart);
    virtual ~CThroughputGraph();

protected:
    BOOL AddTestRun(HttpArray& arHttp,URLMap* m_mapUrls,DWORD dwTestStartTime);
};


class CHitsPerSecondGraph : public CHttpGraph
{
public:
    CHitsPerSecondGraph(CXChartCtrl *pChart);
    virtual ~CHitsPerSecondGraph();

protected:
    BOOL AddTestRun(HttpArray& arHttp,URLMap* m_mapUrls,DWORD dwTestStartTime);
};


class CAvPageRTPerUserGraph : public CHttpGraph
{
public:
    CAvPageRTPerUserGraph(CXChartCtrl *pChart);
    virtual ~CAvPageRTPerUserGraph();

protected:
    BOOL AddTestRun(HttpArray& arHttpData,URLMap* m_mapUrls,DWORD dwTestStartTime);
};

class CErrorsPerUserGraph : public CHttpGraph
{
public:
    CErrorsPerUserGraph(CXChartCtrl *pChart);
    virtual ~CErrorsPerUserGraph();

protected:
    BOOL AddTestRun(HttpArray& arHttp,URLMap* m_mapUrls,DWORD dwTestStartTime);
};


class CErrorsTimeGraph : public CHttpGraph
{
public:
    CErrorsTimeGraph(CXChartCtrl *pChart);
    virtual ~CErrorsTimeGraph();

protected:
    BOOL AddTestRun(HttpArray& arHttp,URLMap* m_mapUrls,DWORD dwTestStartTime);;
};


class CPagesPerSecondGraph : public CHttpGraph
{
public:
    CPagesPerSecondGraph(CXChartCtrl *pChart);
    virtual ~CPagesPerSecondGraph();
protected:
    BOOL AddTestRun(HttpArray& arHttp,URLMap* m_mapUrls,DWORD dwTestStartTime);
};

class CVUsersTimeGraph : public CGraphTypeBase
{
public:
	CVUsersTimeGraph(CXChartCtrl *pChart);
	virtual ~CVUsersTimeGraph();

//	virtual void GetMenuItems(MenuItemArray& arMenuItems);
  //  virtual void OnCommand(UINT uiCommandID,CWnd *pParent);

    BOOL AddTestRun(VUList& VUs,DWORD dwTestStartTime);
};

#endif // !defined(AFX_GRAPHTYPES_H__24021AB3_F437_11D3_BD83_00C04F89E392__INCLUDED_)

