/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


#include "stdafx.h"
#include "VarDBTableList.h"

/////////////////////////////////////////////////////////////////////////////
// CVarDBTableList implementation
//---------------------------------------------------------------------------

CVarDBTableList::CVarDBTableList(CDatabase* pDatabase)
        : CDBTable(pDatabase)
{}

//---------------------------------------------------------------------------


CVarDBTableList::~CVarDBTableList()
{
    Reset();
}

//---------------------------------------------------------------------------

void CVarDBTableList::Reset()
{
    Free();

    for (int i = 0; i < m_aData.GetSize(); i++)
        delete m_aData.GetAt(i);

    m_aData.RemoveAll();
    m_astrOwner.RemoveAll();

}

//---------------------------------------------------------------------------

BOOL CVarDBTableList::Load()
{
    CWaitCursor crwait;

    RETCODE                     ReturnCode ;
    UDWORD                      RowsFetched ;
    UWORD*                      pRowStatus ;
    pFF_DB_ELEMENT_SET_FIELDS   pHostData ;

    ASSERT(m_nRowSetSize > 0) ;

    pRowStatus = new UWORD[m_nRowSetSize] ;
    ASSERT(pRowStatus);

    pHostData = new FF_DB_ELEMENT_SET_FIELDS[m_nRowSetSize] ;
    ASSERT(pHostData) ;

    OpenS() ;
    Reset() ; // Clear everything

    // Prepare the statement for bulk operation
    SQLSetStmtOption(m_hstmtS,SQL_BIND_TYPE,sizeof(FF_DB_ELEMENT_SET_FIELDS));
    SQLSetStmtOption(m_hstmtS,SQL_CONCURRENCY,SQL_CONCUR_READ_ONLY);
    SQLSetStmtOption(m_hstmtS,SQL_ROWSET_SIZE,m_nRowSetSize);

    ReturnCode = ::SQLTables(m_hstmtS,
                             (UCHAR FAR *)NULL, SQL_NTS,
                             (UCHAR FAR *)NULL, SQL_NTS,
                             (UCHAR FAR *)NULL, SQL_NTS,
                             (UCHAR FAR *)"'TABLE'", SQL_NTS);



    SQLBindCol(m_hstmtS,1,SQL_C_CHAR,pHostData[0].m_strQualifier,DB_ELEMENT_CHAR_SIZE,&pHostData[0].m_strQualifierInd);
    SQLBindCol(m_hstmtS,2,SQL_C_CHAR,pHostData[0].m_strOwner    ,DB_ELEMENT_CHAR_SIZE,&pHostData[0].m_strOwnerInd);
    SQLBindCol(m_hstmtS,3,SQL_C_CHAR,pHostData[0].m_strName     ,DB_ELEMENT_CHAR_SIZE,&pHostData[0].m_strNameInd);
    SQLBindCol(m_hstmtS,4,SQL_C_CHAR,pHostData[0].m_strType     ,DB_ELEMENT_CHAR_SIZE,&pHostData[0].m_strTypeInd);
    SQLBindCol(m_hstmtS,5,SQL_C_CHAR,pHostData[0].m_strRemarks  ,DB_ELEMENT_CHAR_SIZE,&pHostData[0].m_strRemarksInd);


    // Fetch and store...
    for (;;)
    {
        ReturnCode = SQLExtendedFetch(m_hstmtS,SQL_FETCH_NEXT,1,&RowsFetched,pRowStatus);
        if (ReturnCode == SQL_ERROR)
        {
            //$$$DisplayDBError( m_pDatabase , m_hstmtS) ;
            return FALSE;
        }
        else if ( ReturnCode == SQL_NO_DATA_FOUND )
            break;

        for (UINT Count = 0; Count < RowsFetched; Count++)
        {
            if  (pRowStatus[Count] != SQL_ROW_DELETED && pRowStatus[Count] != SQL_ROW_ERROR )
            {
                pDB_ELEMENT_SET_FIELDS  pData = new DB_ELEMENT_SET_FIELDS();
                ASSERT(pData);

                pData->m_strQualifier         = GetNullValue( pHostData[Count].m_strQualifierInd, pHostData[Count].m_strQualifier    );
                pData->m_strOwner             = GetNullValue( pHostData[Count].m_strOwnerInd    , pHostData[Count].m_strOwner        );
                pData->m_strName              = GetNullValue( pHostData[Count].m_strNameInd     , pHostData[Count].m_strName         );
                pData->m_strType              = GetNullValue( pHostData[Count].m_strTypeInd     , pHostData[Count].m_strType         );
                pData->m_strRemarks           = GetNullValue( pHostData[Count].m_strRemarksInd  , pHostData[Count].m_strRemarks      );

                m_aData.Add(pData);

                //Owner list creation
                BOOL bFound = FALSE;
                int nb = m_astrOwner.GetSize();
                for(int i = nb - 1 ; i >= 0 && !bFound; i-- )
                {
                    if( m_astrOwner.GetAt(i) == pData->m_strOwner ) bFound = TRUE;
                }

                if( !bFound )
                {
                    m_astrOwner.Add( pData->m_strOwner );
                }
            }
        }

        if  (RowsFetched < (UINT)m_nRowSetSize)
            break;
    }

    delete [] pRowStatus;
    pRowStatus = NULL;
    delete [] pHostData;
    pHostData = NULL;

    Free();
    DropHstmt(&m_hstmtS);

    return (m_aData.GetSize() > 0) ;
}

//---------------------------------------------------------------------------
