/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// VarValListDBPPage.cpp: implementation of the CVarValListDBPPage class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "modeller.h"
#include "VarValListDBPPage.h"
#include "Constantes.h"
#include "ConstantesLib.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
//---------------------------------------------------------------------------

CVarValListDBPPage::CVarValListDBPPage(CVarDBParam* _pVarDBParam,CVarInfo * _pVarInfo)
        :   CVarValListPPage(_pVarInfo),
		m_pVarDBParam(_pVarDBParam)
{
	
}

//---------------------------------------------------------------------------


CVarValListDBPPage::~CVarValListDBPPage()
{
}

//---------------------------------------------------------------------------


void CVarValListDBPPage::Values2Ctrl()
{
    CVarValueList RequestResult ;
    long lCount;


    if(!GetRequestResult(&RequestResult))
        return ;

    lCount = RequestResult.m_Values.GetSize() ;

    ResetGrid();
	int nNextItem = m_ListValue.GetItemCount();
    for(int i=0;i<lCount;i++)
    {
		m_ListValue.InsertItem(nNextItem,RequestResult.m_Values.GetAt(i));
		nNextItem++;
    }

}

//---------------------------------------------------------------------------

BOOL CVarValListDBPPage::GetRequestResult(CVarValueList * _pVarValueList)
{
    CRecordset rs(&(m_pVarDBParam->m_VarDBSession.m_Database)) ;
    CDBVariant varValue ;
    short nFields ;
    short nRow ;
    long  nRowSaved ;
    long  nMaxRowToDisplay ;
    CString strReqSource ;
    CString strValue ;
    BOOL  bMaxRowReached = FALSE ;
    BOOL  bRowLimited ;



    _pVarValueList->m_Values.RemoveAll() ;

    bRowLimited = ( m_pVarDBParam->m_lReqSelType == VAR_ROWSEL_SOME ) ;

    if( bRowLimited )
    {
        nMaxRowToDisplay = m_pVarDBParam->m_lMaxRowNumber ;
    }

    strReqSource = m_pVarDBParam->GetRequestSource() ;

    if(!rs.Open( CRecordset::forwardOnly, strReqSource ))
        return FALSE ;

    nFields = rs.GetODBCFieldCount();

    if(nFields <= 0)
        return FALSE ;

    nRow = rs.GetRecordCount() ;

    if(nRow <= 0)
        return TRUE ;

    nRowSaved = 0 ;

    while( !rs.IsEOF( ) && !bMaxRowReached )
    {
        rs.GetFieldValue( (short)0, varValue ) ;

        switch(varValue.m_dwType)
        {
        case DBVT_BOOL:
            strValue = (varValue.m_boolVal) ? "TRUE" : "FALSE" ;
            break ;

        case DBVT_UCHAR:
            strValue = varValue.m_chVal ;
            break ;

        case DBVT_SHORT:
            strValue = varValue.m_iVal ;
            break ;

        case DBVT_LONG:
            strValue = varValue.m_lVal ;
            break ;

        case DBVT_SINGLE:
            strValue = varValue.m_fltVal ;
            break ;

        case DBVT_DOUBLE:
            strValue = varValue.m_dblVal ;
            break ;

        case DBVT_DATE:
            strValue = varValue.m_pdate->year ;
            strValue += "/" + varValue.m_pdate->month ;
            strValue += "/" + varValue.m_pdate->day ;
            break ;

        case DBVT_STRING:
            strValue = *(varValue.m_pstring) ;
            break ;

        case DBVT_NULL:     /*No union member is valid for access. */
            strValue = "" ;
            break ;

        case DBVT_BINARY:
            strValue.LoadString(IDS_FIELD_UNSUPPORTED) ;
            break ;
        }

        _pVarValueList->m_Values.Add(strValue) ;
        nRowSaved ++ ;

        if( bRowLimited )
        {
            bMaxRowReached = ( nRowSaved == nMaxRowToDisplay ) ;
        }

        rs.MoveNext() ;
    }

    rs.Close() ;

    return TRUE ;
}

//---------------------------------------------------------------------------

LRESULT CVarValListDBPPage::OnWizardBack()
{
    return IDD_VAR_DBROWSEL_PPAGE ;
}

//---------------------------------------------------------------------------

LRESULT CVarValListDBPPage::OnWizardNext()
{
    Ctrl2Values() ;

    return CPropertyPage::OnWizardNext() ;
}
