/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc
///////////////////////////////////////////////////////////////////////////////

#ifndef _OtaExec_h
#define _OtaExec_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include "OtaQueryHandler.h"

///////////////////////////////////////////////////////////////////////////////
// Standard includes
#include <string>

#ifndef VMS
using std::string;
#endif

#ifdef WIN32
#include <windows.h>
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif


///////////////////////////////////////////////////////////////////////////////
// @class   OtaExec | This class allows the querying and retrieval of
//          statistics by issuing a command line.
// @base    public | OtaQueryHandler
class OtaExec : public virtual OtaQueryHandler
{
    // @access Public Members
public:
    OtaExec();                          // @cmember Constructor
    virtual ~OtaExec();                // @cmember Destructor

    bool RunSimpleQuery( const string query, long *result );       // @cmember Runs simple query
    bool RunDetailedQuery( const string query, string *result );    // @cmember Runs detailed query
    string GetDetailedHeaders( const string query );  // @cmember Gets detailed query headers

    // @access Private Members
private:
    bool ProcessExists();
    bool ExecCommand(const string query);
    void TriggerCmd();
    char* GetCmdOutput();
    void CmdTimeout();
    void KillCmd();

#ifdef WIN32
    string LastErrorText();
#endif

#ifdef WIN32
    HANDLE         m_hProcess;          // Command process handle
    HANDLE         m_hStdinWr;          // Handle to write to child's stdin
    HANDLE         m_hStdoutRd;         // Handle to read from child's stdout
    HANDLE         m_hStdoutWr;         // Handle to write to child's stdout
    HANDLE         m_hEvent;            // Synchronization event
#else
    pid_t          m_pid;               // Command process pid
    int            m_childstdin;        // Child's stdin
    int            m_childstdout;       // Child's stdout
#endif

};

#endif
