/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// NTPerfPluginView.cpp : implementation of the CNTPerfPluginView class
//

#include "stdafx.h"
#include "NTPerfPlugin.h"

#include "NTPerfPluginDoc.h"
#include "CntrItem.h"
#include "NTPerfPluginView.h"
#include "EditQueryDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CNTPerfPluginView

IMPLEMENT_DYNCREATE(CNTPerfPluginView, CFormView)

BEGIN_MESSAGE_MAP(CNTPerfPluginView, CFormView)
	//{{AFX_MSG_MAP(CNTPerfPluginView)
	ON_WM_DESTROY()
	ON_WM_SETFOCUS()
	ON_WM_SIZE()
	ON_COMMAND(ID_OLE_INSERT_NEW, OnInsertObject)
	ON_COMMAND(ID_CANCEL_EDIT_CNTR, OnCancelEditCntr)
	ON_COMMAND(ID_CANCEL_EDIT_SRVR, OnCancelEditSrvr)
	ON_BN_CLICKED(IDC_ADD_ROW, OnAddRow)
	ON_BN_CLICKED(IDC_REMOVE_ROW, OnRemoveRow)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
	ON_NOTIFY(NM_DBLCLK,IDC_GRID,OnDblClkQueryList)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CFormView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CFormView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CFormView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CNTPerfPluginView construction/destruction

CNTPerfPluginView::CNTPerfPluginView()
	: CFormView(CNTPerfPluginView::IDD)
{
	m_pSelection = NULL;

	m_bLoaded = false;
}

CNTPerfPluginView::~CNTPerfPluginView()
{
}

BOOL CNTPerfPluginView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CFormView::PreCreateWindow(cs);
}

void CNTPerfPluginView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CNTPerfPluginView)
	DDX_Control(pDX, IDC_GROUP, m_ctlGroup);
	//}}AFX_DATA_MAP
    DDX_Control(pDX, IDC_GRID, m_wndGrid);
}
/////////////////////////////////////////////////////////////////////////////
// CNTPerfPluginView drawing

void CNTPerfPluginView::OnDraw(CDC* pDC)
{
	CNTPerfPluginDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here
}

void CNTPerfPluginView::OnInitialUpdate()
{
	CFormView::OnInitialUpdate();


	// TODO: remove this code when final selection model code is written
	m_pSelection = NULL;    // initialize selection

	//Active documents should always be activated
	COleDocument* pDoc = (COleDocument*) GetDocument();
	if (pDoc != NULL)
	{
		// activate the first one
		POSITION posItem = pDoc->GetStartPosition();
		if (posItem != NULL)
		{
			CDocItem* pItem = pDoc->GetNextItem(posItem);

			// only if it's an Active document
			COleDocObjectItem *pDocObjectItem =
				DYNAMIC_DOWNCAST(COleDocObjectItem, pItem);

			if (pDocObjectItem != NULL)
			{
				pDocObjectItem->DoVerb(OLEIVERB_SHOW, this);
			}
		}
	}

	GetDocument()->SetModifiedFlag();	

	CString strPrfPath = GetDocument()->GetProfilePath();

	GetParentFrame()->RecalcLayout();
	ResizeParentToFit();

    const int nNumColumns = 4;
    CString headers = "Name\tQuery\tInterval\tDelta Value";
	CRect r;
	m_wndGrid.GetWindowRect(r);

    m_wndGrid.SetColumnCount(nNumColumns);
    m_wndGrid.SetRowCount(1);
    m_wndGrid.SetFixedRowCount();
	
	m_wndGrid.SetColumnWidth(0,r.Width()/4);
	m_wndGrid.SetColumnWidth(1,r.Width()/2);
	m_wndGrid.SetColumnWidth(2,r.Width()/8);
	m_wndGrid.SetColumnWidth(3,r.Width()/8);

    m_wndGrid.PasteTextFromBuffer(0,0, headers);

	if (!ReadProfile(strPrfPath))
	{
		//Wait a bit then show add query dlg
		SetTimer(1,500,NULL);
		m_bLoaded = true;
	}
}

/////////////////////////////////////////////////////////////////////////////
// CNTPerfPluginView printing

BOOL CNTPerfPluginView::OnPreparePrinting(CPrintInfo* pInfo)
{
	if (!CFormView::DoPreparePrinting(pInfo))
		return FALSE;
	
	if (!COleDocObjectItem::OnPreparePrinting(this, pInfo))
		return FALSE;

	return TRUE;
}

void CNTPerfPluginView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CNTPerfPluginView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

void CNTPerfPluginView::OnPrint(CDC* pDC, CPrintInfo* pInfo)
{
	// TODO: add customized printing code here
	if(pInfo->m_bDocObject)
		COleDocObjectItem::OnPrint(this, pInfo, TRUE);
	else
		CFormView::OnPrint(pDC, pInfo);
}

void CNTPerfPluginView::OnDestroy()
{
	// Deactivate the item on destruction; this is important
	// when a splitter view is being used.
   CView::OnDestroy();
   COleClientItem* pActiveItem = GetDocument()->GetInPlaceActiveItem(this);
   if (pActiveItem != NULL && pActiveItem->GetActiveView() == this)
   {
      pActiveItem->Deactivate();
      ASSERT(GetDocument()->GetInPlaceActiveItem(this) == NULL);
   }
}


/////////////////////////////////////////////////////////////////////////////
// OLE Client support and commands

BOOL CNTPerfPluginView::IsSelected(const CObject* pDocItem) const
{
	// The implementation below is adequate if your selection consists of
	//  only CNTPerfPluginCntrItem objects.  To handle different selection
	//  mechanisms, the implementation here should be replaced.

	// TODO: implement this function that tests for a selected OLE client item

	return pDocItem == m_pSelection;
}

void CNTPerfPluginView::OnInsertObject()
{
	// Invoke the standard Insert Object dialog box to obtain information
	//  for new CNTPerfPluginCntrItem object.
	COleInsertDialog dlg;
	if (dlg.DoModal(COleInsertDialog::DocObjectsOnly) != IDOK)
		return;

	BeginWaitCursor();

	CNTPerfPluginCntrItem* pItem = NULL;
	TRY
	{
		// Create new item connected to this document.
		CNTPerfPluginDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);
		pItem = new CNTPerfPluginCntrItem(pDoc);
		ASSERT_VALID(pItem);

		// Initialize the item from the dialog data.
		if (!dlg.CreateItem(pItem))
			AfxThrowMemoryException();  // any exception will do
		ASSERT_VALID(pItem);
		
		pItem->DoVerb(OLEIVERB_SHOW, this);

		ASSERT_VALID(pItem);

		// As an arbitrary user interface design, this sets the selection
		//  to the last item inserted.

		// TODO: reimplement selection as appropriate for your application

		m_pSelection = pItem;   // set selection to last inserted item
		pDoc->UpdateAllViews(NULL);
	}
	CATCH(CException, e)
	{
		if (pItem != NULL)
		{
			ASSERT_VALID(pItem);
			pItem->Delete();
		}
		AfxMessageBox(IDP_FAILED_TO_CREATE);
	}
	END_CATCH

	EndWaitCursor();
}

// The following command handler provides the standard keyboard
//  user interface to cancel an in-place editing session.  Here,
//  the container (not the server) causes the deactivation.
void CNTPerfPluginView::OnCancelEditCntr()
{
	// Close any in-place active item on this view.
	COleClientItem* pActiveItem = GetDocument()->GetInPlaceActiveItem(this);
	if (pActiveItem != NULL)
	{
		pActiveItem->Close();
	}
	ASSERT(GetDocument()->GetInPlaceActiveItem(this) == NULL);
}

// Special handling of OnSetFocus and OnSize are required for a container
//  when an object is being edited in-place.
void CNTPerfPluginView::OnSetFocus(CWnd* pOldWnd)
{
	COleClientItem* pActiveItem = GetDocument()->GetInPlaceActiveItem(this);
	if (pActiveItem != NULL &&
		pActiveItem->GetItemState() == COleClientItem::activeUIState)
	{
		// need to set focus to this item if it is in the same view
		CWnd* pWnd = pActiveItem->GetInPlaceWindow();
		if (pWnd != NULL)
		{
			pWnd->SetFocus();   // don't call the base class
			return;
		}
	}

	CView::OnSetFocus(pOldWnd);
}

void CNTPerfPluginView::OnSize(UINT nType, int cx, int cy)
{
	CView::OnSize(nType, cx, cy);
	COleClientItem* pActiveItem = GetDocument()->GetInPlaceActiveItem(this);
	if (pActiveItem != NULL)
		pActiveItem->SetItemRects();

	if (m_ctlGroup.m_hWnd != NULL)
	{
		m_ctlGroup.MoveWindow(5,5,cx-10,cy-10);

		m_wndGrid.MoveWindow(20,25,cx-40,cy-40);
	    m_wndGrid.ExpandColumnsToFit(); 
	}
}

/////////////////////////////////////////////////////////////////////////////
// OLE Server support

// The following command handler provides the standard keyboard
//  user interface to cancel an in-place editing session.  Here,
//  the server (not the container) causes the deactivation.
void CNTPerfPluginView::OnCancelEditSrvr()
{
	GetDocument()->OnDeactivateUI(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// CNTPerfPluginView diagnostics

#ifdef _DEBUG
void CNTPerfPluginView::AssertValid() const
{
	CFormView::AssertValid();
}

void CNTPerfPluginView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CNTPerfPluginDoc* CNTPerfPluginView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CNTPerfPluginDoc)));
	return (CNTPerfPluginDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CNTPerfPluginView message handlers

void CNTPerfPluginView::OnAddRow() 
{
	CEditQueryDlg dlg;
	for(int nRow=1;nRow<m_wndGrid.GetRowCount();nRow++)
		dlg.m_arstrNamesInUse.Add(	m_wndGrid.GetItemText(nRow,enDesc) );

	if(dlg.DoModal() != IDOK)
		return;

	nRow = m_wndGrid.InsertRow("");

	// Setup values
	m_wndGrid.SetCellEnabled(nRow,enDesc,FALSE);
	m_wndGrid.SetCellEnabled(nRow,enQuery,FALSE);
	m_wndGrid.SetCellEnabled(nRow,enInterval,FALSE);
	m_wndGrid.SetCellEnabled(nRow,enDelta,FALSE);

	m_wndGrid.SetItemText(nRow, enDesc, dlg.m_strDescription);
	m_wndGrid.SetItemText(nRow, enQuery, dlg.m_strQuery);
	m_wndGrid.SetItemText(nRow, enInterval, dlg.m_strInterval);
	m_wndGrid.SetItemText(nRow, enDelta, (dlg.m_bDelta ? "Yes" : "No"));

	m_wndGrid.Invalidate();	
}

void CNTPerfPluginView::OnRemoveRow() 
{
	CCellRange range = m_wndGrid.GetSelectedCellRange();
	if (range.GetRowSpan() != 1)
	{
		MessageBox("Please select a single row to remove.", "OpenSTA", MB_OK);
		return;
	}
	m_wndGrid.DeleteRow(range.GetMinRow());
	m_wndGrid.Invalidate();	
}

BOOL CNTPerfPluginView::ReadProfile(CString strFilePath) 
{
	// Check if file exists
	CFileStatus fs;
	if (!CFile::GetStatus(strFilePath, fs))
		return FALSE;

	int nAdded = 0;
	try
	{
		CStdioFile f(strFilePath, CFile::modeRead | CFile::typeText);

		CString strLine ="";
		f.ReadString(strLine);

		for (; f.ReadString(strLine); )
		{
			if (strLine == "SimpleQuery")
			{
				// ignore enabled line
				f.ReadString(strLine);
				// ignore type line
				f.ReadString(strLine);

				CString strDelta = "";
				f.ReadString(strDelta);
				BOOL bDelta = atoi(strDelta.Right(1));

				CString strInterval = "";
				f.ReadString(strInterval);
				int nPos = strInterval.Find(" ", 0);
				int nLen = strInterval.GetLength();
				strInterval = strInterval.Right(nLen-nPos-1);

				CString strHeading = "";
				f.ReadString(strHeading);
				nPos = strHeading.Find(" ", 0);
				nLen = strHeading.GetLength();
				strHeading = strHeading.Right(nLen-nPos-1);

				// ignore text line
				f.ReadString(strLine);

				CString strQuery = "";
				f.ReadString(strQuery);
				strQuery.Remove('\"');

				m_wndGrid.InsertRow("");

				nAdded++;
				int nRow = m_wndGrid.GetRowCount()-1;
				m_wndGrid.SetCellEnabled(nRow,enDesc,FALSE);
				m_wndGrid.SetCellEnabled(nRow,enQuery,FALSE);
				m_wndGrid.SetCellEnabled(nRow,enInterval,FALSE);
				m_wndGrid.SetCellEnabled(nRow,enDelta,FALSE);
				m_wndGrid.SetItemText(nRow, enDesc, strHeading);
				m_wndGrid.SetItemText(nRow, enQuery, strQuery);
				m_wndGrid.SetItemText(nRow, enInterval, strInterval);
				m_wndGrid.SetItemText(nRow, enDelta, (bDelta ? "Yes" : "No"));
			}
		}

		f.Close();
		m_bLoaded = true;
	}
	catch(...)
	{
		return FALSE;
	}

	if(nAdded == 0)
		return FALSE;

	return TRUE;
}

BOOL CNTPerfPluginView::WriteProfile(CString strFilePath) 
{
	if (!m_bLoaded)
		return true;

	try
	{
		CStdioFile f(strFilePath, CFile::modeCreate | CFile::modeWrite | CFile::typeText);
		f.WriteString("tof.6\n");
		f.WriteString("date\n");
		f.WriteString("lastedit\n");
		f.WriteString("name\n");
		f.WriteString("description\n");
		f.WriteString("\n");

		for (int i =1; i<m_wndGrid.GetRowCount(); i++)
		{
			CString szDesc = m_wndGrid.GetItemText(i,enDesc);
			CString szQuery = m_wndGrid.GetItemText(i,enQuery);
			CString szInterval = m_wndGrid.GetItemText(i,enInterval);
			BOOL bDelta = (m_wndGrid.GetItemText(i, enDelta) == "Yes");
			CString szDelta = "";

			if (bDelta)
				szDelta="1";
			else
				szDelta="0";

			if (!szQuery.IsEmpty())
			{
				f.WriteString("SimpleQuery\n");
				f.WriteString("enabled 1\n");
				f.WriteString("type 2\n");
				f.WriteString("delta "+szDelta+"\n");
				f.WriteString("frequency "+szInterval+"\n");
				f.WriteString("heading "+szDesc+"\n");
				f.WriteString("text \"\n");
				f.WriteString(szQuery+"\"\n");
				f.WriteString("\n");
			}
		}

		f.Close();
		return true;
	}
	catch(...)
	{
		MessageBox("Unable to write profile to disk!", "OpenSTA", MB_ICONEXCLAMATION);
	}

	return false;
}

void CNTPerfPluginView::OnDblClkQueryList(NMHDR* pNMHDR,LRESULT *pResult)
{
	NM_GRIDVIEW *pGridView = (NM_GRIDVIEW*)pNMHDR;
	CEditQueryDlg dlg;

	for(int nRow=1;nRow<m_wndGrid.GetRowCount();nRow++)
	{
		if(nRow != pGridView->iRow)
			dlg.m_arstrNamesInUse.Add(	m_wndGrid.GetItemText(nRow,enDesc) );
	}

	dlg.m_strDescription = m_wndGrid.GetItemText(pGridView->iRow,enDesc);
	dlg.m_strQuery = m_wndGrid.GetItemText(pGridView->iRow,enQuery);
	dlg.m_strInterval = m_wndGrid.GetItemText(pGridView->iRow,enInterval);
	dlg.m_bDelta = (m_wndGrid.GetItemText(pGridView->iRow, enDelta) == "Yes");

	if(dlg.DoModal()==IDOK)
	{
		m_wndGrid.SetItemText(pGridView->iRow, enDesc, dlg.m_strDescription);
		m_wndGrid.SetItemText(pGridView->iRow, enQuery, dlg.m_strQuery);
		m_wndGrid.SetItemText(pGridView->iRow, enInterval, dlg.m_strInterval);
		m_wndGrid.SetItemText(pGridView->iRow, enDelta, (dlg.m_bDelta ? "Yes" : "No"));
		m_wndGrid.Invalidate();	
	}

	*pResult=0;
}


void CNTPerfPluginView::OnTimer(UINT nIDEvent) 
{
	if(nIDEvent==1)
	{
		KillTimer(1);
		OnAddRow();
	}
		
	CFormView::OnTimer(nIDEvent);
}
