/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     OtaStandalone
// @module  OtaStandalone |
//
///////////////////////////////////////////////////////////////////////////////

#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif
#include <stdio.h>
#include <string>

#ifndef VMS
using std::string;
#endif

#include "TPLog.h"
#include "OtaDefs.h"
#include <psl_macros.h>
#include <target_rtndef.h>
#include "OtaStandalone.h"
#include "OtaQuery.h"
#include "OtaExec.h"
#include <CECSimpleParse.h>
#include "Smp.h"

#define MAXROWS 100

///////////////////////////////////////////////////////////////////////////////
//
// Run a single query in standalone mode
//
void StandaloneQuery(string queryin, string target)
{
    OtaTarget* pTarget = NULL;
    OtaQueryHandler* pHandler = NULL;

    // Query is specified as "detail/type/query". (detail = 1 if detail query else 0)
    // query may contain \n to indicate newlines.
    // Split up into components
    CECSimpleParse token( queryin);
    string entry = token.getNext('/');
    bool detail = (entry == "1");
    entry = token.getNext('/');
    int queryType = atoi(entry.c_str());
    string query = token.getToEnd();
    if (query[0] == '/')
        query.erase(0,1);
    //replace all \\\\n in string with real \n 's
    string::size_type pos=0;
    pos=query.find("\\\\n");
    while(pos!=string::npos)
    {
        query.replace(pos,pos+3,"\n");
        pos=query.find("\\\\n");
    }
    switch(queryType)
    {
    case QUERY_EXEC:
        pHandler = new OtaExec();
        break;
    case QUERY_SNMP:
        pHandler = new OtaSnmp();
        break;
    default:
        OtaAddTrace(CYR_OTA,ERRORLEVEL,"Invalid query type");
        return;
    }

    long simRes = 0;
    string detRes = "";
    string msg;
    if (detail)
    {
        if (pHandler->RunDetailedQuery( query.data(), &detRes ))
        {
            msg = "Detail query values = \n0\tTest Query\n1\t"
                  + pHandler->GetDetailedHeaders(query.data())
                  + string("\n") + detRes;
            OtaAddTrace(CYR_OTA,TRACE_RESULTS,msg);
        }
    }
    else
    {
        if (pHandler->RunSimpleQuery( query.data(), &simRes ))
        {
            char cval[32];
            sprintf(cval,"%d",simRes);
            msg = "Simple query value = " + string(cval);
            OtaAddTrace(CYR_OTA,TRACE_RESULTS,msg);
        }
    }

    if (pTarget != NULL)
        delete pTarget;
    delete pHandler;
}
