/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// SNMPPluginView.cpp : implementation of the CSNMPPluginView class
//

#include "stdafx.h"
#include "SNMPPlugin.h"

#include "SNMPPluginDoc.h"
#include "SNMPPluginView.h"
#include "SNMPScanDlg.h"
#include "QueryEditDlg.h"
#include "EditQueryDlg.h"
#include "CategoryEdit.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSNMPPluginView

IMPLEMENT_DYNCREATE(CSNMPPluginView, CFormView)

BEGIN_MESSAGE_MAP(CSNMPPluginView, CFormView)
	//{{AFX_MSG_MAP(CSNMPPluginView)
	ON_COMMAND(ID_CANCEL_EDIT_SRVR, OnCancelEditSrvr)
	ON_BN_CLICKED(IDC_ADD_ROW, OnAddRow)
	ON_BN_CLICKED(IDC_REMOVE_ROW, OnRemoveRow)
	ON_BN_CLICKED(IDC_SCAN, OnScan)
	ON_WM_SIZE()
	ON_WM_TIMER()
	ON_COMMAND(ID_EDIT_CATEGORY, OnEditCategory)
	//}}AFX_MSG_MAP
	ON_NOTIFY(NM_DBLCLK,IDC_GRID,OnDblClkGrid)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CFormView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CFormView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CFormView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSNMPPluginView construction/destruction

CSNMPPluginView::CSNMPPluginView()
	: CFormView(CSNMPPluginView::IDD)
{
	//{{AFX_DATA_INIT(CSNMPPluginView)
	//}}AFX_DATA_INIT
	m_bLoaded = false;
}

CSNMPPluginView::~CSNMPPluginView()
{
}

void CSNMPPluginView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSNMPPluginView)
	DDX_Control(pDX, IDC_GROUP, m_ctlGroup);
	//}}AFX_DATA_MAP
    DDX_Control(pDX, IDC_GRID, m_wndGrid);
}

BOOL CSNMPPluginView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CFormView::PreCreateWindow(cs);
}

void CSNMPPluginView::OnInitialUpdate()
{
	CFormView::OnInitialUpdate();
	GetParentFrame()->RecalcLayout();
	ResizeParentToFit();

	GetDocument()->SetModifiedFlag();	

	CString strTofPath = GetDocument()->GetProfilePath();

	GetParentFrame()->RecalcLayout();
	ResizeParentToFit();
	
    const int nNumColumns = 5;
    CString headers = "Name\tSNMP Server\tPort\tQuery\tInterval\tDelta Value";
	CRect r;
	m_wndGrid.GetWindowRect(r);

    m_wndGrid.SetColumnCount(nNumColumns);
    m_wndGrid.SetRowCount(1);
    m_wndGrid.SetFixedRowCount();
	
	m_wndGrid.SetColumnWidth(0,r.Width()/5);
	m_wndGrid.SetColumnWidth(1,r.Width()/5);
	m_wndGrid.SetColumnWidth(2,r.Width()/5);
	m_wndGrid.SetColumnWidth(3,r.Width()*2/5);
	m_wndGrid.SetColumnWidth(4,r.Width()/10);
	m_wndGrid.SetColumnWidth(5,r.Width()/10);

    m_wndGrid.PasteTextFromBuffer(0,0, headers);

	if (!ReadProfile(strTofPath))
	{
		SetTimer(1,500,NULL);
		m_bLoaded = true;
	}
}

/////////////////////////////////////////////////////////////////////////////
// CSNMPPluginView printing

BOOL CSNMPPluginView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CSNMPPluginView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CSNMPPluginView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

void CSNMPPluginView::OnPrint(CDC* pDC, CPrintInfo* /*pInfo*/)
{
	// TODO: add customized printing code here
}

/////////////////////////////////////////////////////////////////////////////
// OLE Server support

// The following command handler provides the standard keyboard
//  user interface to cancel an in-place editing session.  Here,
//  the server (not the container) causes the deactivation.
void CSNMPPluginView::OnCancelEditSrvr()
{
	GetDocument()->OnDeactivateUI(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// CSNMPPluginView diagnostics

#ifdef _DEBUG
void CSNMPPluginView::AssertValid() const
{
	CFormView::AssertValid();
}

void CSNMPPluginView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CSNMPPluginDoc* CSNMPPluginView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CSNMPPluginDoc)));
	return (CSNMPPluginDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSNMPPluginView message handlers
void CSNMPPluginView::OnAddRow() 
{
	CEditQueryDlg dlg;
	dlg.m_arstrServers.Copy(m_arrServers);
	dlg.m_strName = "Name";
	dlg.m_strServer = "Address";
	dlg.m_strQuery = "";
	dlg.m_strInterval = "5";
	dlg.m_strPort = "161";
	dlg.m_bDelta = FALSE;

	for(int nRow=1;nRow<m_wndGrid.GetRowCount();nRow++)
		dlg.m_arstrNamesInUse.Add(	m_wndGrid.GetItemText(nRow,enDesc) );

	if(dlg.DoModal() != IDOK)
		return;

	nRow = m_wndGrid.InsertRow("");

	m_wndGrid.SetCellEnabled(nRow,enDesc,FALSE);
	m_wndGrid.SetCellEnabled(nRow,enServer,FALSE);
	m_wndGrid.SetCellEnabled(nRow,enQuery,FALSE);
	m_wndGrid.SetCellEnabled(nRow,enInterval,FALSE);
	m_wndGrid.SetCellEnabled(nRow,enPort,FALSE);
	m_wndGrid.SetCellEnabled(nRow,enDelta,FALSE);

	// Setup Default values
	m_wndGrid.SetItemText(nRow, enDesc, dlg.m_strName);
	m_wndGrid.SetItemText(nRow, enServer, dlg.m_strServer);
	m_wndGrid.SetItemText(nRow, enQuery, dlg.m_strQuery);
	m_wndGrid.SetItemText(nRow, enInterval, dlg.m_strInterval);
	m_wndGrid.SetItemText(nRow, enPort, dlg.m_strPort);
	m_wndGrid.SetItemText(nRow, enDelta, dlg.m_bDelta ? "Yes" : "No");

	m_wndGrid.Invalidate();		
}

void CSNMPPluginView::OnRemoveRow() 
{
	CCellRange range = m_wndGrid.GetSelectedCellRange();
	if (range.GetRowSpan() != 1)
	{
		MessageBox("Please select a single row to remove.", "OpenSTA", MB_OK);
		return;
	}
	m_wndGrid.DeleteRow(range.GetMinRow());
	m_wndGrid.Invalidate();	
}

void CSNMPPluginView::OnScan() 
{
	CSNMPScanDlg dlg;
	if(dlg.DoModal())
	{
		dlg.GetServers(m_arrServers);
	}
}

BOOL CSNMPPluginView::ReadProfile(CString strFilePath) 
{
	// Check if file exists
	CFileStatus fs;
	if (!CFile::GetStatus(strFilePath, fs))
		return FALSE;

	int nAdded = 0;
	try
	{
		CStdioFile f(strFilePath, CFile::modeRead | CFile::typeText);

		CString strLine ="";
		f.ReadString(strLine);
		for (; f.ReadString(strLine); )
		{
			if (strLine == "SimpleQuery")
			{
				// ignore enabled line
				f.ReadString(strLine);
				// ignore type line
				f.ReadString(strLine);

				CString strDelta = "";
				f.ReadString(strDelta);
				BOOL bDelta = atoi(strDelta.Right(1));

				CString strInterval = "";
				f.ReadString(strInterval);
				int nPos = strInterval.Find(" ", 0);
				int nLen = strInterval.GetLength();
				strInterval = strInterval.Right(nLen-nPos-1);

				CString strHeading = "";
				f.ReadString(strHeading);
				nPos = strHeading.Find(" ", 0);
				nLen = strHeading.GetLength();
				strHeading = strHeading.Right(nLen-nPos-1);

				// ignore text line
				f.ReadString(strLine);

				CString strQuery = "";
				f.ReadString(strQuery);
				strQuery.Remove('\"');
				nPos = strQuery.Find(" ", 0);
				nLen = strQuery.GetLength();

				CString strTemp = strQuery.Left(nPos);
				strQuery = strQuery.Right(nLen-nPos-1);
				nPos = strTemp.Find(':');
				nLen = strTemp.GetLength();
				CString strServer = strTemp.Left(nPos);
				CString strPort = strTemp.Right(nLen-nPos-1);

				m_wndGrid.InsertRow("");
				nAdded++;
				int nRow = m_wndGrid.GetRowCount()-1;
				m_wndGrid.SetCellEnabled(nRow,enDesc,FALSE);
				m_wndGrid.SetCellEnabled(nRow,enServer,FALSE);
				m_wndGrid.SetCellEnabled(nRow,enQuery,FALSE);
				m_wndGrid.SetCellEnabled(nRow,enInterval,FALSE);
				m_wndGrid.SetCellEnabled(nRow,enPort,FALSE);
				m_wndGrid.SetCellEnabled(nRow,enDelta,FALSE);
				m_wndGrid.SetItemText(nRow, enDesc, strHeading);
				m_wndGrid.SetItemText(nRow, enServer, strServer);
				m_wndGrid.SetItemText(nRow, enQuery, strQuery);
				m_wndGrid.SetItemText(nRow, enInterval, strInterval);
				m_wndGrid.SetItemText(nRow, enPort, strPort);

				m_wndGrid.SetItemText(nRow, enDelta, (bDelta ? "Yes" : "No"));
			}
		}

		f.Close();
		m_bLoaded = true;
	}
	catch(...)
	{
		return FALSE;
	}

	if (nAdded == 0)
		return FALSE;

	return TRUE;
}

BOOL CSNMPPluginView::WriteProfile(CString strFilePath) 
{
	if (!m_bLoaded)
		return true;

	try
	{
		CStdioFile f(strFilePath, CFile::modeCreate | CFile::modeWrite | CFile::typeText);
		f.WriteString("tof.6\n");
		f.WriteString("date\n");
		f.WriteString("lastedit\n");
		f.WriteString("name\n");
		f.WriteString("description\n");
		f.WriteString("\n");

		for (int i =1; i<m_wndGrid.GetRowCount(); i++)
		{
			CString szDesc = m_wndGrid.GetItemText(i,enDesc);
			CString szServer = m_wndGrid.GetItemText(i,enServer);
			CString szQuery = m_wndGrid.GetItemText(i,enQuery);
			CString szInterval = m_wndGrid.GetItemText(i,enInterval);
			CString szPort = m_wndGrid.GetItemText(i,enPort);
			BOOL bDelta = (m_wndGrid.GetItemText(i,enDelta) == "Yes");
			CString szDelta = "";

			if (bDelta)
				szDelta="1";
			else
				szDelta="0";

			if (!szServer.IsEmpty() && !szQuery.IsEmpty())
			{
				f.WriteString("SimpleQuery\n");
				f.WriteString("enabled 1\n");
				f.WriteString("type 3\n");
				f.WriteString("delta "+szDelta+"\n");
				f.WriteString("frequency "+szInterval+"\n");
				f.WriteString("heading "+szDesc+"\n");
				f.WriteString("text \"\n");
				f.WriteString(szServer+":"+szPort+" "+szQuery+"\"\n");
				f.WriteString("\n");
			}
		}

		f.Close();
		return true;
	}
	catch(...)
	{
		MessageBox("Unable to write profile to disk!", "OpenSTA", MB_ICONEXCLAMATION);
	}

	return false;
}

void CSNMPPluginView::OnSize(UINT nType, int cx, int cy) 
{
	CFormView::OnSize(nType, cx, cy);
	
	if (m_ctlGroup.m_hWnd != NULL)
	{
		m_ctlGroup.MoveWindow(5,5,cx-10,cy-10);
		m_wndGrid.MoveWindow(20,25,cx-40,cy-40);
	    m_wndGrid.ExpandColumnsToFit(); 
	}	
}

void CSNMPPluginView::OnDblClkGrid(NMHDR* pNMHDR,LRESULT * pResult)
{
	NM_GRIDVIEW *pGridView = (NM_GRIDVIEW*)pNMHDR;
	CEditQueryDlg dlg;

	dlg.m_arstrServers.Copy(m_arrServers);

	for(int nRow=1;nRow<m_wndGrid.GetRowCount();nRow++)
	{
		if(nRow != pGridView->iRow)
			dlg.m_arstrNamesInUse.Add(	m_wndGrid.GetItemText(nRow,enDesc) );
	}

	dlg.m_strName = m_wndGrid.GetItemText(pGridView->iRow,enDesc);
	dlg.m_strServer = m_wndGrid.GetItemText(pGridView->iRow, enServer);
	dlg.m_strQuery = m_wndGrid.GetItemText(pGridView->iRow,enQuery);
	dlg.m_strInterval = m_wndGrid.GetItemText(pGridView->iRow,enInterval);
	dlg.m_strPort = m_wndGrid.GetItemText(pGridView->iRow,enPort);
	dlg.m_bDelta = (m_wndGrid.GetItemText(pGridView->iRow, enDelta) == "Yes");

	if(dlg.DoModal()==IDOK)
	{
		m_wndGrid.SetItemText(pGridView->iRow, enDesc, dlg.m_strName);
		m_wndGrid.SetItemText(pGridView->iRow, enServer, dlg.m_strServer);
		m_wndGrid.SetItemText(pGridView->iRow, enQuery, dlg.m_strQuery);
		m_wndGrid.SetItemText(pGridView->iRow, enInterval, dlg.m_strInterval);
		m_wndGrid.SetItemText(pGridView->iRow,enPort,dlg.m_strPort);
		m_wndGrid.SetItemText(pGridView->iRow, enDelta, (dlg.m_bDelta ? "Yes" : "No"));
		m_wndGrid.Invalidate();	
	}

	*pResult = 0;
}

void CSNMPPluginView::OnTimer(UINT nIDEvent) 
{
	if(nIDEvent==1)
	{
		KillTimer(1);
		OnAddRow();
	}
		
	CFormView::OnTimer(nIDEvent);
}


void CSNMPPluginView::OnEditCategory() 
{
	CCategoryEdit dlg(this,0);
	dlg.DoModal();
}
