// SNMPGraphs.cpp: implementation of the CSNMPGraphs class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SNMPGraphs.h"
#include "GraphTypes.h"
#include "GraphWnd.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSNMPGraphs::CSNMPGraphs():
	m_strCustomSNMP("Custom SNMP")
{
}

CSNMPGraphs::~CSNMPGraphs()
{
	POSITION pos = m_listSNMPData.GetHeadPosition();
	while(pos)
		delete m_listSNMPData.GetNext(pos);
	m_listSNMPData.RemoveAll();
}

void CSNMPGraphs::LoadRun(const CString& strRunFolder)
{
	m_strLoadedRun = strRunFolder;
	m_strLoadedRun.TrimRight("\\");
	LoadStats();

	int nSlashPos = m_strLoadedRun.ReverseFind('\\');
	CString strRunName(m_strLoadedRun.Mid(nSlashPos+1));
	
	int nDay,nMonth,nYear,nHour,nMinute,nSecond;
	char *pszToken = strtok(strRunName.GetBuffer(0),"-");
	if(pszToken)
		nDay = atoi(pszToken);

	if(pszToken = strtok(NULL,"-"))
		nMonth = atoi(pszToken);

	if(pszToken = strtok(NULL," "))
		nYear = atoi(pszToken);

	if(pszToken = strtok(NULL,"-"))
		nHour = atoi(pszToken);

	if(pszToken = strtok(NULL,"-"))
		nMinute = atoi(pszToken);

	if(pszToken = strtok(NULL,"."))
		nSecond = atoi(pszToken);

	CTime T(nYear,nMonth,nDay,nHour,nMinute,nSecond);
	m_dwTestStartTime = T.GetTime();
}

void CSNMPGraphs::GetAvailableGraphs(CStringArray& arstrGraphs)
{
	arstrGraphs.RemoveAll();

	if(m_listSNMPData.GetCount())
	{
		arstrGraphs.Add(m_strCustomSNMP);
	}
}

CWnd* CSNMPGraphs::DisplayGraph(const CString& strName,CWnd *pParent,const CRect& rcBounds)
{
	CGraphWnd *pGraphWnd = new CGraphWnd;
	pGraphWnd->Create(NULL,strName,WS_CHILD|WS_VISIBLE|WS_BORDER,rcBounds,pParent,0);
	
	CSNMPGraph *pGraph = NULL;
	if(strName == m_strCustomSNMP)
		pGraph = new CSNMPGraph(pGraphWnd->m_pChart);
	
	pGraphWnd->m_pGraph = pGraph;

	if(!pGraph || !pGraph->AddTestRun(&m_listSNMPData,m_dwTestStartTime))
	{
		delete pGraphWnd;
		return NULL;
	}
	pGraphWnd->SetGraphStyles();
		
	return pGraphWnd;
}

void CSNMPGraphs::LoadStats()
{
	POSITION pos = m_listSNMPData.GetHeadPosition();
	while(pos)
		delete m_listSNMPData.GetNext(pos);
	m_listSNMPData.RemoveAll();

	CString strHeaderPath(m_strLoadedRun + "\\SNMPHeader.txt");
	CString strStatsPath(m_strLoadedRun + "\\SNMPStat.txt");
    
    try
    {
        CStdioFile SNMPFile(strHeaderPath,CFile::modeRead);

		CString strHeader;
		while(SNMPFile.ReadString(strHeader))
		{
			SNMPHeader *pHeader = new SNMPHeader;
			char *pszToken = strtok(strHeader.GetBuffer(0),"_");
			if(!pszToken)
				continue;
			((LPBYTE)(&pHeader->dwIP))[0] = atoi(pszToken);
			
			if(!(pszToken = strtok(NULL,"_"))) continue;
			((LPBYTE)(&pHeader->dwIP))[1] = atoi(pszToken);
			
			if(!(pszToken = strtok(NULL,"_"))) continue;
			((LPBYTE)(&pHeader->dwIP))[2] = atoi(pszToken);
			
			if(!(pszToken = strtok(NULL,","))) continue;
			((LPBYTE)(&pHeader->dwIP))[3] = atoi(pszToken);
			
			if(!(pszToken = strtok(NULL,",")))
				continue;
			pHeader->nIndex = atoi(pszToken);
			
			if(!(pszToken = strtok(NULL,",")))
				continue;
			pHeader->strQuery = pszToken;

			m_listSNMPData.AddTail(pHeader);
		}

		SNMPFile.Close();

        SNMPFile.Open(strStatsPath,CFile::modeRead);

		CString strStat;
		while(SNMPFile.ReadString(strHeader))
		{
			SNMPStat stat;
			char *pszToken = strtok(strHeader.GetBuffer(0),"_");
			if(!pszToken)
				continue;
			((LPBYTE)(&stat.dwIP))[0] = atoi(pszToken);
			
			if(!(pszToken = strtok(NULL,"_"))) continue;
			((LPBYTE)(&stat.dwIP))[1] = atoi(pszToken);
			
			if(!(pszToken = strtok(NULL,"_"))) continue;
			((LPBYTE)(&stat.dwIP))[2] = atoi(pszToken);
			
			if(!(pszToken = strtok(NULL,","))) continue;
			((LPBYTE)(&stat.dwIP))[3] = atoi(pszToken);
			
			if(!(pszToken = strtok(NULL,",")))
				continue;
			stat.nIndex = atoi(pszToken);

			if(!(pszToken = strtok(NULL,",")))
				continue;
			stat.dwStartTime = atoi(pszToken);

			if(!(pszToken = strtok(NULL,",")))
				continue;
			stat.dwEndTime = atoi(pszToken); 

			if(!(pszToken = strtok(NULL,",")))
				continue;
			stat.dValue = atoi(pszToken);

			//Find header for this stat
			POSITION pos = m_listSNMPData.GetHeadPosition();
			while(pos)
			{
				SNMPHeader *pHeader = m_listSNMPData.GetNext(pos);
				if((pHeader->dwIP == stat.dwIP) && (pHeader->nIndex == stat.nIndex))
				{
					pHeader->listStats.AddTail(stat);
					pos = NULL;
				}
			}
		}
	}
	catch(CFileException *e)
	{
		e->Delete();
	}
	catch(...)
	{
	}
}

